/***************************************************************************
 $RCSfile: aqmauth.cpp,v $
                             -------------------
 cvs         : $Id: aqmauth.cpp,v 1.4 2004/02/04 18:37:52 rbla Exp $
 begin       : Sat Nov 01 2003
 copyright   : (C) 2003 by Martin Preuss
 email       : martin@aquamaniac.de
*/

/***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "aqmauth.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pwd.h>
#include <errno.h>
#include <string.h>

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif


AqMoneyAuth::AqMoneyAuth(Hbci *hbci):Auth(hbci), _secrets(0) {
  _secrets=GWEN_DB_Group_new("secrets");
}



AqMoneyAuth::~AqMoneyAuth(){
  GWEN_DB_Group_free(_secrets);
}



Error AqMoneyAuth::getSecret(Pointer<User> user,
                             const string &token,
                             string &secret,
                             bool createSecret){
  const char *p;

  p=GWEN_DB_GetCharValue(_secrets, token.c_str(), 0, 0);
  if (!p) {
    DBG_INFO(0, "Secret \"%s\" not found, asking", token.c_str());
    return Auth::getSecret(user, token, secret, createSecret);
  }

  secret=p;
  return Error();
}



Error AqMoneyAuth::readPinList(const string &fname){
  struct stat st;
  struct passwd *pw;

  GWEN_DB_ClearGroup(_secrets, 0);
  if (stat(fname.c_str(), &st)) {
    fprintf(stderr,
            I18N("Error: stat(%s): %s\n"),
            fname.c_str(), strerror(errno));
    return Error("AqMoneyAuth::readPinList",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "error stat'ing file");
  }

  if ((st.st_mode & S_IRWXG) ||
      (st.st_mode & S_IRWXO)) {
    fprintf(stderr,
            I18N("=======================================\n"
                 " Very, very serious warning !!!\n"
                 " Your PIN file is world/group readable.\n"
                 " This really is very, very bad !\n"
                 " Evil people are able to read your PINs.\n"
                 " Therefore I abort now.\n"));
    return Error("AqMoneyAuth::readPinList",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "bad file mode");
  }

  pw=getpwuid(st.st_uid);
  if (!pw) {
    fprintf(stderr,
            I18N("Could not get shell info for owner of PIN file\n"));
    return Error("AqMoneyAuth::readPinList",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "bad owner");
  }

  if (strcasecmp(pw->pw_shell, "/bin/false")!=0) {
    fprintf(stderr,
            I18N("=========================================================\n"
                 " Serious warning !!\n"
                 " You should not use the PIN file as normal user !\n"
                 " This way all processes started by you can read the\n"
                 " PIN file.\n"
                 " I recommend:\n"
                 "  - creating a special user and group for AqMoney\n"
                 "  - setting his shell to \"/bin/false\"\n"
                 "  - setting the AqMoney executable to SETUID\n"
                 "  - add yourself to the newly created group\n"
                 "  - CHOWN the PIN file to the new user\n"
                 "  - CHMOD the PIN file so that only the owner can read it\n"
                 " This way even you as normal user can not read the\n"
                 " PIN file, and  neither can any process started by you\n"));
  }

  if (GWEN_DB_ReadFile(_secrets, fname.c_str(),
                       GWEN_DB_FLAGS_DEFAULT |
                       GWEN_PATH_FLAGS_CREATE_GROUP)) {
    return Error("AqMoneyAuth::readPinList",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "error reading file");
  }
  return Error();
}




void AqMoneyAuth::setSecretStatus(Pointer<User> user,
                                  const string &token,
                                  string &secret,
                                  HBCI_AUTH_STATUS st){
  const char *p;

  p=GWEN_DB_GetCharValue(_secrets, token.c_str(), 0, 0);
  if (!p) {
    DBG_INFO(0, "Secret \"%s\" not found", token.c_str());
  }
  else {
    if (st==HBCI_AUTH_STATUS_BAD) {
      DBG_WARN(0, "Secret \"%s\" is bad, please check your PIN file !!!",
               token.c_str());
      GWEN_DB_DeleteVar(_secrets, token.c_str());
    }
    else if (st==HBCI_AUTH_STATUS_OK) {
      DBG_INFO(0, "Secret \"%s\" is ok.", token.c_str());
    }
  }
}



HBCI_AUTH_STATUS AqMoneyAuth::getSecretStatus(Pointer<User> user,
                                              const string &token,
                                              string &secret){
  const char *p;

  p=GWEN_DB_GetCharValue(_secrets, token.c_str(), 0, 0);
  if (!p)
    return HBCI_AUTH_STATUS_UNKNOWN;
  return HBCI_AUTH_STATUS_OK;
}








