/***************************************************************************
 $RCSfile: aqmauth.h,v $
                             -------------------
 cvs         : $Id: aqmauth.h,v 1.5 2004/01/25 18:51:09 aquamaniac Exp $
 begin       : Sat Nov 01 2003
 copyright   : (C) 2003 by Martin Preuss
 email       : martin@aquamaniac.de
*/

/***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifndef AQMAUTH_H
#define AQMAUTH_H


#include <gwenhywfar/db.h>
#include <openhbci2/auth.h>
using namespace HBCI;


class AqMoneyAuth: public Auth {
private:
  GWEN_DB_NODE *_secrets;
public:
  /**
   * Constructor.
   * @author Martin Preuss<openhbci@aquamaniac.de>
   * @param hbci pointer to central HBCI object
   */
  AqMoneyAuth(Hbci *hbci);

  virtual ~AqMoneyAuth();

  /**
   * This method retrieves a PIN and writes it into the string
   * reference argument 'secret'.
   *
   * The default implementation calls @ref
   * Interactor::msgInputPin().  If you want to retrieve the PIN
   * by any other means, you can provide your own derivation of
   * Auth and override this method.  You may use the given token
   * to identify the secret.
   *
   * @author Martin Preuss<openhbci@aquamaniac.de>
   *
   * @return returns a Error object to show the result. Call
   * "isOk()" on it, to see if any error occurred.
   *
   * @param user pointer to the user for whom the secret is to be retrieved
   * From this user object you can get all information you might need to
   * show an explanatory dialog.
   *
   * @param token A string providing additional information to
   * identify the PIN OpenHBCI is asking for. When a MediumDDV
   * (chip card) asks for a pin, it shows here the "cardnumber",
   * whereas a MediumRDH presents the "path/filename" of the
   * medium to be accessed.
   *
   * @param secret reference to a string to receive the secret
   * (e.g. the pin).
   *
   * @param createSecret if true then a new secret is to be created.
   * In this case the user should be asked to enter the pin twice.  */
  virtual Error getSecret(Pointer<User> user,
                          const string &token,
                          string &secret,
                          bool createSecret=false);

  /**
   * Sets the status of the secret. This can be used after verifying a PIN
   * retrieved via @ref getSecret to tell the authentificator about the
   * status of the PIN.
   * If the status is @ref HBCI_AUTH_STATUS_BAD then the authentificator
   * should not return the same secret for the given token again !
   */
  void setSecretStatus(Pointer<User> user,
                       const string &token,
                       string &secret,
                       HBCI_AUTH_STATUS st);

  /**
   * Returns the status of the specified secret.
   */
  HBCI_AUTH_STATUS getSecretStatus(Pointer<User> user,
                                   const string &token,
                                   string &secret);

  /**
   * Reads the pin list
   */
  Error readPinList(const string &fname);

};


#endif




