/***************************************************************************
 $RCSfile: aqminteractor.cpp,v $
                             -------------------
    cvs         : $Id: aqminteractor.cpp,v 1.6 2004/02/09 22:35:21 rbla Exp $
    begin       : Tue Jul 16 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif


#define AQM_BUFSIZ 32


#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif

#include "aqminteractor.h"


#define AQMINTERACTOR_ABORT_CHAR 27
#define AQMINTERACTOR_DELETE_CHAR 127
#define AQMINTERACTOR_ENTER_CHAR 10



AQMInteractor::AQMInteractor(bool noInteraction)
:HBCI::Interactor(), _noInteraction(noInteraction){

}


AQMInteractor::~AQMInteractor(){
}


char AQMInteractor::_readCharFromStdin() {
  int chr;
  struct termios OldAttr, NewAttr;
  int AttrChanged = 0;
#ifdef HAVE_SIGNAL_H
  sigset_t snew, sold;
#endif

  // disable canonical mode to receive a single character
#ifdef HAVE_SIGNAL_H
  sigemptyset(&snew);
  sigaddset(&snew, SIGINT);
  sigaddset(&snew, SIGSTOP);
  sigprocmask(SIG_BLOCK, &snew, &sold);
#endif
  if (0 == tcgetattr (fileno (stdin), &OldAttr)){
    NewAttr = OldAttr;
    NewAttr.c_lflag &= ~ICANON;
    NewAttr.c_lflag &= ~ECHO;
    tcsetattr (fileno (stdin), TCSAFLUSH, &NewAttr);
    AttrChanged = !0;
  }

  for (;;) {
    chr=getchar();
    if (chr==-1 ||
        chr==AQMINTERACTOR_ABORT_CHAR ||
        chr== AQMINTERACTOR_ENTER_CHAR)
      break;
  }

  // re-enable canonical mode (if previously disabled)
  if (AttrChanged)
    tcsetattr (fileno (stdin), TCSADRAIN, &OldAttr);

#ifdef HAVE_SIGNAL_H
  sigprocmask(SIG_BLOCK, &sold, 0);
#endif

  return chr;
}



bool AQMInteractor::_inputPin(string &pin, bool r) {
  HBCI::Pointer<HBCI::Bank> b;
  string msg;
  char buffer[AQM_BUFSIZ];
  struct termios OldInAttr, NewInAttr;
  struct termios OldOutAttr, NewOutAttr;
  int AttrInChanged = 0;
  int AttrOutChanged = 0;
  int chr;
  unsigned int pos;
  bool rv;
#ifdef HAVE_SIGNAL_H
  sigset_t snew, sold;
#endif

  if (r)
    fprintf(stderr,I18N(" Repeat Pin: "));
  else
    fprintf(stderr,I18N(" Pin: "));

  // if possible, disable echo from stdin to stderr during password
  // entry
#ifdef HAVE_SIGNAL_H
  sigemptyset(&snew);
  sigaddset(&snew, SIGINT);
  sigaddset(&snew, SIGSTOP);
  sigprocmask(SIG_BLOCK, &snew, &sold);
#endif

  if (0 == tcgetattr (fileno (stdin), &OldInAttr)){
    NewInAttr = OldInAttr;
    NewInAttr.c_lflag &= ~ECHO;
    NewInAttr.c_lflag &= ~ICANON;
    tcsetattr (fileno (stdin), TCSAFLUSH, &NewInAttr);
    AttrInChanged = !0;
  }
  if (0 == tcgetattr (fileno (stderr), &OldOutAttr)){
    NewOutAttr = OldOutAttr;
    NewOutAttr.c_lflag &= ~ICANON;
    tcsetattr (fileno (stderr), TCSAFLUSH, &NewOutAttr);
    AttrOutChanged = !0;
  }

  // only show asterisks, don't use gets !
  pos=0;
  rv=false;
  for (;;) {
    chr=getchar();
    if (chr==AQMINTERACTOR_DELETE_CHAR) {
      if (pos) {
        pos--;
        fprintf(stderr, "%c %c", 8, 8);

      }
    }
    else if (chr==AQMINTERACTOR_ENTER_CHAR) {
      rv=true;
      fprintf(stderr, "\n");
      buffer[pos]=0;
      break;
    }
    else {
      if (pos<sizeof(buffer)) {
        if (chr==AQMINTERACTOR_ABORT_CHAR) {
          DBG_INFO(0, "User aborted");
          rv=false;
          break;
        }
        else {
          fprintf(stderr, "*");
          buffer[pos++]=chr;
          buffer[pos]=0;
        }
      }
      else {
        // buffer full
        fprintf(stderr, "\007");
      }
    }
  } // for

  // re-enable echo (if previously disabled)
  if (AttrOutChanged)
    tcsetattr (fileno (stderr), TCSADRAIN, &OldOutAttr);
  if (AttrInChanged)
    tcsetattr (fileno (stdin), TCSADRAIN, &OldInAttr);

#ifdef HAVE_SIGNAL_H
  sigprocmask(SIG_BLOCK, &sold, 0);
#endif

  for (unsigned int i=0; i<AQM_BUFSIZ; i++)
    if (buffer[i]<32) {
      buffer[i]=0;
      break;
    }
  pin=buffer;

  // aborted ?
  if (pin.length()==0)
    return false;

  // done
  return rv;
}



bool AQMInteractor::msgInputPin(HBCI::Pointer<HBCI::User> user,
                                string &pin,
                                int minsize,
                                bool newPin){
  HBCI::Pointer<HBCI::Bank> b;
  string msg;
  string lpin;
  bool pinOK;

  DBG_DEBUG(0, "AQMInteractor::msgInputPin");
    msg+=I18N(" PIN Entry\n"
              " --------------------------------------------------------\007\n"
              " You can cancel PIN entry anytime by pressing ESC key\n"
              " Finish PIN entry by pressing ENTER.\n" 
              " --------------------------------------------------------\n"
             );
  if (newPin)
    msg+=I18N(" Please enter the PIN for ");
  else
    msg+=I18N(" Please enter the current PIN for ");
  if (user.isValid()) {
    b=user.ref().bank();
    if (b.isValid()) {
      if (b.ref().name().empty())
        msg+=b.ref().bankCode();
      else
        msg+=b.ref().name();
    }
    else
      msg+=I18N("<unknown bank>");
  } // if user is valid
  else
    msg+=I18N("<unknown bank>");
  msg+=": ";
  fprintf(stderr,"%s\n",msg.c_str());

  pinOK=false;
  while(!pinOK) {
    if (!_inputPin(lpin, false)) {
      fprintf(stderr, I18N(" PIN entry cancelled.\n"));
      return false;
    }
    if (lpin.length()<(unsigned int)minsize) {
      fprintf(stderr,
              I18N(" PIN must be at least %d characters long.\n"),
              minsize);
    }
    if (newPin) {
      string llpin;
      if (!_inputPin(llpin, true)) {
        fprintf(stderr, I18N(" PIN entry cancelled.\n"));
        return false;
      }
      if (lpin!=llpin) {
        fprintf(stderr,
                I18N(" PINs not identical, please try again.\n"));
      }
      else
        pinOK=true;
    }
    else
      pinOK=true;
  } // while

  pin=lpin;

  // done
  return true;
}



void AQMInteractor::msgStartInputPinViaKeypad(Pointer<HBCI::User> user){
  HBCI::Pointer<HBCI::Bank> b;
  string msg;

  DBG_DEBUG(0, "AQMInteractor::msgStartInputPinViaKeypad");
  msg+=I18N(" \007Please enter the PIN for ");
  if (user.isValid()) {
    b=user.ref().bank();
    if (b.isValid()) {
      if (b.ref().name().empty())
        msg+=b.ref().bankCode();
      else
        msg+=b.ref().name();
    }
    else
      msg+=I18N("<unknown bank>");
  } // if user is valid
  else
    msg+=I18N("<unknown bank>");
  msg+=I18N(" at the keypad of your chipcard terminal:\n");
  fprintf(stderr,"%s",msg.c_str());
}



void AQMInteractor::msgFinishedInputPinViaKeypad(Pointer<HBCI::User> user){
  fprintf(stderr,I18N(" Secure PIN entry finished.\n"));
}



bool
AQMInteractor::msgInsertMediumOrAbort(HBCI::Pointer<HBCI::User> user,
                                      MediumType t){
  HBCI::Pointer<HBCI::Bank> b;
  string msg;
  int chr;

  DBG_DEBUG(0, "AQMInteractor::msgInsertMediumOrAbort");
  msg+=I18N(" \007Please insert medium for ");
  if (user.isValid()) {
    b=user.ref().bank();
    if (b.isValid()) {
      if (b.ref().name().empty())
        msg+=b.ref().bankCode();
      else
        msg+=b.ref().name();
    }
    else
      msg+=I18N("<unknown bank>");
  } // if user is valid
  else
    msg+=I18N("<unknown bank>");
  msg+="\n";
  msg+=I18N(" <Please press ENTER (or cancel with ESC)>\n");
  fprintf(stderr,"%s",msg.c_str());
  chr=_readCharFromStdin();
  if (chr==AQMINTERACTOR_ABORT_CHAR) {
    fprintf(stderr, I18N("User cancelled.\n"));
    return false;
  }
  fprintf(stderr, I18N("OK.\n"));
  return true;
}



bool
AQMInteractor::msgInsertCorrectMediumOrAbort(HBCI::Pointer<HBCI::User> user,
                                             MediumType t){
  HBCI::Pointer<HBCI::Bank> b;
  string msg;
  int chr;

  DBG_DEBUG(0, "AQMInteractor::msgInsertCorrectMediumOrAbort");
  msg+=I18N(" \007Please insert correct medium for ");
  if (user.isValid()) {
    b=user.ref().bank();
    if (b.isValid()) {
      if (b.ref().name().empty())
        msg+=b.ref().bankCode();
      else
        msg+=b.ref().name();
    }
    else
      msg+=I18N("<unknown bank>");
  } // if user is valid
  else
    msg+=I18N("<unknown bank>");
  msg+="\n";
  msg+=I18N(" <Please press ENTER (or cancel with ESC)>\n");
  fprintf(stderr,"%s",msg.c_str());
  chr=_readCharFromStdin();
  if (chr==AQMINTERACTOR_ABORT_CHAR) {
    fprintf(stderr, I18N("User cancelled.\n"));
    return false;
  }
  fprintf(stderr, I18N("OK.\n"));
  return true;
}



bool AQMInteractor::msgOpenConnection(TransportType t){
  string msg;
  char chr;

  if (_noInteraction)
    return true;
  msg+=I18N("\007 Please establish your ");
  switch (t) {
  case TransportTypeBTX: msg+=I18N("BTX"); break;
  case TransportTypeSSL:
  case TransportTypeTCP: msg+=I18N("internet"); break;
  default: break;
  } // switch
  msg+=I18N(" connection now...\n");
  msg+=I18N(" <Please press ENTER (or cancel with ESC)>\n");
  fprintf(stderr,"%s",msg.c_str());
  chr=_readCharFromStdin();
  if (chr==AQMINTERACTOR_ABORT_CHAR) {
    fprintf(stderr, I18N("User cancelled.\n"));
    return false;
  }
  fprintf(stderr, I18N("OK.\n"));
  return true;
}



void AQMInteractor::msgCloseConnection(TransportType t){
  string msg;

  msg+=I18N(" You can now terminate your ");
  switch (t) {
  case TransportTypeBTX: msg+=I18N("BTX"); break;
  case TransportTypeSSL:
  case TransportTypeTCP: msg+=I18N("internet"); break;
  default: break;
  } // switch
  msg+=I18N(" connection.\n");
  fprintf(stderr,"%s",msg.c_str());
}



void AQMInteractor::msgStateResponse(const string &msg){
  fprintf(stderr,I18N(" Result: %s\n"),msg.c_str());
}



void AQMInteractor::actionStarted(ActionProgressType t,
                                  const string &descr){
  string s;

  if (Hbci::debugLevel()>3) {
    switch(t) {
    case ACT_PROCESSMSG:
      s=I18N("Processing answer");
      break;
    case ACT_CREATEJOB:
      s+=I18N("Creating job segments");
      break;
    case ACT_HANDLEJOBS:
      s+=I18N("Processing jobs");
      break;
    case ACT_SIGNMSG:
      s+=I18N("Signing message");
      break;
    case ACT_ENCRYPTMSG:
      s+=I18N("Encrypting message");
      break;
    case ACT_VERIFYMSG:
      s+=I18N("Verifying signature");
      break;
    case ACT_DECRYPTMSG:
      s+=I18N("Decrypting message");
      break;
    case ACT_LOW_SENDMSG:
      s+=I18N("Sending message");
      break;
    case ACT_LOW_RECEIVEMSG:
      s+=I18N("Waiting for answer");
      break;
    default:
      break;
    }
  }

  if (Hbci::debugLevel()>2) {
    switch(t) {
    case ACT_ENCODEMSG:
      s+=I18N("Encoding message");
      break;
    case ACT_DECODEMSG:
      s+=I18N("Decoding message");
      break;
    default:
      break;
    }
  }
  if (Hbci::debugLevel()>1) {
    switch(t) {
    case ACT_OPENDIALOG:
      s+=I18N("Opening dialog");
      break;
    case ACT_CLOSEDIALOG:
      s+=I18N("Closing dialog");
      break;
    case ACT_UPDATESYSTEM:
      s+=I18N("Updating bank and user data");
      break;
    default:
      break;
    }
  }
  if (Hbci::debugLevel()>0) {
    switch(t) {
    case ACT_SENDINGMESSAGE:
      s+=I18N("Sending message");
      break;
    case ACT_FILLINGQUEUE:
      s+=I18N("Prepare messages");
      break;
    case ACT_CONTACTINGSERVER:
      s+=I18N("Connecting to server");
      break;
    case ACT_CHKRESULT:
      s+=I18N("Analyze results");
      break;
    case ACT_CLOSECONNECTION:
      s+=I18N("Closing connection to server");
      break;
    default:
      break;
    } // switch
  }
  if (Hbci::debugLevel()>5 && s.empty()) {
    s=descr;
  }
  if (!s.empty()) {
    fprintf(stderr," * %s\n", s.c_str());
  }
}


void AQMInteractor::logMessage(const string &msg){
  fprintf(stderr,"Log: %s\n",msg.c_str());
}




