/***************************************************************************
 $RCSfile: aqminteractor.h,v $
                             -------------------
    cvs         : $Id: aqminteractor.h,v 1.5 2004/01/25 18:51:09 aquamaniac Exp $
    begin       : Tue Jul 16 2002
    copyright   : (C) 2002 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/

#ifndef AQMINTERACTOR_H
#define AQMINTERACTOR_H

#include <openhbci2.h>
using namespace HBCI;

/**
 *
 */
class AQMInteractor: public HBCI::Interactor {
private:
  bool _noInteraction;

  char _readCharFromStdin();
  bool _inputPin(string &pin, bool r);

public:
  AQMInteractor(bool noInteraction);
  ~AQMInteractor();

  /**
   * Ask the user for the pin of a security medium. This method is called
   * by the default HBCI::Authentifikator.
   * @author Martin Preuss<martin@libchipcard.de>
   * @return true if ok, false if the user aborted
   * @param user user for whom the pin is to be asked for
   * @param pin reference to a variable to receive the pin
   * @param minsize minimum size of the pin.
   * @param newPin if true then the user should be asked for a new pin.
   * Most implementations will ask the user for the pin and then ask him
   * to confirm it (thus making the user doubly enter the pin).
   */
  virtual bool msgInputPin(HBCI::Pointer<HBCI::User> user,
                           string &pin,
                           int minsize,
                           bool newPin);
  /**
   * This method is called directly before the user is supposed to enter the
   * pin into the keypad of a card reader. This is used to inform the user
   * that OpenHBCI started secure pin verification. A graphical application
   * might want to open a window that tells the user about this.
   * (new since 0.9.5)
   * @author Martin Preuss<martin@libchipcard.de>
   * @param user user for whom the pin is to be asked for.
   * Please note that this pointer may be invalid.
   */
  virtual void msgStartInputPinViaKeypad(Pointer<HBCI::User> user);

  /**
   * This method is called directly after the secure pin verification (via
   * the card readers keypad) is finished. A graphical applicatition most
   * likely closes the notification window opened at
   * @ref msgStartInputPinViaKeypad().
   * (new since 0.9.5)
   * @author Martin Preuss<martin@libchipcard.de>
   * @param user user for whom the pin is to be asked for.
   * Please note that this pointer may be invalid.
   */
  virtual void msgFinishedInputPinViaKeypad(Pointer<HBCI::User> user);

  /**
   * Tell the user to insert a card.
   * @author Martin Preuss<martin@libchipcard.de>
   * @return true if ok, false if user wants to abort.
   */
  virtual bool msgInsertMediumOrAbort(HBCI::Pointer<HBCI::User> user,
                                      MediumType t);

  /**
   * Tell the user that:<ul>
   * <li>the currently inserted card is not the right one</li>
   * <li>he should now insert the correct card</li>
   * </ul>
   * @author Martin Preuss<martin@libchipcard.de>
   * @return true if ok, false if user wants to abort.
   */
  virtual bool
    msgInsertCorrectMediumOrAbort(HBCI::Pointer<HBCI::User> user,
                                  MediumType t);


  /**
   * Tell the user to open the connection of the given type.
   * In most cases this will result in a message telling the user
   * that (s)he should dial in now into his/her ISP
   * @author Martin Preuss<martin@libchipcard.de>
   * @return true if ok, false to abort
   */
  virtual bool msgOpenConnection(TransportType t);

  /**
   * Tell the user that the connection is no longer needed.
   * @author Martin Preuss<martin@libchipcard.de>
   */
  virtual void msgCloseConnection(TransportType t);

  /**
   * Prints a message that might be used for logging purpose. Does not
   * require interaction
   * This method should be overwritten by your application !
   * @param msg the message to show
   */
  virtual void msgStateResponse(const string &msg);

  /**
   * @short This method is called before an action starts.
   *
   * Tells this class that an action starts, and what action is to
   * be performed now. A derived class can use this to show what is
   * currently done.
   *
   * If you do not plan to give any kind of report to the user then you
   * have to overload this method with your version just doing nothing.
   * The default version of this method writes a short description to
   * stderr (for now, I guess I will later make this method do nothing.
   * So it is generally a good idea to overload this method).
   * @author Martin Preuss<martin@aquamaniac.de>
   * @param type Constant describing the type of action that is performed.
   * @param descr short description of what is to be done (like "sending
   * data to server").  */
  virtual void actionStarted(ActionProgressType type,
			     const string &descr);

  /**
   * Prints a message that might be used for logging purpose. Does not
   * require interaction
   * This method should be overwritten by your application !
   * @param msg the message to show
   */
  virtual void logMessage(const string &msg);

};





#endif // AQMINTERACTOR_H



