/***************************************************************************
  $RCSfile: account.cpp,v $
  -------------------
  cvs         : $Id: account.cpp,v 1.9 2004/01/25 18:51:09 aquamaniac Exp $
  begin       : Tue Oct 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "account.h"
#include "storagefile.h"
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>



Account::Balance::Balance(){
}


Account::Balance::Balance(const Value &value,
                          const Date &date,
                          const Time &tim)
:_value(value)
,_date(date)
,_time(tim) {
}



Account::Balance::~Balance(){
}






Account::Account(AqMoneyAPI *api,
                 const string &accountId,
                 const string &subId,
                 int country,
                 const string &bankCode,
                 const string &serverBankCode,
                 const string &accountName,
                 const string &customer,
                 const string &currency,
                 const string &userName,
                 const string &storageType,
                 const string &storageAddr)
:_api(api)
,_accountId(accountId)
,_subId(subId)
,_country(country)
,_bankCode(bankCode)
,_serverBankCode(serverBankCode)
,_accountName(accountName)
,_customer(customer)
,_currency(currency)
,_userName(userName)
,_standingOrders(0)
,_datedTransfers(0)
,_notedTransfers(0){
  _storage=_api->storageFactory(storageType,
                                country,
                                bankCode,
                                accountId,
                                storageAddr,
                                "transactions");
  _transferStorage=_api->storageFactory(storageType,
                                        country,
                                        bankCode,
                                        accountId,
                                        storageAddr,
                                        "transfers");
  _standingOrders=GWEN_DB_Group_new("standingOrders");
  _datedTransfers=GWEN_DB_Group_new("datedTransfers");
  _notedTransfers=GWEN_DB_Group_new("notedTransfers");
}


Account::Account(AqMoneyAPI *api, GWEN_DB_NODE *db) throw (Error)
:_api(api)
,_country(280)
,_standingOrders(0)
,_datedTransfers(0){
  Error err;

  _standingOrders=GWEN_DB_Group_new("standingOrders");
  _datedTransfers=GWEN_DB_Group_new("datedTransfers");
  _notedTransfers=GWEN_DB_Group_new("notedTransfers");
  err=fromConfig(db);
  if (!err.isOk())
    throw Error("Account::Account", err);
}



Account::~Account(){
  GWEN_DB_Group_free(_standingOrders);
  GWEN_DB_Group_free(_datedTransfers);
  GWEN_DB_Group_free(_notedTransfers);
}



Error Account::fromConfig(GWEN_DB_NODE *db){
  GWEN_DB_NODE *n;
  Value val;
  Date d;
  Time t;

  _accountId=GWEN_DB_GetCharValue(db, "accountid", 0, "");
  _subId=GWEN_DB_GetCharValue(db, "accountsubid", 0, "");
  _accountName=GWEN_DB_GetCharValue(db, "accountname", 0, "");
  _country=GWEN_DB_GetIntValue(db, "country", 0, 280);
  _bankCode=GWEN_DB_GetCharValue(db, "bankcode", 0, "");
  _serverBankCode=GWEN_DB_GetCharValue(db, "serverbankcode", 0,
                                       _bankCode.c_str());
  _customer=GWEN_DB_GetCharValue(db, "customer", 0, "");
  _currency=GWEN_DB_GetCharValue(db, "currency", 0, "");
  _userName=GWEN_DB_GetCharValue(db, "username", 0, "");
  _bankLine=Value(GWEN_DB_GetCharValue(db, "creditline/value", 0, "0,0"),
                  GWEN_DB_GetCharValue(db, "creditline/currency", 0, "EUR"));

  /* read transactions */
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "storage");
  if (!n)
    return Error("Account::fromConfig",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_UNKNOWN,
                 ERROR_ADVISE_DONTKNOW,
                 "No storage info given");
  try {
    _storage=_api->storageFactory(n,
                                  _country,
                                  _bankCode,
                                  _accountId);
  }
  catch (Error xerr) {
    return Error("Account::fromConfig", xerr);
  }

  /* read transfers */
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "transferstorage");
  if (!n)
    return Error("Account::fromConfig",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_UNKNOWN,
                 ERROR_ADVISE_DONTKNOW,
                 "No transfer storage info given");
  try {
    _transferStorage=_api->storageFactory(n,
                                          _country,
                                          _bankCode,
                                          _accountId);
  }
  catch (Error xerr) {
    return Error("Account::fromConfig", xerr);
  }

  // read booked balance
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "booked-balance");
  if (n) {
    DBG_INFO(0, "loading booked balance");
    val=Value(GWEN_DB_GetCharValue(n, "value/value", 0, "0,0"),
              GWEN_DB_GetCharValue(n, "value/currency", 0, "EUR"));

    d=Date(GWEN_DB_GetCharValue(n, "date", 0, ""));
    t=Time(GWEN_DB_GetCharValue(n, "time", 0, ""));
    _bookedBalance=Balance(val, d, t);
  }

  // read noted balance
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "noted-balance");
  if (n) {
    DBG_INFO(0, "Loading noted balance");
    val=Value(GWEN_DB_GetCharValue(n, "value/value", 0, "0,0"),
              GWEN_DB_GetCharValue(n, "value/currency", 0, "EUR"));
    d=Date(GWEN_DB_GetCharValue(n, "date", 0, ""));
    t=Time(GWEN_DB_GetCharValue(n, "time", 0, ""));
    _notedBalance=Balance(val, d, t);
  }

  // read standing orders
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "standingOrders");
  if (n) {
    DBG_INFO(0, "Loading standing orders");
    GWEN_DB_AddGroupChildren(_standingOrders, n);
  }

  // read dated transfers
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "datedTransfers");
  if (n) {
    DBG_INFO(0, "Loading dated transfers");
    GWEN_DB_AddGroupChildren(_datedTransfers, n);
  }

  // read noted transfers
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_NAMEMUSTEXIST,
                     "notedTransfers");
  if (n) {
    DBG_INFO(0, "Loading noted transfers");
    GWEN_DB_AddGroupChildren(_notedTransfers, n);
  }

  return Error();
}



Error Account::toConfig(GWEN_DB_NODE *db){
  GWEN_DB_NODE *n;
  Error err;

  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "accountid",
                       _accountId.c_str());
  if (!_subId.empty())
      GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                           "accountsubid",
                           _subId.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "accountname",
                       _accountName.c_str());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT,
                      "country",
                      _country);
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "bankcode",
                       _bankCode.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "serverbankcode",
                       _serverBankCode.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "customer",
                       _customer.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "currency",
                       _currency.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "username",
                       _userName.c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "creditline/value",
                       _bankLine.toString(true).c_str());
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT,
                       "creditline/currency",
                       _bankLine.getCurrency().c_str());

  // store transaction storage info
  n=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
                     "storage");
  assert(n);
  err=_storage.ref().toConfig(n);
  if (!err.isOk())
    return Error("Account::toConfig", err);

  // store transfer storage info
  n=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT,
                     "transferstorage");
  assert(n);
  err=_transferStorage.ref().toConfig(n);
  if (!err.isOk())
    return Error("Account::toConfig", err);

  // store booked balance
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT,
                     "booked-balance");
  assert(n);
  DBG_INFO(0, "Saving booked balance");
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "value/value",
                       _bookedBalance.value().toString(true).c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "value/currency",
                       _bookedBalance.value().getCurrency().c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "date",
                       _bookedBalance.date().toString().c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "time",
                       _bookedBalance.time().toString().c_str());

  // store noted balance
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT,
                     "noted-balance");
  assert(n);
  DBG_INFO(0, "Saving noted balance");
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "value/value",
                       _notedBalance.value().toString(true).c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "value/currency",
                       _notedBalance.value().getCurrency().c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "date",
                       _notedBalance.date().toString().c_str());
  GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                       "time",
                       _notedBalance.time().toString().c_str());

  // store standing orders
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT,
                     "standingOrders");
  assert(n);
  DBG_INFO(0, "Saving standing orders");
  GWEN_DB_AddGroupChildren(n, _standingOrders);

  // store dated transfers
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT,
                     "datedTransfers");
  assert(n);
  DBG_INFO(0, "Saving dated transfers");
  GWEN_DB_AddGroupChildren(n, _datedTransfers);

  // store noted transfers
  n=GWEN_DB_GetGroup(db,
                     GWEN_DB_FLAGS_DEFAULT,
                     "notedTransfers");
  assert(n);
  DBG_INFO(0, "Saving noted transfers");
  GWEN_DB_AddGroupChildren(n, _notedTransfers);

  return Error();
}



Error Account::addStandingOrder(GWEN_DB_NODE *xa){
  GWEN_DB_NODE *n;

  n=GWEN_DB_GetGroup(_standingOrders,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_CREATE_GROUP,
                     "standingOrder");
  assert(n);
  if (GWEN_DB_AddGroupChildren(n, xa)) {
    return Error("Account::addStandingOrder",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_UNKNOWN,
                 ERROR_ADVISE_DONTKNOW,
                 "Could not add standing order");
  }
  return Error();
}



void Account::clearStandingOrders(){
  GWEN_DB_ClearGroup(_standingOrders, 0);
}



GWEN_DB_NODE *Account::getStandingOrders(int country,
                                         const string &bankCode,
                                         const string &accountId){
  GWEN_DB_NODE *xas;

  xas=_getMatchingTransfers(_standingOrders,
                            "standingOrder",
                            country,
                            bankCode,
                            accountId);
  return xas;
}



Error Account::addDatedTransfer(GWEN_DB_NODE *xa){
  GWEN_DB_NODE *n;

  n=GWEN_DB_GetGroup(_datedTransfers,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_CREATE_GROUP,
                     "datedTransfer");
  assert(n);
  if (GWEN_DB_AddGroupChildren(n, xa)) {
    return Error("Account::addDatedTransfer",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_UNKNOWN,
                 ERROR_ADVISE_DONTKNOW,
                 "Could not add dated transfer");
  }
  return Error();
}



void Account::clearDatedTransfers(){
  GWEN_DB_ClearGroup(_datedTransfers, 0);
}



GWEN_DB_NODE *Account::getDatedTransfers(int country,
                                         const string &bankCode,
                                         const string &accountId){
  GWEN_DB_NODE *xas;

  xas=_getMatchingTransfers(_datedTransfers,
                            "datedTransfer",
                            country,
                            bankCode,
                            accountId);
  return xas;
}



Error Account::addNotedTransfer(GWEN_DB_NODE *xa){
  GWEN_DB_NODE *n;

  n=GWEN_DB_GetGroup(_notedTransfers,
                     GWEN_DB_FLAGS_DEFAULT |
                     GWEN_PATH_FLAGS_CREATE_GROUP,
                     "notedTransfer");
  assert(n);
  if (GWEN_DB_AddGroupChildren(n, xa)) {
    return Error("Account::addDatedTransfer",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_UNKNOWN,
                 ERROR_ADVISE_DONTKNOW,
                 "Could not add noted transfer");
  }
  return Error();
}



void Account::clearNotedTransfers(){
  GWEN_DB_ClearGroup(_notedTransfers, 0);
}



GWEN_DB_NODE *Account::getNotedTransfers(int country,
                                         const string &bankCode,
                                         const string &accountId){
  GWEN_DB_NODE *xas;

  xas=_getMatchingTransfers(_notedTransfers,
                            "notedTransfer",
                            country,
                            bankCode,
                            accountId);
  return xas;
}



GWEN_DB_NODE *Account::_getMatchingTransfers(GWEN_DB_NODE *transfers,
                                             const char *tname,
                                             int country,
                                             const string &bankCode,
                                             const string &accountId){
  GWEN_DB_NODE *n;
  GWEN_DB_NODE *xas;
  int nxa;

  xas=GWEN_DB_Group_new("matchingTransfers");

  n=GWEN_DB_GetFirstGroup(transfers);
  while(n) {
    bool matches;

    matches=true;
    // check other institute
    if (-1==
        GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(n,
                                                      "otherBankCode",
                                                      0,
                                                      ""),
                                 bankCode.c_str(), 0)){
      DBG_INFO(0, "otherBankCode doesnt match");
      matches=false;
    }

    // check other country
    if (country && country!=GWEN_DB_GetIntValue(n,
                                                "otherCountryCode",
                                                0,
                                                280)) {
      DBG_INFO(0, "otherCountry doesnt match");
      matches=false;
    }

    // check other account
    if (-1==
        GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(n,
                                                      "otherAccount",
                                                      0,
                                                      ""),
                                 accountId.c_str(), 0)) {
      DBG_INFO(0, "otherAccount doesnt match");
      matches=false;
    }

    if (matches) {
      // add transaction
      GWEN_DB_NODE *nuTarget;

      DBG_INFO(0, "Transaction matches");
      nuTarget=GWEN_DB_GetGroup(xas,
                                GWEN_DB_FLAGS_DEFAULT |
                                GWEN_PATH_FLAGS_CREATE_GROUP,
                                tname);
      GWEN_DB_AddGroupChildren(nuTarget,
                               n);
      nxa++;
    }
    else {
      DBG_DEBUG(0, "Transfer does not match");
    }
    n=GWEN_DB_GetNextGroup(n);
  } // while

  if (!nxa) {
    GWEN_DB_Group_free(xas);
    return 0;
  }
  return xas;
}





