/***************************************************************************
  $RCSfile: adminjobs.cpp,v $
  -------------------
  cvs         : $Id: adminjobs.cpp,v 1.9 2004/02/10 17:10:38 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) dgettext("aqmoney2",m)
#else
# define I18N(m) m
#endif

#include "adminjobs.h"
#include "aqmstring.h"
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>


AdminJob::AdminJob(const string &name,
                   Pointer<Customer> c)
:OutboxJob(name, c) {
  assert(c.isValid());
}



AdminJob::~AdminJob(){
}


Error AdminJob::commit(){
  return Error();
}


Error AdminJob::evaluate(Pointer<Outbox> outbox){
  if (results().empty()) {
    if (outbox.ref().results().empty()) {
      setStatus(HBCI_JOB_STATUS_DONE);
      setResult(HBCI_JOB_RESULT_FAILED);
      return Error("AdminJob::evaluate",
                   ERROR_LEVEL_NORMAL,
                   HBCI_ERROR_CODE_INVALID,
                   ERROR_ADVISE_DONTKNOW,
                   "no results");
    }
  }
  return OutboxJob::evaluate(outbox);
}



string AdminJob::resultString(bool sh){
  list<MsgJob::Result>::const_iterator rit;
  Error err;
  string res;
  Pointer<Bank> bank;
  bool haveWarnings;

  haveWarnings=false;
  if (!results().empty()) {
    for (rit=results().begin();
         rit!=results().end();
         rit++) {
      if ((*rit).resultCode()>=3000 &&
          (*rit).resultCode()<4000) {
        haveWarnings=true;
        break;
      }
    } // for
  }

  bank=customer().ref().user().ref().bank();
  res+=AqmoneyString::format(I18N(" %s at %s"),
                             name().c_str(),
                             bank.ref().name().empty()
                               ? bank.ref().bankCode().c_str()
                               : bank.ref().name().c_str()
                            );
  if (sh) {
    res+=": ";

  }
  else {
    res+="\n";

    res+=I18N("  Overall result: ");
  }

  switch(result()) {
  case HBCI_JOB_RESULT_SUCCESS:        res+=I18N("OK"); break;
  case HBCI_JOB_RESULT_PARTLY_SUCCESS: res+=I18N("partly OK"); break;
  case HBCI_JOB_RESULT_FAILED:         res+=I18N("failed"); break;
  default:                             res+=I18N("unknown"); break;
  } // switch

  if (haveWarnings)
    res+=I18N(" (with warnings)");
  res+="\n";

  if (!sh || haveWarnings) {
    if (!results().empty()) {
      res+=I18N("  Individual results:\n");
      for (rit=results().begin();
           rit!=results().end();
           rit++) {
        res+="   ";
        if ((*rit).resultCode()>=9000)
          res+=I18N("Error  : ");
        else if ((*rit).resultCode()>=3000 && (*rit).resultCode()<4000)
          res+=I18N("Warning: ");
        else
          res+=I18N("Notice : ");
        res+=String::num2string((*rit).resultCode(), false, 4);
        res+=" ";
        res+=(*rit).resultMessage();
        res+="\n";
      } // for result
    }
    else {
      res+=I18N("  No individual results for this job\n");
    }
  }
  return res;
}





JobGetAccounts::JobGetAccounts(Pointer<Customer> c)
:AdminJob("JobGetAccounts", c) {
  Pointer<Bank> bank;
  Pointer<User> user;
  Pointer<Medium> medium;
  string mid;

  assert(c.isValid());
  user=c.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();

  GWEN_DB_NODE *jobData=debugData();

  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setProperty("open/ident/customerid", c.ref().custId());
  setIntProperty("open/prepare/updversion",0);

  if (medium.ref().securityMode()==HBCI_SECURITY_RDH) {
    mid=medium.ref().mediumId();
    if (mid.empty())
      mid="0";
    setProperty("open/ident/systemId", mid.c_str());
  }

  jobData=debugData();

}



JobGetAccounts::~JobGetAccounts(){
}



Error JobGetAccounts::commit(){
  return Error();
}





JobGetBankInfo::JobGetBankInfo(Pointer<Customer> c)
:AdminJob("JobGetBankInfo", c) {
  Pointer<Bank> bank;
  Pointer<User> user;
  Pointer<Medium> medium;
  string mid;

  assert(c.isValid());
  user=c.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();

  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setIntProperty("open/prepare/bpdversion",0);
  setIntProperty("open/prepare/updversion",0);
}



JobGetBankInfo::~JobGetBankInfo(){
}



Error JobGetBankInfo::commit(){
  return Error();
}



JobUpdateBankInfo::JobUpdateBankInfo(Pointer<Customer> c)
:AdminJob("JobUpdateBankInfo", c) {
  Pointer<Bank> bank;
  Pointer<User> user;
  Pointer<Medium> medium;
  string mid;

  assert(c.isValid());
  user=c.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();

  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setIntProperty("open/prepare/bpdversion",0);
  setIntProperty("open/prepare/updversion",0);
}



JobUpdateBankInfo::~JobUpdateBankInfo(){
}





Error JobUpdateBankInfo::commit(){
  return Error();
}







JobSync::JobSync(Pointer<Customer> c, AQM_SYNC_MODE syncmode)
:AdminJob("JobSync", c), _syncmode(syncmode) {
  Pointer<Bank> bank;
  Pointer<User> user;

  assert(c.isValid());
  user=c.ref().user();
  bank=user.ref().bank();

  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setProperty("open/ident/customerid", c.ref().custId());
  setIntProperty("open/sync/mode", syncmode);
  if (syncmode==AqmSyncModeSystemId) {
    setProperty("open/ident/systemid", "0");
    //setProperty("open/ident/status", "0");
  }

}



JobSync::~JobSync(){
}










JobGetSystemId::JobGetSystemId(Pointer<Customer> c)
:JobSync(c, AqmSyncModeSystemId) {
}



JobGetSystemId::~JobGetSystemId(){
}



Error JobGetSystemId::commit(){
  Pointer<User> user;
  string sysid;

  sysid=getProperty("response/syncresponse/systemid", "");
  if (sysid.empty()) {
    DBG_ERROR(0, "No system id");
    return Error("JobGetSystemId::commit",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "no system id");
  }
  DBG_INFO(0, "Got a systemId (%s)", sysid.c_str());
  user=customer().ref().user();
  user.ref().setSystemId(sysid);

  return Error();
}





JobGetKeys::JobGetKeys(Pointer<Customer> c)
:AdminJob("JobGetKeys", c) {
  Pointer<Bank> bank;
  Pointer<User> user;
  Pointer<Medium> medium;

  assert(c.isValid());
  user=c.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();

  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setProperty("open/ident/customerId", "9999999999");
  setProperty("open/ident/systemId", "0");

  setIntProperty("open/signkey/key/country", bank.ref().country());
  setProperty("open/signkey/key/bankcode", bank.ref().bankCode());

  setIntProperty("open/cryptkey/key/country", bank.ref().country());
  setProperty("open/cryptkey/key/bankcode", bank.ref().bankCode());
}



JobGetKeys::~JobGetKeys(){
}


Error JobGetKeys::commit(){
  Pointer<MediumRDHBase> mrdh;
  Pointer<Bank> bank;
  Pointer<User> user;
  Error err;

  if (!_cryptKey.isValid() && !_signKey.isValid())
    return Error("JobGetKeys::commit",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "no keys");
  try {
    // get some vars
    user=customer().ref().user();
    bank=user.ref().bank();
    mrdh=customer().ref().user().ref().medium().cast<MediumRDHBase>();

    // mount medium
    err=mrdh.ref().mountMedium();
    if (!err.isOk())
      return Error("JobGetKeys::commit", err);

    // select context
    err=mrdh.ref().selectContext(bank.ref().country(),
                                 bank.ref().bankCode(),
                                 user.ref().userId());
    if (!err.isOk()) {
      mrdh.ref().unmountMedium();
      return Error("JobGetKeys::commit", err);
    }

    // set crypt key
    if (_cryptKey.isValid()) {
      DBG_NOTICE(0, "Setting Institute Crypt Key");
      if (!_cryptKey.ref().isCryptoKey()) {
        DBG_ERROR(0, "Crypt key expected");
        return Error("JobGetKeys::commit",
                     ERROR_LEVEL_NORMAL,
                     HBCI_ERROR_CODE_INVALID,
                     ERROR_ADVISE_DONTKNOW,
                     "bad key order");
      }
      err=mrdh.ref().setInstituteCryptKey(_cryptKey);
      if (!err.isOk()) {
        mrdh.ref().unmountMedium();
        return Error("JobGetKeys::commit", err);
      }
    }

    // set sign key
    if (_signKey.isValid()) {
      DBG_NOTICE(0, "Setting Institute Sign Key");
      err=mrdh.ref().setInstituteSignKey(_signKey);
      if (!err.isOk()){
        mrdh.ref().unmountMedium();
        return Error("JobGetKeys::commit", err);
      }
      if (!mrdh.ref().hasInstSignKey()) {
        DBG_ERROR(0,
                  "What ??? I just set the signkey but there is none ?!");
      }
    }
    err=mrdh.ref().unmountMedium();
    if (!err.isOk()){
      return Error("JobGetKeys::commit", err);
    }
    DBG_NOTICE(0, "New institute keys activated");
  }
  catch (Error xerr) {
    return Error("JobGetKeys::commit", xerr);
  }

  return Error();
}


Error JobGetKeys::evaluate(Pointer<Outbox> outbox) {
  GWEN_DB_NODE *rsp;
  GWEN_DB_NODE *n;
  Error err;

  err=AdminJob::evaluate(outbox);
  if (!err.isOk())
    return Error("JobGetKeys::evaluate", err);

  rsp=responseData();
  if (!rsp) {
    setStatus(HBCI_JOB_STATUS_DONE);
    setResult(HBCI_JOB_RESULT_FAILED);
    return Error("JobGetKeys::evaluate",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "no response data");
  }

  n=GWEN_DB_GetFirstGroup(rsp);
  while(n) {
    if (strcasecmp(GWEN_DB_GroupName(n), "GetKeyResponse")==0) {
      Pointer<User> user;
      Pointer<Medium> medium;
      unsigned int bs;
      const void *p;
      GWEN_DB_NODE *keydb;
      const char defaultExpo[3]={0x01, 0x00, 0x01};
      bool iscrypt;

      DBG_NOTICE(0, "Found Key response");
      iscrypt=false;

      keydb=GWEN_DB_Group_new("key");
      GWEN_DB_SetCharValue(keydb,
                           GWEN_DB_FLAGS_OVERWRITE_VARS,
                           "type",
                           "RSA");
      // TODO: check for the correct exponent (for now assume 65537)
      GWEN_DB_SetBinValue(keydb,
                          GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "data/e",
                          defaultExpo,
                          sizeof(defaultExpo));

      GWEN_DB_SetIntValue(keydb,
                          GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "data/public",
                          1);

      iscrypt=(strcasecmp(GWEN_DB_GetCharValue(n,
                                               "keyname/keytype", 0,
                                               "V"), "V")==0);
      GWEN_DB_SetCharValue(keydb,
                           GWEN_DB_FLAGS_OVERWRITE_VARS,
                           "name",
                           GWEN_DB_GetCharValue(n,
                                                "keyname/keytype", 0,
                                                "V"));
      GWEN_DB_SetCharValue(keydb,
                           GWEN_DB_FLAGS_OVERWRITE_VARS,
                           "owner",
                           GWEN_DB_GetCharValue(n, "keyname/userId", 0, ""));
      GWEN_DB_SetIntValue(keydb,
                          GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "number",
                          GWEN_DB_GetIntValue(n, "keyname/keynum", 0, 0));
      GWEN_DB_SetIntValue(keydb,
                          GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "version",
                          GWEN_DB_GetIntValue(n, "keyname/keyversion", 0, 0));


      p=GWEN_DB_GetBinValue(n, "key/modulus", 0, 0, 0 , &bs);
      if (!p || !bs) {
	setStatus(HBCI_JOB_STATUS_DONE);
	setResult(HBCI_JOB_RESULT_FAILED);
	return Error("JobGetKeys::evaluate",
		     ERROR_LEVEL_NORMAL,
		     HBCI_ERROR_CODE_INVALID,
		     ERROR_ADVISE_DONTKNOW,
		     "no modulus");
      }
      GWEN_DB_SetBinValue(keydb,
			  GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "data/n",
			  p, bs);

      if (iscrypt)
	_cryptKey=new RSAKey(iscrypt, keydb);
      else
	_signKey=new RSAKey(iscrypt, keydb);
      DBG_NOTICE(0, "Created %s key", iscrypt?"crypt":"sign");
    } // if we have a key response
    n=GWEN_DB_GetNextGroup(n);
  } // while

  // use result returned from lower class
  return Error();
}









JobSendKeys::JobSendKeys(Pointer<Customer> c,
                         Pointer<RSAKey> signKey,
                         Pointer<RSAKey> cryptKey)
:AdminJob("JobSendKeys", c), _signKey(signKey), _cryptKey(cryptKey){
  Pointer<Bank> bank;
  Pointer<User> user;

  // check for valid keys
  if (!signKey.isValid())
    throw Error("JobSendKeys::JobSendKeys",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_INVALID,
                ERROR_ADVISE_DONTKNOW,
                "no signkey");
  if (!cryptKey.isValid())
    throw Error("JobSendKeys::JobSendKeys",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_INVALID,
                ERROR_ADVISE_DONTKNOW,
                "no cryptkey");

  user=c.ref().user();
  bank=user.ref().bank();

  // prepare identification
  setProperty("open/ident/systemid", "0");
  setIntProperty("open/ident/country", bank.ref().country());
  setProperty("open/ident/bankcode", bank.ref().bankCode());
  setProperty("open/ident/customerid", c.ref().custId());

  // prepare sign key
  setProperty("open/signkey/keyname/bankCode", bank.ref().bankCode());
  setProperty("open/signkey/keyname/userid", user.ref().userId());
  setIntProperty("open/signkey/keyname/keynum", signKey.ref().number());
  setIntProperty("open/signkey/keyname/keyversion", signKey.ref().version());
  setBinProperty("open/signkey/key/modulus", signKey.ref().getModulusData());
  setBinProperty("open/signkey/key/exponent", signKey.ref().getExpData());

  // prepare crypt key
  setProperty("open/cryptkey/keyname/bankCode", bank.ref().bankCode());
  setProperty("open/cryptkey/keyname/userid", user.ref().userId());
  setIntProperty("open/cryptkey/keyname/keynum", cryptKey.ref().number());
  setIntProperty("open/cryptkey/keyname/keyversion", cryptKey.ref().version());
  setBinProperty("open/cryptkey/key/modulus", cryptKey.ref().getModulusData());
  setBinProperty("open/cryptkey/key/exponent", cryptKey.ref().getExpData());
}



JobSendKeys::~JobSendKeys(){
}



Error JobSendKeys::evaluate(Pointer<Outbox> outbox){
  return AdminJob::evaluate(outbox);
}



Error JobSendKeys::commit(){
  return Error();
}






JobGetReports::JobGetReports(Pointer<Customer> c)
:AdminJob("JobGetProtocol", c){
}



JobGetReports::~JobGetReports(){
}



Error JobGetReports::evaluate(Pointer<Outbox> outbox){
  return AdminJob::evaluate(outbox);
}



Error JobGetReports::commit(){
  return Error();
}










