/***************************************************************************
  $RCSfile: aqmapi.h,v $
  -------------------
  cvs         : $Id: aqmapi.h,v 1.7 2004/01/25 18:51:24 aquamaniac Exp $
  begin       : Tue Oct 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifndef AQMONEY_API_H
#define AQMONEY_API_H

#define AQMONEY_DEFAULT_HBCI_VERSION 210
#define AQMONEY_DEFAULT_MSG_ENGINE "msgxml"
#define AQMONEY_DEFAULT_CONN_TYPE "xptcp"
#define AQMONEY_DEFAULT_CONN_PORT "3000"
#define AQMONEY_DEFAULT_CONN_TIMEOUT 500



class AqMoneyAPI;

#include <aqmoney2/account.h>
#include <aqmoney2/storage.h>
#include <aqmoney2/bankmanager.h>
#include <aqmoney2/aqmconverter.h>

#include <list>
#include <openhbci2.h>
using namespace HBCI;


/**
 *
 */
class AqMoneyAPI: public API {
private:
  list<Pointer<Account> > _accounts;
  string _defaultStorageType;
  string _defaultStorageAddr;
  GWEN_DB_NODE *_bankMessages;
  Pointer<BankManager> _bankManager;

public:
  AqMoneyAPI(bool readonly = false);
  virtual ~AqMoneyAPI();

  virtual Error loadEnvironment(const string &openhbciFile,
                                const string &aqmoneyFile);

  virtual Error saveEnvironment(const string &openhbciFile,
                                const string &aqmoneyFile);

  Error addAccount(Pointer<Account> acc);
  Pointer<Account> findAccount(int country,
                               const string &bankCode,
                               const string &accountId,
                               const string &accountSubId);
  list<Pointer<Account> > getAccounts(int country=0,
                                      const string &bankCode="*",
                                      const string &accountId="*",
                                      const string &accountSubId="*");

  Pointer<Account>
    accountFactory(const string &customer,
                   const string &userName,
                   const string &accountId,
                   const string &accountSubId,
                   int country,
                   const string &bankCode,
                   const string &serverBankCode="",
                   const string &accountName="",
                   const string &currency="EUR",
                   const string &storageType="",
                   const string &storageAddr="");

  /**
   * Returns a private copy of the stored bank messages.
   * The caller is responsible for freeing the data returned (by calling
   * GWEN_DB_Group_free).
   */
  GWEN_DB_NODE *getBankMessages();

  Pointer<BankManager> bankManager() { return _bankManager;};

  void setDefaultStorage(const string &tname,
                         const string &taddr);
  Pointer<TransactionStorage> storageFactory(const string &name,
                                             int country=280,
                                             const string &bankCode="",
                                             const string &account="",
                                             const string &addr="",
                                             const string &addrsuffix="");
  Pointer<TransactionStorage> storageFactory(GWEN_DB_NODE *db,
                                             int country=280,
                                             const string &bc="",
                                             const string &ac="");
  Pointer<AQMConverter> converterFactory(const string &name,
                                         const string &param);

  /**
   * Processes the Outbox response stored inside the given node.
   * @param db db node which contains the outbox response.
   * This method takes over ownership of that node.
   */
  Error processOutboxResponse(GWEN_DB_NODE *db);

};


#endif


