/***************************************************************************
 $RCSfile: aqmconverter.h,v $
                             -------------------
    cvs         : $Id: aqmconverter.h,v 1.4 2004/01/25 18:51:24 aquamaniac Exp $
    begin       : Sat May 17 2003
    copyright   : (C) 2003 by Martin Preuss
    email       : martin@aquamaniac.de
*/

/***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifndef AQMONEY_CONVERTER_H
#define AQMONEY_CONVERTER_H


#include <openhbci2.h>
#include <gwenhywfar/db.h>
#include <gwenhywfar/bufferedio.h>
#include <stdio.h>


using namespace std;
using namespace HBCI;



/**
 * This is a generic transaction converter class.
 * @author Martin Preuss<martin@libchipcard.de>
 */
class AQMConverter {
private:
  GWEN_DB_NODE *_transactions;
public:
  AQMConverter();

  /*
   * Please note that after destryoing this object all pointers returned by
   * @ref transactions() become invalid !
   */
  virtual ~AQMConverter();

  /**
   * Reads a stream until the end of stream is reached and translates the
   * input data to a list of transactions.
   */
  virtual Error readFromStream(GWEN_BUFFEREDIO *bio);

  /**
   * Writes the internal list of transactions to the given stream using
   * this classes own format (well, not exactly this classes, this is just
   * an abstract base class. However, inheriting classes define their own
   * format).
   */
  virtual Error writeToStream(GWEN_BUFFEREDIO *bio);

  /**
   * Clears the internal list of transactions.
   * Please note that after calling this all pointers returned by
   * @ref transactions() become invalid !
   */
  void clear();

  /**
   * Adds the given transaction to the internal list.
   * This method does NOT take over ownership of the given object.
   */
  void addTransaction(GWEN_DB_NODE *xa);

  /**
   * Adds the given transactions to the internal list.
   * This method does NOT take over ownership of the given object.
   */
  void addTransactions(GWEN_DB_NODE *xa);

  /**
   * Replaces the internal transaction list by the given one.
   * @param take if true this method takes over ownership of the object
   */
  void setTransactions(GWEN_DB_NODE *xa, bool take);

  /**
   * Returns the list of transactions.
   * You should treat this as CONST (I could return a private copy here, but
   * I want to avoid copying whenever possible).
   * You MUST NOT free this pointer.
   */
  GWEN_DB_NODE *transactions();
};


#endif // AQMONEY_CONVERTER_H

