/***************************************************************************
  $RCSfile: bankmanager.cpp,v $
  -------------------
  cvs         : $Id: bankmanager.cpp,v 1.2 2004/01/25 18:51:25 aquamaniac Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "bankmanager.h"

#ifdef USE_KTOBLZCHECK
# include <ktoblzcheck.h>
#endif



BankManager::BankInfo::BankInfo()
:_country(280){

}



BankManager::BankInfo::BankInfo(unsigned int country,
                                const string &code,
                                const string &name,
                                const string &location)
:_country(country)
,_bankCode(code)
,_bankName(name)
,_bankLocation(location) {
}



BankManager::BankInfo::~BankInfo(){
}





BankManager::BankManager(const string &datafile)
#ifdef USE_KTOBLZCHECK
:_checker(AccountNumberCheck(datafile))
#endif
{

}



BankManager::~BankManager(){
}



BANKMANAGER_CHECKRESULT BankManager::checkAccount(unsigned int country,
                                                  const string &bankCode,
                                                  const string &accountId){
#ifndef USE_KTOBLZCHECK
  return BankManagerCheckResultUnknown;
#else
  AccountNumberCheck::Result result;

  result=_checker.check(bankCode, accountId);
  switch(result) {
  case AccountNumberCheck::OK:
    return BankManagerCheckResultOk;
    break;
  case AccountNumberCheck::ERROR:
    return BankManagerCheckResultFailed;
    break;
  case AccountNumberCheck::BANK_NOT_KNOWN:
    return BankManagerCheckResultBankUnknown;
    break;
  default:
    return BankManagerCheckResultUnknown;
    break;
  }
#endif
}



BankManager::BankInfo
BankManager::getBankInfo(unsigned int country,
                         const string &bankCode) throw (Error){
#ifndef USE_KTOBLZCHECK
  throw Error("BankManager::getBankInfo",
              ERROR_LEVEL_NORMAL,
              HBCI_ERROR_CODE_UNSUPPORTED,
              ERROR_ADVISE_DONTKNOW,
              "method not supported",
              "please install ktoblzcheck");
#else
  AccountNumberCheck::Record bankData;
  BankInfo bi;
  bool found;

  if (country!=280) {
    throw Error("BankManager::getBankInfo",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_UNSUPPORTED,
                ERROR_ADVISE_DONTKNOW,
                "Bank not found"
                "for now use 280 for country");
  }
  found=false;
  try {
    bankData=_checker.findBank(bankCode);
    found=true;
  }
  catch (int i) {
    found=false;
  }

  if (!found)
    throw Error("BankManager::getBankInfo",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_UNSUPPORTED,
                ERROR_ADVISE_DONTKNOW,
                "Bank not found");
  return BankInfo(country,
                  bankCode,
                  bankData.bankName,
                  bankData.location);
#endif
}











#ifdef USE_KTOBLZCHECK
#endif






