/***************************************************************************
  $RCSfile: converterbin.cpp,v $
  -------------------
  cvs         : $Id: converterbin.cpp,v 1.3 2004/01/25 18:51:28 aquamaniac Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "converterbin.h"
#include <gwenhywfar/xml.h>
#include <gwenhywfar/db.h>
#include <gwenhywfar/buffer.h>
#include <gwenhywfar/bufferedio.h>
#include <assert.h>



AQMConverterBin::AQMConverterBin(AqMoneyAPI *api,
				 const string &name,
				 const string &subtype)
: AQMConverter()
,_api(api)
,_name(name)
,_subtype(subtype){
}


AQMConverterBin::~AQMConverterBin(){
}



Error AQMConverterBin::readFromStream(GWEN_BUFFEREDIO *bio){
  GWEN_DB_NODE *db;
  GWEN_XMLNODE *n;
  GWEN_BUFFER *gbuf;
  Pointer<ParserPlugin> p;
  GWEN_BUFFEREDIO *bbuf;
  char buffer[512];
  unsigned int bsize;
  GWEN_ERRORCODE gerr;
  int rv;

  p=_api->getParser(_name);
  if (!p.isValid()) {
    return Error("AQMConverterBin::readFromStream",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_DONTKNOW,
		 "no matching parser found for type",
		 _name);
  }

  // create buffer and bufferedio
  gbuf=GWEN_Buffer_new(0, 1024, 0, 1);
  GWEN_Buffer_SetMode(gbuf, GWEN_BUFFER_MODE_DYNAMIC);
  bbuf=GWEN_BufferedIO_Buffer_new(gbuf);
  GWEN_BufferedIO_SetWriteBuffer(bbuf, 0, 1024);

  // read data
  while(!GWEN_BufferedIO_CheckEOF(bio)) {
    bsize=sizeof(buffer);
    gerr=GWEN_BufferedIO_ReadRaw(bio, buffer, &bsize);
    if (!GWEN_Error_IsOk(gerr)) {
      DBG_INFO_ERR(0, gerr);
      GWEN_BufferedIO_Abandon(bbuf);
      GWEN_BufferedIO_free(bbuf);
      return Error("AQMConverterBin::readFromStream",
		   ERROR_LEVEL_NORMAL,
		   HBCI_ERROR_CODE_INVALID,
		   ERROR_ADVISE_DONTKNOW,
		   "error reading from stream");
    }
    gerr=GWEN_BufferedIO_WriteRaw(bbuf, buffer, &bsize);
    if (!GWEN_Error_IsOk(gerr)) {
      DBG_INFO_ERR(0, gerr);
      GWEN_BufferedIO_Abandon(bbuf);
      GWEN_BufferedIO_free(bbuf);
      return Error("AQMConverterBin::readFromStream",
		   ERROR_LEVEL_NORMAL,
		   HBCI_ERROR_CODE_INVALID,
		   ERROR_ADVISE_DONTKNOW,
                   "error writing to stream");
    }
  } /* while */
  GWEN_BufferedIO_Close(bbuf);

  // gbuf now contains the data
  DBG_NOTICE(0, "Read %d bytes", GWEN_Buffer_GetUsedBytes(gbuf));
  // setup data for parser
  db=GWEN_DB_Group_new("transactions");
  assert(db);
  n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "ELEM");
  GWEN_XMLNode_SetProperty(n, "type", "bin");
  GWEN_XMLNode_SetProperty(n, "bintype", _name.c_str());
  if (!_subtype.empty())
    GWEN_XMLNode_SetProperty(n, "binsubtype", _subtype.c_str());

  rv=p.ref().readBinType(n, db, gbuf);
  GWEN_BufferedIO_free(bbuf);
  GWEN_XMLNode_free(n);
  if (rv) {
    GWEN_DB_Group_free(db);
    return Error("AQMConverterBin::readFromStream",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_DONTKNOW,
		 "error in input stream");
  }
  setTransactions(db, true);
  return Error();
}



Error AQMConverterBin::writeToStream(GWEN_BUFFEREDIO *bio){
  GWEN_DB_NODE *db;
  GWEN_XMLNODE *n;
  GWEN_BUFFER *gbuf;
  Pointer<ParserPlugin> p;
  char buffer[512];
  unsigned int bsize;
  GWEN_ERRORCODE gerr;
  int rv;

  p=_api->getParser(_name);
  if (!p.isValid()) {
    return Error("AQMConverterBin::writeToStream",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_DONTKNOW,
		 "no matching parser found for type",
		 _name);
  }

  // create buffer and bufferedio
  gbuf=GWEN_Buffer_new(0, 1024, 0, 1);
  GWEN_Buffer_SetMode(gbuf, GWEN_BUFFER_MODE_DYNAMIC);

  // setup data for parser
  db=transactions();
  assert(db);
  n=GWEN_XMLNode_new(GWEN_XMLNodeTypeTag, "ELEM");
  GWEN_XMLNode_SetProperty(n, "type", "bin");
  GWEN_XMLNode_SetProperty(n, "bintype", _name.c_str());
  if (!_subtype.empty())
    GWEN_XMLNode_SetProperty(n, "binsubtype", _subtype.c_str());

  rv=p.ref().writeBinType(n, db, gbuf);
  GWEN_XMLNode_free(n);
  if (rv) {
    GWEN_DB_Group_free(db);
    return Error("AQMConverterBin::readFromStream",
		 ERROR_LEVEL_NORMAL,
		 HBCI_ERROR_CODE_INVALID,
		 ERROR_ADVISE_DONTKNOW,
		 "error in input stream");
  }

  GWEN_Buffer_Rewind(gbuf);

  // write data
  while(GWEN_Buffer_BytesLeft(gbuf)) {
    // read bytes
    bsize=sizeof(buffer);
    DBG_NOTICE(0, "Reading from buffer");
    if (GWEN_Buffer_ReadBytes(gbuf, buffer, &bsize)) {
      return Error("AQMConverterBin::writeToStream",
		   ERROR_LEVEL_NORMAL,
		   HBCI_ERROR_CODE_INVALID,
		   ERROR_ADVISE_DONTKNOW,
		   "error reading from buffer");
    }

    // write bytes
    DBG_NOTICE(0, "Writing to file (%d bytes)", bsize);
    gerr=GWEN_BufferedIO_WriteRaw(bio, buffer, &bsize);
    if (!GWEN_Error_IsOk(gerr)) {
      DBG_INFO_ERR(0, gerr);
      return Error("AQMConverterBin::writeToStream",
		   ERROR_LEVEL_NORMAL,
		   HBCI_ERROR_CODE_INVALID,
		   ERROR_ADVISE_DONTKNOW,
		   "error writing to stream");
    }
  } /* while */

  return Error();
}








