/***************************************************************************
  $RCSfile: converterold.cpp,v $
  -------------------
  cvs         : $Id: converterold.cpp,v 1.5 2004/01/25 18:51:29 aquamaniac Exp $
  begin       : Wed Oct 22 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "converterold.h"
#include "oldconf.h"
#include <assert.h>

#include <openhbci2/value.h>



AQMConverterOld::AQMConverterOld(const string &path)
:AQMConverter()
,_path(path){
}


AQMConverterOld::~AQMConverterOld(){
}



Error AQMConverterOld::readFromStream(GWEN_BUFFEREDIO *bio){
  GWEN_DB_NODE *n;
  CONFIGOLDGROUP *gr;
  CONFIGOLDGROUP *old;

  old=ConfigOld_new();
  if (ConfigOld_ReadStream(old, bio,
                           CONFIGOLDMODE_DEFAULT |
                           CONFIGOLDMODE_NAMECREATE_GROUP)) {
    return Error("AQMConverterOld::readFromStream",
                 ERROR_LEVEL_NORMAL,
                 HBCI_ERROR_CODE_INVALID,
                 ERROR_ADVISE_DONTKNOW,
                 "error in input stream");
  }
  n=GWEN_DB_Group_new("transactions");
  assert(n);

  // translate
  if (!_path.empty()) {
    DBG_NOTICE(0, "Selecting \"%s\"", _path.c_str());
    gr=ConfigOld_GetGroup(old, _path.c_str(), CONFIGOLDMODE_NAMEMUSTEXIST);
    if (gr)
      gr=gr->groups;
  }
  else
    gr=old->groups;
  while(gr) {
    DBG_NOTICE(0, "Group is %s", gr->name);
    if (strcasecmp(gr->name, "transaction")==0 ||
        strcasecmp(gr->name, "transfer")==0 ||
        strcasecmp(gr->name, "debitnote")==0 ||
        strcasecmp(gr->name, "debit_note")==0){
      string tmp;
      Value val;
      unsigned int i;
      GWEN_DB_NODE *nn;

      // found an old transaction, transform it
      if (strcasecmp(gr->name, "debit_note")==0 ||
          strcasecmp(gr->name, "debitnote")==0)
        nn=GWEN_DB_Group_new("debit_note");
      else
        nn=GWEN_DB_Group_new("transaction");
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "valutadate",
                           ConfigOld_GetValue(gr, "valutadate", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "date",
                           ConfigOld_GetValue(gr, "date", "", 0));
      val=Value(ConfigOld_GetValue(gr, "value", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "value/value", val.toString(true).c_str());
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "value/currency", val.getCurrency().c_str());
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "key",
                           ConfigOld_GetValue(gr, "key", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "custref",
                           ConfigOld_GetValue(gr, "customerreference","", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "code",
                           ConfigOld_GetValue(gr, "code", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "otheraccount/bankcode",
                           ConfigOld_GetValue(gr, "otherinstitute", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "otheraccount/accountid",
                           ConfigOld_GetValue(gr, "otherid", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "ouraccount/bankcode",
                           ConfigOld_GetValue(gr, "institute", "", 0));
      GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT,
                           "ouraccount/accountid",
                           ConfigOld_GetValue(gr, "id", "", 0));

      // purpose
      for (i=0; ; i++) {
        const char *p;

        p=ConfigOld_GetValue(gr, "description", 0, i);
        if (!p)
          break;
        GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT, "purpose", p);
      } // for
      if (!i) {
        return Error("AQMConverterOld::readFromStream",
                     ERROR_LEVEL_NORMAL,
                     HBCI_ERROR_CODE_INVALID,
                     ERROR_ADVISE_DONTKNOW,
                     "error in input stream",
                     "no purpose lines");
      }

      // otherName
      for (i=0; ; i++) {
        const char *p;

        p=ConfigOld_GetValue(gr, "othername", 0, i);
        if (!p)
          break;
        DBG_NOTICE(0, "Othername: %s (%d)", p, i);
        GWEN_DB_SetCharValue(nn, GWEN_DB_FLAGS_DEFAULT, "othername", p);
      } // for
      addTransaction(nn);
      GWEN_DB_Group_free(nn);
    } // if transaction found
    else {
      DBG_WARN(0, "Unknown section \"%s\"", gr->name);
    }
    gr=gr->next;
  } // while still transactions

  return Error();
}





