/***************************************************************************
  $RCSfile: storage.cpp,v $
  -------------------
  cvs         : $Id: storage.cpp,v 1.6 2004/01/25 18:51:26 aquamaniac Exp $
  begin       : Mon Oct 13 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "storage.h"
#include <gwenhywfar/text.h>




TransactionStorage::TransactionStorage(AqMoneyAPI *api,
                                       int country,
                                       const string &bankCode,
                                       const string &account)
:_api(api)
,_country(country)
,_bankCode(bankCode)
,_account(account){

}



TransactionStorage::TransactionStorage(AqMoneyAPI *api,
                                       GWEN_DB_NODE *db,
                                       int country,
                                       const string &bankCode,
                                       const string &account)
:_api(api)
,_country(country)
,_bankCode(bankCode)
,_account(account){
}



TransactionStorage::~TransactionStorage(){
}



GWEN_DB_NODE *TransactionStorage::getTransactions(int country,
                                                  const string &bankCode,
                                                  const string &account,
                                                  const Date &fromDate,
                                                  const Date &toDate,
                                                  unsigned int flags){
  int year, month, day;
  int nxa;
  Error err;
  GWEN_DB_NODE *xas;
  unsigned int fulldate;
  unsigned int fullthisdate;

  nxa=0;
  xas=GWEN_DB_Group_new("Transactions");
  err=getFirstDay(year, month, day);

  for (;;) {
    if (!err.isOk()) {
      if (!nxa) {
	DBG_ERROR(0, "No transactions (%s)", err.errorString().c_str());
	GWEN_DB_Group_free(xas);
	return 0;
      }
      return xas;
    }

    fullthisdate=year<<16;
    fullthisdate+=month<<8;
    fullthisdate+=day;

    // we have a day, check range
    if (fromDate.isValid()) {
      fulldate=fromDate.year()<<16;
      fulldate+=fromDate.month()<<8;
      fulldate+=fromDate.day();
    }
    if (!fromDate.isValid() ||
        fulldate<=fullthisdate) {
      DBG_INFO(0, "Date %04d/%02d/%02d matches fromdate",
               year, month, day);
      // fromDate matches
      if (fromDate.isValid()) {
        fulldate=toDate.year()<<16;
        fulldate+=toDate.month()<<8;
        fulldate+=toDate.day();
      }
      if (!toDate.isValid() ||
          fulldate>=fullthisdate) {
        unsigned int hdl;
        GWEN_DB_NODE *newXA;

	DBG_INFO(0, "Date %04d/%02d/%02d matches todate",
		 year, month, day);
	// toDate matches
	try {

	  DBG_INFO(0, "Found matching day %04d/%02d%d02",
                   year, month, day);
	  hdl=openDay(year, month, day, false);
	  newXA=getFirstTransaction(hdl);
	  for (;;) {
	    Value val;
            bool matches;

	    matches=true;
	    if (!newXA)
	      break;

            // check value
            val=Value(GWEN_DB_GetCharValue(newXA, "value/value", 0, "0,"));
	    if (val.getValue()<0.0) {
              if (!(flags & AQM_TRANSACTION_MATCH_NEGATIVE)) {
                DBG_INFO(0, "negative value doesnt match");
                matches=false;
              }
	    }
	    else {
              if (!(flags & AQM_TRANSACTION_MATCH_POSITIVE)) {
                DBG_INFO(0, "positive value doesnt match");
                matches=false;
              }
	    }

	    // check other institute
	    if (-1==
		GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(newXA,
							      "otherBankCode",
                                                              0,
							      ""),
					 bankCode.c_str(), 0)){
	      DBG_INFO(0, "otherBankCode doesnt match");
	      matches=false;
	    }

	    // check other country
	    if (country && country!=GWEN_DB_GetIntValue(newXA,
							"otherCountryCode",
							0,
							280)) {
	      DBG_INFO(0, "otherCountry doesnt match");
	      matches=false;
	    }

	    // check other account
	    if (-1==
		GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(newXA,
							      "otherAccount",
							      0,
							      ""),
					 account.c_str(), 0)) {
	      DBG_INFO(0, "otherAccount doesnt match");
	      matches=false;
	    }

	    if (matches) {
              // add transaction
	      GWEN_DB_NODE *nuTarget;

              DBG_INFO(0, "Transaction matches");
	      nuTarget=GWEN_DB_GetGroup(xas,
                                        GWEN_DB_FLAGS_DEFAULT |
                                        GWEN_PATH_FLAGS_CREATE_GROUP,
					"transaction");
	      GWEN_DB_AddGroupChildren(nuTarget,
                                       newXA);
	      nxa++;
            }
            else {
              DBG_INFO(0, "Transaction does not match");
            }
	    GWEN_DB_Group_free(newXA);

	    newXA=getNextTransaction(hdl);
	  } // for transaction

	  // close day
	  err=closeDay(hdl);
	  if (!err.isOk()) {
	    DBG_ERROR(0, "Error: %s", err.errorString().c_str());
	    GWEN_DB_Group_free(xas);
	    return 0;
	  }
	} // try
	catch (Error xerr) {
	  DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
	  GWEN_DB_Group_free(xas);
	  return 0;
	}

      } // if toDate matches
    } // if fromdate matches
    err=getNextDay(year, month, day);
  } // for

  return xas;
}



Error TransactionStorage::getLastDay(int &year, int &month, int &day){
  int lyear, lmonth, lday;
  int syear, smonth, sday;
  Error err;

  err=getFirstDay(lyear, lmonth, lday);
  if (!err.isOk())
    return Error("TransactionStorage::getLastDay", err);

  while(err.isOk()) {
    syear=lyear;
    smonth=lmonth;
    sday=lday;
    err=getNextDay(lyear, lmonth, lday);
  } // while

  year=syear;
  month=smonth;
  day=sday;
  return Error();
}









