/***************************************************************************
  $RCSfile: storage.h,v $
  -------------------
  cvs         : $Id: storage.h,v 1.7 2004/01/25 18:51:27 aquamaniac Exp $
  begin       : Mon Oct 13 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifndef AQMONEY_STORAGE_H
#define AQMONEY_STORAGE_H

class TransactionStorage;

#include <openhbci2.h>
#include <gwenhywfar/db.h>

#include <aqmoney2/aqmapi.h>


#include <string>
using namespace std;
using namespace HBCI;

class TransactionStorage {
private:
  AqMoneyAPI *_api;
  int _country;
  string _bankCode;
  string _account;

protected:
  virtual Error fromConfig(GWEN_DB_NODE *db)=0;

public:
  TransactionStorage(AqMoneyAPI *api,
                     int country=280,
                     const string &bankCode="",
                     const string &account="");
  TransactionStorage(AqMoneyAPI *api,
                     GWEN_DB_NODE *db,
                     int country=280,
                     const string &bankCode="",
                     const string &account="");
  virtual ~TransactionStorage();

  virtual Error toConfig(GWEN_DB_NODE *db)=0;

  AqMoneyAPI *api() { return _api;};

  int country() const { return _country;};
  const string &bankCode() const { return _bankCode;};
  const string &account() const { return _account;};

  virtual Error connect()=0;
  virtual Error disconnect()=0;
  virtual bool connected()=0;

  virtual Error clear()=0;
  virtual unsigned int openDay(int year,
                               int month,
                               int day,
                               bool write) throw(Error)=0;
  virtual Error clearDay(unsigned int hdl)=0;
  virtual Error closeDay(unsigned int hdl)=0;
  virtual Error abandonDay(unsigned int hdl)=0;

  virtual Error addTransaction(unsigned int hdl, GWEN_DB_NODE *xa)=0;
  virtual Error delTransaction(unsigned int hdl, GWEN_DB_NODE *xa)=0;
  virtual Error setOpeningBalance(unsigned int hdl, GWEN_DB_NODE *bal)=0;
  virtual Error setClosingBalance(unsigned int hdl, GWEN_DB_NODE *bal)=0;
  virtual GWEN_DB_NODE *getOpeningBalance(unsigned int hdl) throw(Error)=0;
  virtual GWEN_DB_NODE *getClosingBalance(unsigned int hdl) throw(Error)=0;

  /**
   * Returns a DB group containing the first transaction of the day.
   * The caller is responsible for freeing the transaction data by means
   * of calling GWEN_DB_Group_free() on the returned data.
   */
  virtual GWEN_DB_NODE *getFirstTransaction(unsigned int hdl)=0;

  /**
   * Returns a DB group containing the next transaction of the day.
   * The caller is responsible for freeing the transaction data by means
   * of calling GWEN_DB_Group_free() on the returned data.
   */
  virtual GWEN_DB_NODE *getNextTransaction(unsigned int hdl)=0;


  virtual Error getFirstDay(int &year, int &month, int &day)=0;
  virtual Error getNextDay(int &year, int &month, int &day)=0;

  /**
   * This is a conveniance function which checks for the last day for which
   * there are transactions.
   */
  virtual Error getLastDay(int &year, int &month, int &day);


  /**
   * Returns a list of transactions which match the given criteria.
   */
  GWEN_DB_NODE *getTransactions(int country,
                                const string &bankCode,
                                const string &account,
                                const Date &fromDate,
                                const Date &toDate,
                                unsigned int flags);


};


#endif




