/***************************************************************************
  $RCSfile: storagefile.h,v $
  -------------------
  cvs         : $Id: storagefile.h,v 1.3 2004/01/25 18:51:41 aquamaniac Exp $
  begin       : Mon Oct 13 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifndef AQMONEY_STORAGE_FILE_H
#define AQMONEY_STORAGE_FILE_H

#define STORAGEFILE_TYPENAME "FILE"

#include "storage.h"



/**
 * This implementation of a TransactionStorage uses the filesystem
 * to store transactions.
 * The following directory hierarcy will be created below the given
 * directory:
 * COUNTRY/BANKCODE/ACCOUNT/YEAR/MONTH/YEAR-MONTH-DAY.trans
 * where COUNTRY is the country code, BANKCODE is the bank code (BLZ)
 * ACCOUNT is the account id (Kontonummer) and so on.
 * For every day of a month exactly ONE file will be used.
 * This file is read/written upon openDay/closeDay, respectively.
 * @author Martin Preuss<martin@libchipcard.de>
 */
class TransactionStorageFile: public TransactionStorage {
private:
  class OpenDay{
  private:
    int _year;
    int _month;
    int _day;
    string _path;
    unsigned int _handle;
    GWEN_DB_NODE *_data;
    bool _readOnly;
    bool _changed;
    GWEN_DB_NODE *_currTrans;
  public:
    OpenDay(int year, int month, int day,
            const string &path,
            unsigned int hdl,
            GWEN_DB_NODE *data,
            bool readonly);
    ~OpenDay();
    int year() const { return _year;};
    int month() const { return _month;};
    int day() const { return _day;};
    unsigned int handle() const { return _handle;};
    GWEN_DB_NODE *data() { return _data;};
    const string &path() const { return _path;};
    void setChanged(bool c) { _changed=c;};
    bool changed() const { return _changed;};
    bool readOnly() const { return _readOnly;};
    GWEN_DB_NODE *getFirstTransaction();
    GWEN_DB_NODE *getNextTransaction();

  };

  string _dir;
  string _subdir;
  list<Pointer<OpenDay> > _openDays;
  list<unsigned int> _availDays;
  list<unsigned int>::iterator _availDaysIt;
  bool _connected;
  static unsigned int _nextHandle;

  string _getPath(int year, int month, int day, bool create);
  Pointer<OpenDay> _findOpenDay(unsigned int hdl);
  Error _buildIndex();
  static void *_handlePathElement(const char *entry,
                                  void *data,
                                  unsigned int flags);

  bool _cmpCharValue(GWEN_DB_NODE *xa,
                     GWEN_DB_NODE *nn,
                     const char *name,
                     unsigned int idx);
  bool _cmpIntValue(GWEN_DB_NODE *xa,
                    GWEN_DB_NODE *nn,
                    const char *name,
                    unsigned int idx);
  GWEN_DB_NODE *_findTransaction(Pointer<OpenDay> d,
                                 GWEN_DB_NODE *xa);

protected:
  virtual Error fromConfig(GWEN_DB_NODE *db);

public:
  TransactionStorageFile(AqMoneyAPI *api,
                         const string &dir,
                         const string &subDir,
                         int country=280,
                         const string &bankCode="",
                         const string &account="");
  TransactionStorageFile(AqMoneyAPI *api,
                         GWEN_DB_NODE *db,
                         int country=280,
                         const string &bankCode="",
                         const string &account="") throw (Error);
  virtual ~TransactionStorageFile();

  virtual Error toConfig(GWEN_DB_NODE *db);

  virtual Error connect();
  virtual Error disconnect();
  virtual bool connected();

  virtual Error clear();
  virtual unsigned int openDay(int year,
                               int month,
                               int day,
                               bool write) throw(Error);
  virtual Error clearDay(unsigned int hdl);
  virtual Error closeDay(unsigned int hdl);
  virtual Error abandonDay(unsigned int hdl);

  virtual Error addTransaction(unsigned int hdl, GWEN_DB_NODE *xa);
  virtual Error delTransaction(unsigned int hdl, GWEN_DB_NODE *xa);
  virtual Error setOpeningBalance(unsigned int hdl, GWEN_DB_NODE *bal);
  virtual GWEN_DB_NODE *getOpeningBalance(unsigned int hdl) throw(Error);
  virtual GWEN_DB_NODE *getClosingBalance(unsigned int hdl) throw(Error);
  virtual Error setClosingBalance(unsigned int hdl, GWEN_DB_NODE *bal);
  virtual GWEN_DB_NODE *getFirstTransaction(unsigned int hdl);
  virtual GWEN_DB_NODE *getNextTransaction(unsigned int hdl);


  virtual Error getFirstDay(int &year, int &month, int &day);
  virtual Error getNextDay(int &year, int &month, int &day);


};


#endif




