/***************************************************************************
  $RCSfile: main.cpp,v $
  -------------------
  cvs         : $Id: main.cpp,v 1.24 2004/02/29 21:30:36 rbla Exp $
  begin       : Tue Oct 14 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/




#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif

#include "aqmapi.h"
#include "aqmmodule.h"
#include "aqmauth.h"
#include "aqminteractor.h"

// modules, add here the includes for new modules
#include "modules/getbalance.h"
#include "modules/gettransactions.h"
#include "modules/createuser.h"
#include "modules/getaccounts.h"
#include "modules/transfer.h"
#include "modules/getkeys.h"
#include "modules/sendkeys.h"
#include "modules/createkeys.h"
#include "modules/getsystemid.h"
#include "modules/update.h"
#include "modules/dump.h"
#include "modules/convert.h"
#include "modules/debitnote.h"
#include "modules/mtransfer.h"
#include "modules/mdebitnote.h"
#include "modules/texport.h"
#include "modules/version.h"
#include "modules/mkpinfile.h"
#include "modules/getdatedtransfers.h"
#include "modules/deletedatedtransfer.h"
#include "modules/changedatedtransfer.h"
#include "modules/getstandingorders.h"
#include "modules/startsto.h"
#include "modules/delsto.h"
#include "modules/addaccount.h"
#include "modules/accupdate.h"
#include "modules/iniletter.h"


#include <gwenhywfar/directory.h>
#include <gwenhywfar/logger.h>
#include <string>

using namespace std;
using namespace HBCI;


list<Pointer<AqmoneyModule> > modules;


#define k_PRG "aqmoney2"
#define k_PRG_VERSION_INFO \
  "AqMoney-TNG "AQMONEY_VERSION_FULL_STRING"\n"\
  "Copyright (C) 2003-2004 Martin Preuss <martin@libchipcard.de>\n" \
  "This program is free software licensed under the GPL.\n"\
  "See the file COPYING for details.\n"


/**
 * This function simply adds all modules to the list of modules.
 * So this is the only place where you need to add new modules if you
 * create them.
 */
void enumerateModules() {
  Pointer<AqmoneyModule> m;

  m=new ModuleCreateUser();
  modules.push_back(m);

  m=new ModuleCreateKeys();
  modules.push_back(m);

  m=new ModuleGetKeys();
  modules.push_back(m);

  m=new ModuleSendKeys();
  modules.push_back(m);

  m=new ModuleIniLetter();
  modules.push_back(m);

  m=new ModuleGetSystemId();
  modules.push_back(m);

  m=new ModuleGetAccounts();
  modules.push_back(m);

  m=new ModuleAddAccount();
  modules.push_back(m);

  m=new ModuleUpdate();
  modules.push_back(m);

  m=new ModuleAccountUpdate();
  modules.push_back(m);

  m=new ModuleGetBalance();
  modules.push_back(m);

  m=new ModuleTransfer();
  modules.push_back(m);

  m=new ModuleMultiTransfer();
  modules.push_back(m);

  m=new ModuleDebitNote();
  modules.push_back(m);

  m=new ModuleMultiDebitNote();
  modules.push_back(m);

  m=new ModuleGetTransactions();
  modules.push_back(m);

  m=new ModuleGetStandingOrders();
  modules.push_back(m);

  m=new ModuleStartStandingOrder();
  modules.push_back(m);

  m=new ModuleDeleteStandingOrder();
  modules.push_back(m);

  m=new ModuleDeleteDatedTransfer();
  modules.push_back(m);

  m=new ModuleChangeDatedTransfer();
  modules.push_back(m);

  m=new ModuleGetDatedTransfers();
  modules.push_back(m);

  m=new ModuleExport();
  modules.push_back(m);

  m=new ModuleDump();
  modules.push_back(m);

  m=new ModuleConvert();
  modules.push_back(m);

  m=new ModuleMkPinFile();
  modules.push_back(m);

  m=new ModuleVersion();
  modules.push_back(m);

}


void usage(const string &name) {
  list<Pointer<AqmoneyModule> >::iterator mit;

  fprintf(stdout,"%s%s%s%s",
          I18N("AqMoney-TNG - A console homebanking tool\n"
               "Copyright (C) 2003-2004 Martin Preuss <martin@libchipcard.de>\n"
               "This program is free software; you can redistribute it and/or modify\n"
               "it under the terms of the GNU General Public License as published by\n"
               "the Free Software Foundation; either version 2 of the License, or\n"
               "(at your option) any later version.\n\n"),
          I18N("Usage\n"),
          name.c_str(),
          I18N(" [global options] COMMAND [command specific options]\n\n"
               "Commands:\n"));
  for (mit=modules.begin(); mit!=modules.end(); mit++) {
    fprintf(stdout, " %s:\n   ", (*mit).ref().name().c_str());
    (*mit).ref().dumpDescription(stdout);
    fprintf(stdout, "\n");
  }
  fprintf(stdout, "\n");
  fprintf(stdout,"%s%s%s",
          I18N("Global Options\n"
               " [-h]                         show this help\n"
               " [-fh]                        show helps screens of ALL commands\n"
               " [-V]                         show program version\n"
               " [-v]                         increase verbosity level\n"
               " [-r]                         start in read-only mode\n"
               " [-ni]                        non-interactive mode\n"
               "                                in this mode AqMoney will omit some\n"
               "                                interaction\n"
               " [-C CONFIGFILE]              configuration file to be used for OpenHBCI\n"
               " [-D CONFIGFILE]              configuration file to be used for AqMoney\n"
               " [-p PINFILE]                 file containing PINs\n"
               " [--logfile FILE]             use given FILE as log file\n"
               " [--logtype TYPE]             use given TYPE as log type\n"
               "                                These are the valid types:\n"
               "                                stderr (log to standard error channel)\n"
               "                                file   (log to the file given by --logfile)\n"),
#ifdef HAVE_SYSLOG_H
          I18N("                                syslog (log via syslog)\n"),
#else
               "",
#endif
          I18N("                                Default is \"stderr\"\n"
               " [--loglevel LEVEL]           set the loglevel\n"
               "                                Valid levels are:\n"
               "                                emergency, alert, critical, error,\n"
               "                                warning, notice, info and debug\n"
               "                                Default is \"warning\".\n"
               "\n"
               "Most commands allow the command specific option \"-h\"\n"
               "which shows a help screen specific to that command\n"
               "\n"
               "Please note: Options BEFORE the COMMAND are always global\n"
               "options, options behind COMMAND are always command specific!\n"));
}



void dumpAllDescriptions() {
  list<Pointer<AqmoneyModule> >::iterator it;

  for (it=modules.begin(); it!=modules.end(); it++) {
    fprintf(stderr,
	    "================================================================================\n");
    fprintf(stderr, " %s:\n   ", (*it).ref().name().c_str());
    (*it).ref().dumpDescription(stderr);
    fprintf(stderr, "\n");
    (*it).ref().usage();
    fprintf(stderr, "\n");
  }
}


struct s_args {
  string openhbciFile;         // -C
  string aqmoneyFile;          // -D
  string pinFile;              // -p
  string logFile;              // --logfile
  GWEN_LOGGER_LOGTYPE logType; // --logtype
  GWEN_LOGGER_LEVEL logLevel;  // --loglevel
  bool readOnly;               // -r
  int verbosity;               // -v
  bool noInteraction;          // -ni
};



/**
 * Checks all arguments until a non-option element is encountered or until
 * the end is reached.
 * @return -1 on error, -2 if program should exit gracefully, other values
 * indicate the argument where this function stopped.
 */
int checkArgs(s_args &args, int argc, char **argv) {
  int i;
  string tmp;
  char homebuffer[256];

  // create default path
  if (GWEN_Directory_GetHomeDirectory(homebuffer, sizeof(homebuffer))) {
    DBG_ERROR(0, "Buffer for home path too small");
    return -1;
  }
  i=1;
  args.openhbciFile=homebuffer;
  args.openhbciFile+="/openhbci-tng.conf";
  args.aqmoneyFile=homebuffer;
  args.aqmoneyFile+="/aqmoney-tng.conf";
  args.logFile="aqmoney2.log";
  args.logType=GWEN_LoggerTypeConsole;
  args.logLevel=GWEN_LoggerLevelWarning;
  args.readOnly=false;
  args.noInteraction=false;
  args.verbosity=0;

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty argument\n"));
      return -1;
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.openhbciFile=argv[i];
    }
    else if (tmp=="-D") {
      i++;
      if (i>=argc)
        return -1;
      args.aqmoneyFile=argv[i];
    }
    else if (tmp=="-p") {
      i++;
      if (i>=argc)
        return -1;
      args.pinFile=argv[i];
    }
    else if (tmp=="-ni") {
      args.noInteraction=true;
    }
    else if (tmp=="-r") {
      args.readOnly=true;
    }
    else if (tmp=="-v") {
      args.verbosity++;
    }
    else if (tmp=="--logfile") {
      i++;
      if (i>=argc)
	return -1;
      args.logFile=argv[i];
    }

    else if (tmp=="--logtype") {
      i++;
      if (i>=argc)
	return -1;
      if (strcasecmp(argv[i], "stderr")==0)
        args.logType=GWEN_LoggerTypeConsole;
      else if (strcasecmp(argv[i], "file")==0)
	args.logType=GWEN_LoggerTypeFile;
#ifdef HAVE_SYSLOG_H
      else if (strcasecmp(argv[i], "syslog")==0)
	args.logType=GWEN_LoggerTypeSyslog;
#endif
      else {
	fprintf(stderr,I18N("Unknown log type \"%s\"\n"),
		argv[i]);
	return -1;
      }
    }

    else if (tmp=="--loglevel") {
      i++;
      if (i>=argc)
	return -1;
      if (strcmp(argv[i], "emergency")==0)
	args.logLevel=GWEN_LoggerLevelEmergency;
      else if (strcmp(argv[i], "alert")==0)
	args.logLevel=GWEN_LoggerLevelAlert;
      else if (strcmp(argv[i], "critical")==0)
	args.logLevel=GWEN_LoggerLevelCritical;
      else if (strcmp(argv[i], "error")==0)
	args.logLevel=GWEN_LoggerLevelError;
      else if (strcmp(argv[i], "warning")==0)
	args.logLevel=GWEN_LoggerLevelWarning;
      else if (strcmp(argv[i], "notice")==0)
	args.logLevel=GWEN_LoggerLevelNotice;
      else if (strcmp(argv[i], "info")==0)
	args.logLevel=GWEN_LoggerLevelInfo;
      else if (strcmp(argv[i], "debug")==0)
	args.logLevel=GWEN_LoggerLevelDebug;
      else {
	fprintf(stderr,
		I18N("Unknown log level \"%s\"\n"),
		argv[i]);
	return -1;
      }
    }

    else if (tmp=="-h" || tmp=="--help") {
      usage(argv[0]);
      return -2;
    }
    else if (tmp=="-fh") {
      dumpAllDescriptions();
      return -2;
    }
    else if (tmp=="-V" || tmp=="--version") {
      fprintf(stdout,k_PRG_VERSION_INFO);
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr,
                I18N("Option \"%s\" is not known to be a global option"
                     " (maybe it belongs to the command ?\n"),
                tmp.c_str());
        return -1;
      }
      // otherwise break
      break;
    }
    i++;
  } // while
  // that's it
  return i;
}



int main(int argc, char **argv) {
  s_args args;
  int rv;
  ModuleError merr;
  string cmd;
  Pointer<AqMoneyAPI> api;
  list<Pointer<AqmoneyModule> >::iterator mit;
  Pointer<AQMInteractor> interactor;
  Pointer<AqMoneyAuth> auth;
  char numbuffer[16];

#ifdef ENABLE_NLS
  setlocale(LC_ALL,"");
  if (bindtextdomain("aqmoney2",  LOCALEDIR)==0) {
    fprintf(stderr," Error bindtextdomain()\n");
  }
  if (textdomain("aqmoney2")==0) {
    fprintf(stderr," Error textdomain()\n");
  }
#endif

  enumerateModules();
  rv=checkArgs(args,argc,argv);
  if (rv==-2)
    return 0;
  else if (rv==-1)
    return 1;

  if (rv>=argc) {
    fprintf(stderr, I18N("Error: No command given\n"));
    usage(argv[0]);
    return 1;
  }
  api=new AqMoneyAPI(args.readOnly);
  if (GWEN_Logger_Open(0,
                       "aqmoney2",
                       args.logFile.c_str(),
                       args.logType,
                       GWEN_LoggerFacilityUser)) {
    fprintf(stderr,I18N("Could not start logging, aborting.\n"));
    return 2;
  }
  GWEN_Logger_SetLevel(0, args.logLevel);

  snprintf(numbuffer, sizeof(numbuffer), "%d", AQMONEY_VERSION_MAJOR);
  api.ref().setSystemName("AqMoney-TNG");
  api.ref().setSystemVersion(numbuffer);
  api.ref().setApplicationName("AqMoney2 v"AQMONEY_VERSION_FULL_STRING);
  interactor=new AQMInteractor(args.noInteraction);
  auth=new AqMoneyAuth(api.ptr());
  if (!args.pinFile.empty()) {
    Error err;

    err=auth.ref().readPinList(args.pinFile);
    if (!err.isOk()) {
      return 1;
    }
  }
  api.ref().setInteractor(interactor.cast<Interactor>());
  api.ref().setAuthentificator(auth.cast<Auth>());

  api.ref().setDebugLevel(args.verbosity);
  for (mit=modules.begin(); mit!=modules.end(); mit++) {
    if (strcasecmp((*mit).ref().name().c_str(), argv[rv])==0) {
      // found module
      merr=(*mit).ref().run(api.ptr(),
                            argc-rv-1, &(argv[rv+1]),
                            args.openhbciFile,
                            args.aqmoneyFile);
      GWEN_Logger_Close(0);
      return merr;
    }
  } // for
  fprintf(stderr, "%s \"%s\"\n",
          I18N("Unknown command"),
          argv[rv]);
  GWEN_Logger_Close(0);
  return 1;
}






