/***************************************************************************
  $RCSfile: addaccount.cpp,v $
  -------------------
  cvs         : $Id: addaccount.cpp,v 1.18 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "adminjobs.h"
#include "addaccount.h"
#include "accountjobs.h"

#include <gwenhywfar/debug.h>


ModuleAddAccount::ModuleAddAccount()
:AqmoneyModule() {
}



ModuleAddAccount::~ModuleAddAccount(){
}



void ModuleAddAccount::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Manually adds an account. You will only need this\n"
               "   if the command \"acclist\" fails\n"));
}


void ModuleAddAccount::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 accadd [-b %s] [-c %s] [-C %s]\n"
                  "%s   -a %s [-as %s] [-A %s]\n"
                  "%s   [-B %s] -n %s\n",
                  I18N("Usage:"),
                  I18N("BANKCODE"), I18N("COUNTRYCODE"), I18N("CUSTOMERID"),
                  I18N("      "),
                  I18N("ACCOUNTID"), I18N("ACCOUNTSUBID"), I18N("ACCOUNTNAME"),
                  I18N("      "), I18N("BANKCODE"), I18N("NAME"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleAddAccount::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.serverBankCode="*";
  args.customerId="*";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.serverBankCode=argv[i];
    }
    else if (tmp=="-B") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.customerId=argv[i];
    }
    else if (tmp=="-n") {
      i++;
      if (i>=argc)
	return -1;
      args.name=argv[i];
    }
    else if (tmp=="-a") {
      i++;
      if (i>=argc)
        return -1;
      args.accountId=argv[i];
    }
    else if (tmp=="-as") {
      i++;
      if (i>=argc)
        return -1;
      args.accountSubId=argv[i];
    }
    else if (tmp=="-A") {
      i++;
      if (i>=argc)
        return -1;
      args.accountName=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
	fprintf(stderr,
		I18N("Option \"%s\" is unknown to this command\n"),
		tmp.c_str());
	return -1;
      }
      fprintf(stderr,
	      I18N("Argument \"%s\" is not an option to this command\n"),
	      tmp.c_str());
      if (!args.name.empty()) {
	fprintf(stderr,
		I18N("Maybe your forgot to escape the name with\n"
		     "quotation marks?\n"));
      }
      return -1;
    }
    i++;
  } // while
  // that's it
  // some checks
  if (args.name.empty()) {
    fprintf(stderr,
	    I18N("Please specify your name with -n\n"));
    return -1;
  }
  if (args.accountId.empty()) {
    fprintf(stderr,
	    I18N("Please specify your account id with -a\n"));
    return -1;
  }
  return 0;
}




ModuleError ModuleAddAccount::run(AqMoneyAPI *api,
                                  int argc, char **argv,
                                  const string &openhbciFile,
                                  const string &aqmoneyFile){
  int rv;
  Error err;
  list<Pointer<Customer> > custs;
  Pointer<Customer> cust;
  Pointer<Bank> bank;
  Pointer<Account> account;
  string bankCode;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }
  custs=api->getCustomers(_args.country,
                          _args.serverBankCode,
                          _args.customerId);
  if (custs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("No matching customers found.\n"));
    return ModuleErrorInvalidArgs;
  }

  if (custs.size()>1) {
    fprintf(stderr,
	    "%s",
	    I18N("Ambiguous customer specification.\n"));
    return ModuleErrorInvalidArgs;
  }

  try {
    cust=custs.front();

    bank=cust.ref().user().ref().bank();
    bankCode=_args.bankCode.empty() ? bank.ref().bankCode() : _args.bankCode;
    account=
        api->accountFactory(cust.ref().custId(),
                            _args.name,
                            _args.accountId,
                            _args.accountSubId,
                            bank.ref().country(),
                            bankCode,
                            bank.ref().bankCode(),
                            _args.accountName);
    err=api->addAccount(account);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error: %s", err.errorString().c_str());
      return ModuleErrorExecution;
    }

    fprintf(stdout,
	    I18N("Added account %d/%s/%s\n"),
	    bank.ref().country(),
	    _args.bankCode.c_str(),
	    _args.accountId.c_str());

    DBG_INFO(0, "Saving environment");
    err=api->saveEnvironment(openhbciFile, aqmoneyFile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return ModuleErrorSave;
    }
    DBG_INFO(0, "Saving environment done.");
  } // try
  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  return ModuleErrorSuccess;
}









