/***************************************************************************
  $RCSfile: createuser.cpp,v $
  -------------------
  cvs         : $Id: createuser.cpp,v 1.14 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "createuser.h"
#include "accountjobs.h"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>


ModuleCreateUser::ModuleCreateUser()
:AqmoneyModule() {
}



ModuleCreateUser::~ModuleCreateUser(){
}



void ModuleCreateUser::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Creates a user\n"));
}


void ModuleCreateUser::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 createuser -t file -m %s -s %s -i\n",
                  I18N("Usage:"), I18N("MEDIUM"), I18N("SERVER"));
  fprintf(stderr, "%s aqmoney2 createuser -t file -m %s -s %s\n"
                  "%s   -b %s -u %s [-c %s] [-C %s]\n",
                  //TRANSLATORS: this has the same length as "Usage:"
                  I18N("      "), I18N("MEDIUM"), I18N("SERVER"),
                  I18N("      "), I18N("BANKCODE"),
                  I18N("USERID"), I18N("COUNTRYCODE"), I18N("CUSTOMERID"));
  fprintf(stderr, "%s aqmoney2 createuser -t card [-m %s] [-s %s] -i\n",
                  I18N("      "), I18N("MEDIUM"), I18N("SERVER"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleCreateUser::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=280;
  args.import=false;
  args.readonly=false;

  if (argc==0)
    return -1;

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-u") {
      i++;
      if (i>=argc)
        return -1;
      args.user=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.customer=argv[i];
    }
    else if (tmp=="-s") {
      i++;
      if (i>=argc)
        return -1;
      args.server=argv[i];
    }
    else if (tmp=="-t") {
      i++;
      if (i>=argc)
        return -1;
      args.mediumtype=argv[i];
    }
    else if (tmp=="-m") {
      i++;
      if (i>=argc)
        return -1;
      args.mediumname=argv[i];
    }
    else if (tmp=="-i") {
      args.import=true;
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it
  return 0;
}



bool ModuleCreateUser::_checkMedium(AqMoneyAPI *api,
                                    bool isFile,
                                    string &mediumName,
                                    string &mediumTypeName) {
  Pointer<MediumPluginList> mediumPlugins;
  list<Pointer<MediumPlugin> >::const_iterator it;
  HBCI::Error err;

  fprintf(stderr,
          I18N("Checking for plugin which supports this medium,"
               "please wait (%s).\n"),
          mediumName.c_str());

  mediumPlugins=api->enumerateMediumPlugins();
  if (!mediumPlugins.isValid()) {
    fprintf(stderr,
            I18N("Error listing installed plugins.\n"
                 "Please report this incidence to:\n"
                 "martin@libchipcard.de\n"));
    return false;
  }
  if (mediumPlugins.ref().plugins().empty()) {
    fprintf(stderr,
            I18N("There are no plugins installed.\n"
                 "At least one plugin is shipped with "
                 "OpenHBCI-TNG,\n"
                 "so I guess OpenHBCI-TNG is not properly "
                 "installed.\n"
                 "You can get OpenHBCI-TNG along with plugins from:\n"
                 "http://www.openhbci.de\n"));
    return false;
  }

  for (it=mediumPlugins.ref().plugins().begin();
       it!=mediumPlugins.ref().plugins().end();
       it++) {
    fprintf(stderr,
            I18N("Checking plugin \"%s\" (%d, %d)\n"),
            (*it).ref().pluginName().c_str(), isFile,
            (*it).ref().mediumType());
    if (!(((*it).ref().mediumType()==MediumTypeFile) ^ isFile)) {
      // medium types are equal, let the plugin if this is his medium
      //if (api->debugLevel()>2)
	fprintf(stderr,
                I18N("Checking whether plugin \"%s\" "
                     "supports this medium \n"),
                (*it).ref().pluginName().c_str());
      err=(*it).ref().mediumCheck(mediumName);
      if (!err.isOk()) {
	// error, now check what kind of
	if (err.code()==HBCI_ERROR_CODE_BAD_MEDIUM) {
	  // this plugin definitely does NOT support this medium, try next
          //if (api->debugLevel()>0)
	    fprintf(stderr,
                    I18N("Medium not supported by plugin \"%s\"\n"),
                    (*it).ref().pluginName().c_str());
          continue;
	}
	if (err.code()==HBCI_ERROR_CODE_WRONG_MEDIUM) {
	  // this plugin supports this medium, however the names do not match
	  fprintf(stderr,
                  I18N("Found a plugin which supports this medium.\n"
                       "However, the name does not match.\n"
                       "Please specify the correct name "
                       "or leave it blank.\n"));
          mediumTypeName=(*it).ref().pluginName();
          return false;
        }
	else {
	  // more severe error
	  fprintf(stderr, I18N("Error: %s\n"),
                  err.errorString().c_str());
	  return false;
	}
      } // if err.isOk()
      else {
	// medium plugin found
	fprintf(stderr,
                I18N("Medium supported by plugin \"%s\"\n"),
                (*it).ref().pluginName().c_str());
	mediumTypeName=(*it).ref().pluginName();
	return true;
      }
    } // if mediumtype matches
  } // for

  fprintf(stderr,
          I18N("No plugin seems to support this medium. Sorry.\n"));
  mediumTypeName.erase();
  return false;
}



Error ModuleCreateUser::_reallyCreateUser(AqMoneyAPI *api,
                                          const string &mediumname,
                                          const string &mediumType,
                                          bool readonly,
                                          int country,
                                          const string &instid,
                                          const string &userid,
                                          const string &customerid,
                                          const string &server) {
  Pointer<Bank> bank;
  Pointer<User> user;
  Pointer<Customer> cust;
  Pointer<Medium> medium;
  Error err;
  string ucid;

  // create medium
  fprintf(stderr,I18N(
	  "Creating this user:\n"
	  "  Bank code: %s\n"
	  "  User Id  : %s\n"
	  "  Server   : %s\n"),
	  instid.c_str(),userid.c_str(),server.c_str());
  if (country==0 ||
      instid.empty() ||
      userid.empty() ||
      server.empty())
    return HBCI::Error("reallyCreateUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_DONTKNOW,
		       "empty arguments");

  // create user and medium
  try {
    // check if the bank exists
    bank=api->findBank(country,instid);
    if (!bank.isValid()) {
      Pointer<MessageEngine> me;
      Pointer<Transporter> tr;

      // need to create it
      DBG_INFO(0, "Creating bank %d/%s", country, instid.c_str());
      bank=api->bankFactory(country,
                            instid,
                            server);
      DBG_INFO(0, "Setting HBCI version %d", AQMONEY_DEFAULT_HBCI_VERSION);
      bank.ref().setHbciVersion(AQMONEY_DEFAULT_HBCI_VERSION);
      DBG_INFO(0, "Creating MessageEngine %s", AQMONEY_DEFAULT_MSG_ENGINE);
      me=api->engineFactory(country,
                            instid,
                            AQMONEY_DEFAULT_HBCI_VERSION,
                            AQMONEY_DEFAULT_MSG_ENGINE);
      DBG_INFO(0, "Creating Transporter %s", AQMONEY_DEFAULT_CONN_TYPE);
      tr=api->transporterFactory(server,
                                 AQMONEY_DEFAULT_CONN_PORT,
                                 AQMONEY_DEFAULT_CONN_TIMEOUT,
                                 AQMONEY_DEFAULT_CONN_TYPE);
      bank.ref().setMessageEngine(me);
      bank.ref().setTransporter(tr);
      api->addBank(bank);
    }
    else {
      DBG_NOTICE(0, "Bank %d/%s already exists", country, instid.c_str());
    }

    // check if user exists
    user=bank.ref().findUser(userid);
    if (!user.isValid()) {
      // need to create him
      DBG_INFO(0, "Creating user \"%s\"", userid.c_str());
      // check if medium exists
      medium=api->findMedium(mediumname);
      if (!medium.isValid())
	// need to create the medium
	  medium=api->createNewMedium(mediumType,
                                      readonly,
                                      country,
                                      instid,
                                      userid,
                                      mediumname);
      // then the user himself
      user= api->userFactory(bank,
                             medium,
                             userid);
      // add customer to the system
      bank.ref().addUser(user);

      // done
    }
    else {
      DBG_WARN(0, "User \"%s\" already exists", userid.c_str());
    }
    // create customer, if given
    if (customerid.empty())
      ucid=userid;
    else
      ucid=customerid;
    cust=user.ref().findCustomer(ucid);
    if (!cust.isValid()) {
      DBG_INFO(0, "Creating customer \"%s\"", ucid.c_str());
      cust=api->customerFactory(user,
                                ucid);
      user.ref().addCustomer(cust);
    }
    else {
      DBG_WARN(0, "Customer \"%s\" already exists", customerid.c_str());
    }

  } // try
  catch (HBCI::Error lerr) {
    err=lerr;
  }

  return err;
}



Error ModuleCreateUser::_importUser(AqMoneyAPI *api,
                                    string mediumname,
                                    const string &mtype,
                                    bool readonly,
                                    int country,
                                    const string &instid,
                                    const string &userid,
                                    const string &customerid,
                                    const string &server) {
  string tmp;
  Pointer<Medium> medium;
  int i;
  int j;
  int lcountry;
  string linstid;
  string luserid;
  string lserver;
  Error err;

  // now import the contexts from the medium
  medium=api->mediumFactory(mediumname,mtype);
  try {
    i=1;
    j=0;
    err=medium.ref().mountMedium();
    if (!err.isOk())
      return HBCI::Error("importUser",err);
    while(1) {
      // get context
      err=medium.ref().getContext(i,
				  lcountry,
				  linstid,
				  luserid,
				  lserver);
      if (!err.isOk())
	break;
      // add context, if it matches the criteria
      if (
	  (!country || lcountry==country) &&
          (-1!=GWEN_Text_ComparePattern(linstid.c_str(),
                                        instid.c_str(),
                                        0) || instid.empty())
         ) {
        mediumname=medium.ref().mediumName();
        err=_reallyCreateUser(api,
                              mediumname,
                              mtype,
                              readonly,
                              lcountry,
                              linstid,
                              userid.empty()?luserid:userid,
                              customerid,
                              lserver.empty()?server:lserver);
	if (!err.isOk())
	  fprintf(stderr,I18N("Error: %s\n"),
		  err.errorString().c_str());
	j++;
      }
      i++;
    } // while
    medium.ref().unmountMedium();
  } // try
  catch (Error lerr) {
    fprintf(stderr,"Exception: %s\n",
	    lerr.errorString().c_str());
    return HBCI::Error("importUser",err);
  }
  if (!j) {
    fprintf(stderr,I18N("No user added.\n"));
    return HBCI::Error("importUser()",
		       ERROR_LEVEL_NORMAL,
		       0,
		       ERROR_ADVISE_ABORT,
		       "no user added");
  }

  return HBCI::Error();
}





ModuleError ModuleCreateUser::run(AqMoneyAPI *api,
                                  int argc, char **argv,
                                  const string &openhbciFile,
                                  const string &aqmoneyFile){
  int rv;
  Error err;
  string userid;
  string customerid;
  string instid;
  string mediumname;
  string mtypename;
  bool isFile;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  // get the medium type
  mediumname=_args.mediumname;
  mtypename.erase();
  if (-1!=GWEN_Text_ComparePattern(_args.mediumtype.c_str(),
                                   "file",0)) {
    DBG_INFO(0, "Type is file");
    isFile=true;
    if (mediumname.empty()) {
      fprintf(stderr,I18N("File name needed (\"-m FILE\")\n"));
      return ModuleErrorArguments;
    }
    if (mediumname.at(0)!='/') {
      // make medium path absolute
    }
  }
  else if (-1!=GWEN_Text_ComparePattern(_args.mediumtype.c_str(),
                                        "card",0)) {
    DBG_INFO(0, "Type is card");
    isFile=false;
  }
  else {
    DBG_INFO(0, "Type is unknown");
    if (!_args.import) {
      fprintf(stderr,I18N(
              "Please use only \"file\" or "
              "\"card\" as MEDIUMTYPE\n"));
      return ModuleErrorInvalidArgs;
    }
  }

  if (_args.import || !isFile) {
    // get the medium type name for this medium
    if (!_checkMedium(api, isFile, mediumname, mtypename)) {
      fprintf(stderr, I18N("Could not determine the correct medium plugin.\n"));
      return ModuleErrorExecution;
    }
  }
  else if (isFile && !_args.import) {
    mtypename="RDHFile";
  }

  if (_args.import)
    err=_importUser(api,
                    mediumname,
                    mtypename,
                    _args.readonly,
                    _args.country,
                    _args.bankCode,
                    _args.user,
                    _args.customer,
                    _args.server);
  else
    err=_reallyCreateUser(api,
                          mediumname,
                          mtypename,
                          _args.readonly,
                          _args.country,
                          _args.bankCode,
                          _args.user,
                          _args.customer,
                          _args.server);
  if (!err.isOk()) {
    fprintf(stderr, I18N("Error: %s\n"), err.errorString().c_str());
    return ModuleErrorExecution;
  }

  DBG_INFO(0, "Saving environment");
  err=api->saveEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error saving environment (%s)",
              err.errorString().c_str());
    return ModuleErrorSave;
  }
  DBG_INFO(0, "Saving environment done.");

  return ModuleErrorSuccess;
}









