/***************************************************************************
  $RCSfile: dump.cpp,v $
  -------------------
  cvs         : $Id: dump.cpp,v 1.21 2004/04/02 05:28:29 aquamaniac Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif


#define ROWS 20
#define COLS 5

#define SIZE_DATE    12
#define SIZE_TEXT    16
#define SIZE_NAME    30
#define SIZE_PURPOSE 30
#define SIZE_VALUE   30
#define SIZE_TURNUS  20
#define SIZE_ALL (SIZE_DATE+SIZE_TEXT+SIZE_NAME+SIZE_VALUE+9)
#define SIZE_ALL_STO (SIZE_DATE+SIZE_TEXT+SIZE_NAME+SIZE_VALUE+SIZE_TURNUS+10)


#include "dump.h"
#include "accountjobs.h"
#include "aqmstring.h"

#include <gwenhywfar/debug.h>
#include <time.h>


ModuleDump::ModuleDump()
:AqmoneyModule() {
}



ModuleDump::~ModuleDump(){
}



void ModuleDump::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Dumps some information about the configuration\n"
               "   and data of your accounts etc\n"));
}


void ModuleDump::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 dump [-a %s] [-b %s] [-c %s]\n"
                  "%s   [--no-ah] [--balance] [--users] [--sto] [--dated] [--noted]\n"
                  "%s   [ --transactions [-oa %s] [-ob %s] [-oc %s]\n"
                  "%s                    [-f %s] [-t %s] [--pos|--neg] ]\n",
                  I18N("Usage:"),
                  I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("COUNTRYCODE"),
                  I18N("      "), I18N("      "),
                  I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("COUNTRYCODE"),
                  I18N("      "), I18N("YYYYMMDD"), I18N("YYYYMMDD"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleDump::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.accountId="*";
  args.userId="*";
  args.otherCountry=0;
  args.otherBankCode="*";
  args.otherAccount="*";
  args.flags=AQM_DUMP_FLAGS_DEFAULT;

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-a") {
      i++;
      if (i>=argc)
        return -1;
      args.accountId=argv[i];
    }
    else if (tmp=="-oa") {
      i++;
      if (i>=argc)
        return -1;
      args.otherAccount=argv[i];
    }
    else if (tmp=="-ob") {
      i++;
      if (i>=argc)
        return -1;
      args.otherBankCode=argv[i];
    }
    else if (tmp=="-oc") {
      i++;
      if (i>=argc)
        return -1;
      args.otherCountry=atoi(argv[i]);
    }
    else if (tmp=="-f") {
      i++;
      if (i>=argc)
        return -1;
      args.fromDate=Date(argv[i]);
      if (!args.fromDate.isValid()) {
        fprintf(stderr,
                I18N("Bad format of fromDate (must be YYYYMMDD)\n"));
        return -1;
      }
    }
    else if (tmp=="-t") {
      i++;
      if (i>=argc)
        return -1;
      args.toDate=Date(argv[i]);
      if (!args.toDate.isValid()) {
        fprintf(stderr,
                I18N("Bad format of toDate (must be YYYYMMDD)\n"));
        return -1;
      }
    }
    else if (tmp=="--transactions") {
      args.flags|=AQM_DUMP_FLAGS_TRANSACTIONS;
    }
    else if (tmp=="--sto") {
      args.flags|=AQM_DUMP_FLAGS_STANDINGORDERS;
    }
    else if (tmp=="--dated") {
      args.flags|=AQM_DUMP_FLAGS_DATEDXFERS;
    }
    else if (tmp=="--noted") {
      args.flags|=AQM_DUMP_FLAGS_NOTEDXFERS;
    }
    else if (tmp=="--balance") {
      args.flags|=AQM_DUMP_FLAGS_BALANCE;
    }
    else if (tmp=="--users") {
      args.flags|=AQM_DUMP_FLAGS_USERS;
    }
    else if (tmp=="--no-ah") {
      args.flags&=~AQM_DUMP_FLAGS_ACCOUNT_HEADER;
    }
    else if (tmp=="--neg") {
      args.flags&=~AQM_TRANSACTION_MATCH_POSITIVE;
    }
    else if (tmp=="--pos") {
      args.flags&=~AQM_TRANSACTION_MATCH_NEGATIVE;
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it
  return 0;
}



string ModuleDump::_condensedString(const string &s) {
  bool lastWasBlank;
  unsigned int i;
  string result;

  lastWasBlank=false;
  for (i=0; i<s.length(); i++) {
    if (s[i]==32 || s[i]==9) {
      if (!lastWasBlank) {
	result+=s[i];
      }
      lastWasBlank=true;
    }
    else {
      result+=s[i];
      lastWasBlank=false;
    }
  } // for
  return result;
}



string ModuleDump::_breakString(const string &s, unsigned int brk) {
  string result;
  unsigned int i;
  unsigned int lastBlankPos;
  unsigned int cnt;

  lastBlankPos=0;
  cnt=0;
  for (i=0; i<s.length(); i++) {
    if (s[i]==32 || s[i]==9)
      lastBlankPos=i;
    result+=s[i];
    cnt++;

    if (cnt>=brk && lastBlankPos) {
      result[lastBlankPos]='\n';
      lastBlankPos=0;
      cnt=0;
    }
  } // for
  return result;
}


string ModuleDump::_indentString(const string &s, unsigned int ins) {
  string result;
  unsigned int i;

  for (i=0; i<s.length(); i++) {
    result+=s[i];
    if (s[i]=='\n')
      result+=string(ins, ' ');
  } // for
  return result;
}



bool ModuleDump::_xactionToEntry(GWEN_DB_NODE *n,
                                 string *e,
                                 int &height) {
  string tmp1;
  string date;
  list<string> text;
  list<string>name;
  list<string>purpose;
  string value;
  list<string>::const_iterator iter;
  int i;
  const char *p;
  Date tdate;
  Value val;

  height=0;
  // date
  tdate=Date(GWEN_DB_GetCharValue(n, "valutaDate",0,""));
  if (!tdate.isValid())
    tdate=Date(GWEN_DB_GetCharValue(n, "date",0, ""));
  date=String::num2string(tdate.day());
  date+=".";
  date+=String::num2string(tdate.month());
  date+=".";
  date+=String::num2string(tdate.year());

  // text (and status)

  tmp1=GWEN_DB_GetCharValue(n, "text",0,"");
  if (tmp1.empty()) {
    switch(GWEN_DB_GetIntValue(n, "textkey",0,-1)) {
    case 51:
      text.push_back(I18N("transfer"));
      break;
    case 4:
    case 5:
      text.push_back(I18N("debit note"));
      break;
    case -1:
      text.push_back(I18N("unknown"));
      break;
    default:
      text.push_back(String::num2string(GWEN_DB_GetIntValue(n, "textkey",
                                                            0,0)));
      break;
    } // switch
  }
  else {
    text.push_back(tmp1.substr(0,SIZE_TEXT));
  }

  // name
  for (i=0; i<ROWS; i++) {
    p=GWEN_DB_GetCharValue(n, "othername",i, 0);
    if (!p)
      break;
    name.push_back(p);
  } // for

  // purpose
  for (i=0; i<ROWS; i++) {
    p=GWEN_DB_GetCharValue(n, "purpose",i, 0);
    if (!p)
      break;
    purpose.push_back(p);
  } // for

  // value
  val=Value(GWEN_DB_GetCharValue(n, "value/value",0, "0,"),
            GWEN_DB_GetCharValue(n, "value/currency",0, "EUR"));
  if (val.getValue()>=0)
    value=" ";
  value+=val.toReadableString();

  // now put it together
  e[0]=date;

  i=0;
  for (iter=text.begin();
       iter!=text.end() && i<(ROWS-1);
       iter++) {
    e[1+(COLS*i)]=*iter;
    i++;
  }
  if (i>height)
    height=i;

  i=0;
  for (iter=name.begin();
       iter!=name.end() && i<(ROWS-1);
       iter++) {
    e[2+(COLS*i)]=*iter;
    i++;
  }
  e[2+(COLS*i)]="";
  i++;
  for (iter=purpose.begin();
       iter!=purpose.end() && i<(ROWS-1);
       iter++) {
    e[2+(COLS*i)]=*iter;
    i++;
  }
  if (i>height)
    height=i;
  if (!height)
    height=1;
  e[3]=value;

  return true;
}



string ModuleDump::_dumpXactionTXT(GWEN_DB_NODE *n) {
  string e[COLS*ROWS];
  int i;
  string result;
  string tmp;
  int height;

  if (!_xactionToEntry(n,e,height))
    return "";
  for (i=0; i<(height*COLS); i+=COLS) {
    // date
    tmp=e[i+0];
    tmp=tmp.substr(0,SIZE_DATE);
    if (tmp.length()<SIZE_DATE)
      tmp+=string(SIZE_DATE-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // text
    tmp=e[i+1];
    tmp=tmp.substr(0,SIZE_TEXT);
    if (tmp.length()<SIZE_TEXT)
      tmp+=string(SIZE_TEXT-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // name / purpose
    tmp=e[i+2];
    tmp=tmp.substr(0,SIZE_NAME);
    if (tmp.length()<SIZE_NAME)
      tmp+=string(SIZE_NAME-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // value
    tmp=e[i+3];
    tmp=tmp.substr(0,SIZE_VALUE);
    if (tmp.length()<SIZE_VALUE)
      tmp+=string(SIZE_VALUE-tmp.length(),' ');
    result+=tmp;
    result+="\n";
  } // for
  return result;
}



bool ModuleDump::_stoToEntry(GWEN_DB_NODE *n,
                             string *e,
                             int &height) {
  string tmp1;
  string date;
  list<string> text;
  list<string>name;
  list<string>purpose;
  string value;
  list<string>::const_iterator iter;
  int i;
  const char *p;
  Date tdate;
  Value val;
  string turnus;
  char buffer[128];

  height=0;
  // date
  tdate=Date(GWEN_DB_GetCharValue(n, "valutaDate",0,""));
  if (!tdate.isValid())
    tdate=Date(GWEN_DB_GetCharValue(n, "date",0, ""));
  date=String::num2string(tdate.day());
  date+=".";
  date+=String::num2string(tdate.month());
  date+=".";
  date+=String::num2string(tdate.year());

  // text (and status)

  tmp1=GWEN_DB_GetCharValue(n, "text",0,"");
  if (tmp1.empty()) {
    switch(GWEN_DB_GetIntValue(n, "textkey",0, -1)) {
    case 51:
      text.push_back(I18N("transfer"));
      break;
    case 4:
    case 5:
      text.push_back(I18N("debit note"));
      break;
    case -1:
      text.push_back(I18N("unknown"));
      break;
    default:
      text.push_back(String::num2string(GWEN_DB_GetIntValue(n, "textkey",
                                                            0,0)));
      break;
    } // switch
  }
  else {
    text.push_back(tmp1.substr(0,SIZE_TEXT));
  }

  // name
  for (i=0; i<ROWS; i++) {
    p=GWEN_DB_GetCharValue(n, "othername",i, 0);
    if (!p)
      break;
    name.push_back(p);
  } // for

  // purpose
  for (i=0; i<ROWS; i++) {
    p=GWEN_DB_GetCharValue(n, "purpose",i, 0);
    if (!p)
      break;
    purpose.push_back(p);
  } // for

  // value
  val=Value(GWEN_DB_GetCharValue(n, "value/value",0, "0,"),
            GWEN_DB_GetCharValue(n, "value/currency",0, "EUR"));
  if (val.getValue()>=0)
    value=" ";
  value+=val.toReadableString();

  if (strcasecmp(GWEN_DB_GetCharValue(n, "unit", 0, "M"), "M")==0) {
    // monthly
    int j;

    j=GWEN_DB_GetIntValue(n, "turnus", 0, 1);
    if (j==1)
      turnus=I18N("Every month");
    else {
      sprintf(buffer, I18N("Every %d months"), j);
      turnus=buffer;
    }
  }
  else if (strcasecmp(GWEN_DB_GetCharValue(n, "unit", 0, "M"), "W")==0) {
    // weekly
    int j;

    j=GWEN_DB_GetIntValue(n, "turnus", 0, 1);
    if (j==1)
      turnus=I18N("Every week");
    else {
      sprintf(buffer, I18N("Every %d weeks"), j);
      turnus=buffer;
    }
  }
  else {
    // unknown
    turnus=I18N("Unknown cycle");
  }

  // now put it together
  e[0]=date;

  i=0;
  for (iter=text.begin();
       iter!=text.end() && i<(ROWS-1);
       iter++) {
    e[1+(COLS*i)]=*iter;
    i++;
  }
  if (i>height)
    height=i;

  i=0;
  for (iter=name.begin();
       iter!=name.end() && i<(ROWS-1);
       iter++) {
    e[2+(COLS*i)]=*iter;
    i++;
  }
  e[2+(COLS*i)]="";
  i++;
  for (iter=purpose.begin();
       iter!=purpose.end() && i<(ROWS-1);
       iter++) {
    e[2+(COLS*i)]=*iter;
    i++;
  }
  if (i>height)
    height=i;
  if (!height)
    height=1;
  e[3]=value;
  e[4]=turnus;
  return true;
}


string ModuleDump::_dumpStandingOrderTXT(GWEN_DB_NODE *n) {
  string e[COLS*ROWS];
  int i;
  string result;
  string tmp;
  int height;

  if (!_stoToEntry(n,e,height))
    return "";
  for (i=0; i<(height*COLS); i+=COLS) {
    // date
    tmp=e[i+0];
    tmp=tmp.substr(0,SIZE_DATE);
    if (tmp.length()<SIZE_DATE)
      tmp+=string(SIZE_DATE-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // text
    tmp=e[i+1];
    tmp=tmp.substr(0,SIZE_TEXT);
    if (tmp.length()<SIZE_TEXT)
      tmp+=string(SIZE_TEXT-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // name / purpose
    tmp=e[i+2];
    tmp=tmp.substr(0,SIZE_NAME);
    if (tmp.length()<SIZE_NAME)
      tmp+=string(SIZE_NAME-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // value
    tmp=e[i+3];
    tmp=tmp.substr(0,SIZE_VALUE);
    if (tmp.length()<SIZE_VALUE)
      tmp+=string(SIZE_VALUE-tmp.length(),' ');
    result+=tmp;
    result+=" | ";
    // turnus
    tmp=e[i+4];
    tmp=tmp.substr(0,SIZE_TURNUS);
    if (tmp.length()<SIZE_TURNUS)
      tmp+=string(SIZE_TURNUS-tmp.length(),' ');
    result+=tmp;

    result+="\n";
  } // for
  return result;
}



string ModuleDump::_dumpBalanceTXT(Pointer<Account> acc){
  string result;
  char buf[80];
  struct tm tm;
  const Account::Balance &noted=acc.ref().notedBalance();
  const Account::Balance &booked=acc.ref().bookedBalance();

  result+=I18N("\nAccount balances\n");
  result+=string(SIZE_ALL,'-')+"\n";
  if (!booked.date().isValid()) {
    result+=I18N(" --- no booked balance ---");
  }
  else {
    tm.tm_sec=booked.time().second();
    tm.tm_min=booked.time().minute();
    tm.tm_hour=booked.time().hour();
    tm.tm_mday=booked.date().day();
    tm.tm_mon=booked.date().month()-1;
    tm.tm_year=booked.date().year()-1900;
    mktime(&tm);
    strftime(buf,sizeof(buf),I18N("Booked balance of %F %T : "),&tm);
    result+=buf;
    result+=booked.value().toReadableString();
  }
  result+="\n";

  if (!noted.date().isValid()) {
    result+=I18N(" --- no noted balance ---");
  }
  else {
    tm.tm_sec=noted.time().second();
    tm.tm_min=noted.time().minute();
    tm.tm_hour=noted.time().hour();
    tm.tm_mday=noted.date().day();
    tm.tm_mon=noted.date().month()-1;
    tm.tm_year=noted.date().year()-1900;
    mktime(&tm);
    strftime(buf,sizeof(buf),I18N("Noted balance of %F %T  : "),&tm);
    result+=buf;
    result+=noted.value().toReadableString();
  }
  result+="\n";

  return result;
}



string ModuleDump::_dumpXactionsTXT(Pointer<Account> acc){
  GWEN_DB_NODE *xas;
  string result;

  xas=acc.ref().storage().ref().getTransactions(_args.otherCountry,
                                                _args.otherBankCode,
                                                _args.otherAccount,
                                                _args.fromDate,
                                                _args.toDate,
                                                _args.flags);
  if (xas) {
    GWEN_DB_NODE *n;

    result+=I18N("\nBooked transactions\n");
    result+=string(SIZE_ALL,'-')+"\n";
    n=GWEN_DB_GetFirstGroup(xas);
    if (n) {
      while(n) {
	if (strcasecmp(GWEN_DB_GroupName(n), "transaction")==0) {
	  DBG_NOTICE(0, "Adding transaction");
	  result+=_dumpXactionTXT(n);
	  result+=string(SIZE_ALL,'-');
	  result+="\n";
	} // if transaction
	n=GWEN_DB_GetNextGroup(n);
      } // while n
    }
    else {
      result+=I18N(" --- no booked transactions ---\n");
    }
    GWEN_DB_Group_free(xas);
  } // if transactions
  else {
    result+=I18N(" --- no booked transactions ---\n");
  }
  return result;
}



string ModuleDump::_dumpDatedTransfersTXT(Pointer<Account> acc){
  GWEN_DB_NODE *xas;
  string result;

  result+=I18N("\nPending dated transfers\n");
  result+=string(SIZE_ALL,'-')+"\n";
  xas=acc.ref().getDatedTransfers(_args.otherCountry,
                                  _args.otherBankCode,
                                  _args.otherAccount);
  if (xas) {
    GWEN_DB_NODE *n;

    n=GWEN_DB_GetFirstGroup(xas);
    if (n) {
      while(n) {
	if (strcasecmp(GWEN_DB_GroupName(n), "datedTransfer")==0) {
	  DBG_NOTICE(0, "Adding transaction");
	  result+=_dumpXactionTXT(n);
	  result+=string(SIZE_ALL,'-');
	  result+="\n";
	} // if transaction
	n=GWEN_DB_GetNextGroup(n);
      } // while n
    } // if subgroup
    else {
      result+=I18N(" --- no pending dated transfers ---\n");
    }
    GWEN_DB_Group_free(xas);
  } // if transactions
  else {
    result+=I18N(" --- no pending dated transfers ---\n");
  }

  return result;
}



string ModuleDump::_dumpNotedTransfersTXT(Pointer<Account> acc){
  GWEN_DB_NODE *xas;
  string result;

  xas=acc.ref().getNotedTransfers(_args.otherCountry,
                                  _args.otherBankCode,
                                  _args.otherAccount);
  result+=I18N("\nNoted transfers\n");
  result+=string(SIZE_ALL,'-')+"\n";

  if (xas) {
    GWEN_DB_NODE *n;

    n=GWEN_DB_GetFirstGroup(xas);
    if (n) {
      n=GWEN_DB_GetFirstGroup(xas);
      while(n) {
	if (strcasecmp(GWEN_DB_GroupName(n), "notedTransfer")==0) {
	  DBG_NOTICE(0, "Adding transaction");
	  result+=_dumpXactionTXT(n);
	  result+=string(SIZE_ALL,'-');
	  result+="\n";
	} // if transaction
	n=GWEN_DB_GetNextGroup(n);
      } // while n
    } // if transactions
    else {
      result+=I18N(" --- no noted transfers ---\n");
    }
    GWEN_DB_Group_free(xas);
  }
  else {
    result+=I18N(" --- no noted transfers ---\n");
  }

  return result;
}



string ModuleDump::_dumpStandingOrdersTXT(Pointer<Account> acc){
  GWEN_DB_NODE *xas;
  string result;

  xas=acc.ref().getStandingOrders(_args.otherCountry,
                                  _args.otherBankCode,
                                  _args.otherAccount);
  if (xas) {
    GWEN_DB_NODE *n;

    result+=I18N("\nStanding orders\n");
    result+=string(SIZE_ALL_STO,'-')+"\n";
    n=GWEN_DB_GetFirstGroup(xas);
    if (n) {
      while(n) {
	if (strcasecmp(GWEN_DB_GroupName(n), "standingOrder")==0) {
	  DBG_NOTICE(0, "Adding standing order");
	  result+=_dumpStandingOrderTXT(n);
	  result+=string(SIZE_ALL_STO,'-');
	  result+="\n";
	} // if transaction
	n=GWEN_DB_GetNextGroup(n);
      } // while n
    }
    else {
      result+=I18N(" --- no standing orders ---\n");
    }
    GWEN_DB_Group_free(xas);
  } // if transactions
  else {
    result+=I18N(" --- no standing orders ---\n");
  }

  return result;
}


int ModuleDump::_dumpAccountRelated(AqMoneyAPI *api) {
  list<Pointer<Account> > accs;
  list<Pointer<Account> >::iterator ait;
  string result;
  bool first;

  accs=api->getAccounts(_args.country,
                        _args.bankCode,
                        _args.accountId);
  if (accs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("No matching accounts found.\n"));
    return 2;
  }

  first=true;
  for (ait=accs.begin(); ait!=accs.end(); ait++) {
    Pointer<Bank> bank;

    bank=api->findBank((*ait).ref().country(),
                       (*ait).ref().serverBankCode());
    if (!bank.isValid()) {
      DBG_ERROR(0, "Bank %d/%s not found",
                (*ait).ref().country(),
                (*ait).ref().serverBankCode().c_str());
      return 3;
    }
    if (_args.flags & AQM_DUMP_FLAGS_ACCOUNT_HEADER) {
      if (first)
        first=false;
      else {
        result+=string(SIZE_ALL_STO,'=')+"\n";
        result+="\n";
      }
      
      result+=AqmoneyString::format(I18N("Account %s at %s (bankcode: %s)\n"),
                                    (*ait).ref().accountId().c_str(),
                                    bank.ref().name().c_str(),
                                    bank.ref().bankCode().c_str());

    }

    if (_args.flags & AQM_DUMP_FLAGS_TRANSACTIONS)
      result+=_dumpXactionsTXT(*ait);

    if (_args.flags & AQM_DUMP_FLAGS_STANDINGORDERS)
      result+=_dumpStandingOrdersTXT(*ait);

    if (_args.flags & AQM_DUMP_FLAGS_DATEDXFERS)
      result+=_dumpDatedTransfersTXT(*ait);

    if (_args.flags & AQM_DUMP_FLAGS_NOTEDXFERS)
      result+=_dumpNotedTransfersTXT(*ait);

    if (_args.flags & AQM_DUMP_FLAGS_BALANCE) {
      result+=_dumpBalanceTXT(*ait);
    }

    result+="\n";
  } // for accounts

  fprintf(stdout, "%s", result.c_str());
  return 0;
}



string ModuleDump::_dumpUserTXT(Pointer<User> u) {
  list<Pointer<Customer> >::const_iterator cit;
  string result;
  Pointer<Bank> bank;

  bank=u.ref().bank();
  if (!bank.isValid()) {
    fprintf(stderr, I18N("Internal Error: No bank pointer in User \"%s\"\n"),
            u.ref().userId().c_str());
    return "";
  }
  result+=I18N("User ID        : ");
  result+=u.ref().userId();
  result+="\n";
  result+=I18N("Bank           : ");
  if (bank.ref().name().empty())
    result+=bank.ref().bankCode();
  else
    result+=bank.ref().name();
  result+="\n";
  if (!(u.ref().customers().empty())) {
    result+=I18N("Customer list\n");
    for (cit=u.ref().customers().begin();
         cit!=u.ref().customers().end();
         cit++) {
      result+=I18N("  Customer ID   : ");
      result+=(*cit).ref().custId();
      result+="\n";
    }
  }
  return result;
}


string ModuleDump::_dumpUsersTXT(AqMoneyAPI *api) {
  list<Pointer<User> > users;
  list<Pointer<User> >::iterator uit;
  string result;

  users=api->getUsers(_args.country,
                      _args.bankCode,
                      _args.userId);
  if (!users.empty()) {
    result+=I18N("User list\n");
    result+=string(SIZE_ALL,'-')+"\n";
    for (uit=users.begin();
         uit!=users.end();
         uit++) {
      result+=_dumpUserTXT(*uit);
      result+=string(SIZE_ALL,'-')+"\n";
    } // for
  }
  result+="\n";

  return result;
}


int ModuleDump::_dumpUsers(AqMoneyAPI *api) {
  string s;

  s=_dumpUsersTXT(api);
  fprintf(stdout, "%s", s.c_str());
  return 0;
}


ModuleError ModuleDump::run(AqMoneyAPI *api,
                            int argc, char **argv,
                            const string &openhbciFile,
                            const string &aqmoneyFile){
  int rv;
  Error err;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  rv=_dumpAccountRelated(api);
  if (!rv && (_args.flags & AQM_DUMP_FLAGS_USERS))
    rv=_dumpUsers(api);

  if (rv)
    return ModuleErrorExecution;
  return ModuleErrorSuccess;
}









