/***************************************************************************
  $RCSfile: getaccounts.cpp,v $
  -------------------
  cvs         : $Id: getaccounts.cpp,v 1.12 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "adminjobs.h"
#include "getaccounts.h"
#include "accountjobs.h"

#include <gwenhywfar/debug.h>


ModuleGetAccounts::ModuleGetAccounts()
:AqmoneyModule() {
}



ModuleGetAccounts::~ModuleGetAccounts(){
}



void ModuleGetAccounts::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Retrieves the list of your accounts at a bank\n"));
}


void ModuleGetAccounts::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 acclist [-b %s] [-c %s] [-C %s]\n",
                  I18N("Usage:"),
                  I18N("BANKCODE"), I18N("COUNTRYCODE"), I18N("CUSTOMERID"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleGetAccounts::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.customerId="*";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.customerId=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it
  return 0;
}




ModuleError ModuleGetAccounts::run(AqMoneyAPI *api,
                                   int argc, char **argv,
                                   const string &openhbciFile,
                                   const string &aqmoneyFile){
  int rv;
  Error err;
  Pointer<Outbox> queue;
  list<Pointer<Customer> > custs;
  list<Pointer<Customer> >::iterator cit;
  list<Pointer<JobGetAccounts> > jobs;
  list<Pointer<JobGetAccounts> >::iterator jit;
  Pointer<JobGetAccounts> job;
  bool errorFlag;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }
  custs=api->getCustomers(_args.country,
                          _args.bankCode,
                          _args.customerId);
  if (custs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("No matching customers found.\n"));
    return ModuleErrorInvalidArgs;
  }

  try {
    DBG_INFO(0, "Creating new outbox");
    queue=new Outbox();

    for (cit=custs.begin(); cit!=custs.end(); cit++) {
      Pointer<Bank> bank;

      bank=(*cit).ref().user().ref().bank();
      job=new JobGetAccounts(*cit);
      DBG_INFO(0, "Adding job for customer %d%s%s to queue",
               bank.ref().country(),
               bank.ref().bankCode().c_str(),
               (*cit).ref().custId().c_str());
      jobs.push_back(job);
      queue.ref().addJob(job.cast<OutboxJob>());
    } // for

    DBG_INFO(0, "Executing queue");
    err=api->executeQueue(queue);
    if (!err.isOk()) {
      fprintf(stderr, I18N("Error: %s\n"), err.errorString().c_str());
    }
    DBG_INFO(0, "Queue executed.");

    DBG_INFO(0, "Committing results");
    err=api->processOutboxResponse(queue.ref().response());
    if (!err.isOk()) {
      DBG_ERROR(0, "Error: %s", err.errorString().c_str());
      return ModuleErrorExecution;
    }
    DBG_INFO(0, "Ok.");
    errorFlag=false;
    for (jit=jobs.begin(); jit!=jobs.end(); jit++) {
      list<MsgJob::Result>::const_iterator rit;
      for (rit=job.ref().results().begin();
           rit!=job.ref().results().end();
           rit++) {
        fprintf(stderr, I18N("Result: %d %s\n"),
                (*rit).resultCode(),
                (*rit).resultMessage().c_str());
      }
      if ((*jit).ref().result()!=HBCI_JOB_RESULT_FAILED) {
      }
      else {
        DBG_ERROR(0, "Error in job");
        errorFlag=true;
      }
    } // for
    DBG_INFO(0, "Saving environment");
    err=api->saveEnvironment(openhbciFile, aqmoneyFile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return ModuleErrorSave;
    }
    DBG_INFO(0, "Saving environment done.");
  } // try
  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  if (errorFlag)
    return ModuleErrorExecution;
  return ModuleErrorSuccess;
}









