/***************************************************************************
  $RCSfile: iniletter.cpp,v $
  -------------------
  cvs         : $Id: iniletter.cpp,v 1.14 2004/03/24 21:13:39 rbla Exp $
  begin       : Wed Dec 17 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# include <langinfo.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "iniletter.h"
#include <gwenhywfar/debug.h>
#include <time.h>
#include <errno.h>
#include <aqmstring.h>

ModuleIniLetter::ModuleIniLetter()
:AqmoneyModule() {
}



ModuleIniLetter::~ModuleIniLetter(){
}



void ModuleIniLetter::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Creates an ini letter [RDH mode]\n"));
}


void ModuleIniLetter::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 iniletter [-b %s] [-c %s] [-C %s]\n"
                  "%s   [-l user|institute] [-ot txt|ps] [-o %s]\n",
                  I18N("Usage:"),
                  I18N("BANKCODE"), I18N("COUNTRYCODE"), I18N("CUSTOMERID"),
                  I18N("      "), I18N("FILE"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleIniLetter::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.customerId="*";
  args.keyType="user";
  args.outputType="txt";
  args.outputFile="";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-l") {
      i++;
      if (i>=argc)
        return -1;
      args.keyType=argv[i];
    }
    else if (tmp=="-ot") {
      i++;
      if (i>=argc)
        return -1;
      args.outputType=argv[i];
    }
    else if (tmp=="-o") {
      i++;
      if (i>=argc)
        return -1;
      args.outputFile=argv[i];
    }
    else if (tmp=="-C") {
      i++;
      if (i>=argc)
        return -1;
      args.customerId=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it
  return 0;
}



string ModuleIniLetter::dumpHexString(string s) {
    string result;
    unsigned int pos;

    result+="   ";
    for (pos=0; pos<s.length(); pos++) {
        if ((pos & 31)==0)
            result+="\n   ";
        else if ((pos & 1)==0)
            result+=" ";
        result+=s.at(pos);
    } // for
    result+="\n";
    return result;
}

string ModuleIniLetter::dumpHexStringPS(string s, int *x, int *y, int offset) {
    string result;
    unsigned int pos;

    result+=AqmoneyString::format("%d %d moveto\n(", *x, *y);
    for (pos=0; pos<s.length(); pos++) {
        if ((pos & 31)==0) {
            if (pos != 0) {
                result+=") show\n";
                *y+=offset;
                result+=AqmoneyString::format("%d %d moveto\n(", *x, *y);
            }
        }
        else if ((pos & 1)==0)
            result+=" ";
        result+=s.at(pos);
    } // for
    result+=") show\n\n";
    return result;
}


string ModuleIniLetter::makeTxtLetter(string hash,
                                      string modulus,
                                      string exponent,
                                      int keyversion,
                                      int keynumber,
                                      string userid,
                                      string sysid,
                                      bool user,
                                      bool usecrypt) {
    string result;
    string tmp;
    time_t curtime=time(NULL);
    struct tm *tm=localtime(&curtime);
    char buf[80];

    result=I18N("\n\n\nINI-Letter\n\n");
    if (user) {
        result+=I18N("Username           : __________________________\n");
        // xgettext:no-c-format
        strftime(buf,sizeof(buf),I18N("Date               : %F\n"),tm);
        result+=buf;
        strftime(buf,sizeof(buf),I18N("Time               : %T\n"),tm);
        result+=buf;
        result+=I18N("User               : ")+userid+"\n";
        result+=I18N("Key number         : ")+HBCI::String::num2string(keynumber)+"\n";
        result+=I18N("Key version        : ")+HBCI::String::num2string(keyversion)+"\n";
        result+=I18N("Customer system    : ")+sysid+"\n";
        result+="\n";

        result+=I18N("Public key for electronic signature\n\n");
        result+=I18N("  Exponent\n");

        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexString(exponent);

        result+="\n";
        result+=I18N("  Modulus\n");
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexString(modulus);

        result+="\n";
        result+=I18N("  Hash\n");
        result+="\n";
        result+=dumpHexString(hash);

        result+="\n\n";
        result+=I18N("I confirm that I found above key for my electronic signature.\n");
        result+="\n\n";

        result+=I18N("_________________________  ____________________________\n"
                     "Place, date                Signature\n");
    }
    else {
        // server key
        // xgettext:no-c-format
        strftime(buf,sizeof(buf),I18N("Date               : %F\n"),tm);
        result+=buf;
        strftime(buf,sizeof(buf),I18N("Time               : %T\n"),tm);
        result+=buf;
        result+=I18N("Institute          : ")+userid+"\n";
        result+=I18N("Customer system    : ")+sysid+"\n";
        result+="\n";
        result+=I18N("Public key for electronic ");
        if (!usecrypt)
            result+=I18N("signature");
        else
            result+=I18N("encryption");
        result+="\n\n";
        result+=I18N("  Exponent\n");

        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexString(exponent);

        result+="\n";
        result+=I18N("  Modulus\n");
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexString(modulus);

        result+="\n";
        result+=I18N("  Hash\n");
        result+=dumpHexString(hash);
    }
    return result;
}


string ModuleIniLetter::makePSLetter(string hash,
                                       string modulus,
                                       string exponent,
                                       int keyversion,
                                       int keynumber,
                                       string userid,
                                       string sysid,
                                       bool user,
                                       bool usecrypt) {
    string result;
    string tmp;
    time_t curtime=time(NULL);
    struct tm *tm=localtime(&curtime);
    char buf[80];
    int x, y;
    char *domain;
    char *codeset;
  
#ifdef ENABLE_NLS
    domain=textdomain(NULL);
    codeset=bind_textdomain_codeset(domain, NULL);
    if (codeset==NULL) // locale charset
        codeset=nl_langinfo(CODESET);
    bind_textdomain_codeset(domain, "ISO8859-1"); // PS standard fonts only have latin1
#endif

    // PostScript Code based on PS output from OpenOffice.org
    result="%!PS-Adobe-2.0\n"
           "%%Creator: AqMoney-TNG\n"
           "%%Title: iniletter\n"
           "%%Pages: 1\n"
           "%%BoundingBox: 0 0 595 842\n"
           "%%DocumentFonts: Times-Bold Times-Roman Courier\n"
           "%%DocumentPaperSizes: a4\n"
           "%%EndComments\n"
           "%%EndProlog\n"
           "/Times-Roman findfont dup length dict begin\n"
           "{ 1 index /FID ne { def } {pop pop} ifelse } forall\n"
           "/Encoding ISOLatin1Encoding def currentdict end\n"
           "/Times-Roman exch definefont pop\n"
           "/Times-Bold findfont dup length dict begin\n"
           "{ 1 index /FID ne { def } {pop pop} ifelse } forall\n"
           "/Encoding ISOLatin1Encoding def currentdict end\n"
           "/Times-Bold exch definefont pop\n"
           "/Courier findfont dup length dict begin\n"
           "{ 1 index /FID ne { def } {pop pop} ifelse } forall\n"
           "/Encoding ISOLatin1Encoding def currentdict end\n"
           "/Courier exch definefont pop\n"
           "%%BeginSetup\n"
           "%%EndSetup\n"
           "%%Page: 1 1\n"
           "%%PageBoundingBox: 18 18 577 824\n"
           "%%BeginPageSetup\n"
           "%\n"
           "[{\n"
           "%%BeginFeature: *PageSize A4\n"
           "<</PageSize [595 842] /ImagingBBox null>> setpagedevice\n"
           "%%EndFeature\n"
           "} stopped cleartomark\n"
           "gsave\n"
           "[0.24 0 0 -0.24 18 824] concat\n"
           "gsave\n"
           "%%EndPageSetup\n"
           "0 0 0 setrgbcolor\n";
    result+="(Times-Bold) cvn findfont 67 -67 matrix scale makefont setfont\n";
    x=162; y=225;
    result+=AqmoneyString::format("%d %d moveto (%s) show\n", 1027, y, I18N("INI-Letter"));
    if (user) {
        result+="(Times-Roman) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=253, I18N("Username:"));
        result+=AqmoneyString::format("%d %d moveto (____________________________) show\n", x+600, y);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Date:"));
        // xgettext:no-c-format
        /* TRANSLATORS: strftime format string for current date */
        strftime(buf,sizeof(buf),I18N("%F"),tm);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, buf);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Time:"));
        // xgettext:no-c-format
        /* TRANSLATORS: strftime format string for current time */
        strftime(buf,sizeof(buf),I18N("%T"),tm);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, buf);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("User:"));
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, userid.c_str());
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Key number:"));
        result+=AqmoneyString::format("%d %d moveto (%d) show\n", x+600, y, keynumber);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Key version:"));
        result+=AqmoneyString::format("%d %d moveto (%d) show\n", x+600, y, keyversion);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Customer system:"));
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, sysid.c_str());
        result+="\n";

        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=183,
                                      I18N("Public key for electronic signature"));

        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Exponent"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexStringPS(exponent, &x, &y, 55);
        x-=90;
 
        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Modulus"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexStringPS(modulus, &x, &y, 55);
        x-=90;

        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Hash"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        result+=dumpHexStringPS(hash, &x, &y, 55);
        x-=90;

        result+="(Times-Roman) cvn findfont 50 -50 matrix scale makefont setfont\n";
	result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=183,
			I18N("I confirm that I found above key for my electronic signature.\n"));

        result+=AqmoneyString::format("%d %d moveto (____________________________) show\n", x, y+=183);
        result+=AqmoneyString::format("%d %d moveto (____________________________) show\n", x+861, y);
        result+="(Times-Roman) cvn findfont 42 -42 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=53, I18N("Place, date"));
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+861, y, I18N("Signature"));
    }
    else {
        // server key
        result+="(Times-Roman) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Date:"));
        // xgettext:no-c-format
        /* TRANSLATORS: strftime format string for current date */
        strftime(buf,sizeof(buf),I18N("%F"),tm);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, buf);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Time:"));
        // xgettext:no-c-format
        /* TRANSLATORS: strftime format string for current time */
        strftime(buf,sizeof(buf),I18N("%T"),tm);
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Institute:"));
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, userid.c_str());
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=61, I18N("Customer system:"));
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x+600, y, sysid.c_str());
        result+="\n";

        result+=AqmoneyString::format("%d %d moveto (%s %s) show\n", x, y+=183,
                                      I18N("Public key for electronic"),
                                      !usecrypt?I18N("signature"):I18N("encryption"));

        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Exponent"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexStringPS(exponent, &x, &y, 55);
        x-=90;
 
        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Modulus"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        if (exponent.length()<192)
            throw HBCI::Error("makeTxtLetter()",
                          ERROR_LEVEL_NORMAL,
                          0,
                          ERROR_ADVISE_DONTKNOW,
                          "exponent too short");
        result+=dumpHexStringPS(modulus, &x, &y, 55);
        x-=90;

        result+="(Times-Bold) cvn findfont 50 -50 matrix scale makefont setfont\n";
        result+=AqmoneyString::format("%d %d moveto (%s) show\n", x, y+=123, I18N("Hash"));
        result+="(Courier) cvn findfont 50 -50 matrix scale makefont setfont\n";
        x+=90; y+=120;
        result+=dumpHexStringPS(hash, &x, &y, 55);
    }

#ifdef ENABLE_NLS
    bind_textdomain_codeset(domain, codeset);
#endif

    result+="showpage\n%%Trailer\n%%EOF\n";
    return result;
}






ModuleError ModuleIniLetter::run(AqMoneyAPI *api,
                                 int argc, char **argv,
                                 const string &openhbciFile,
                                 const string &aqmoneyFile){
  int rv;
  Error err;
  list<Pointer<Customer> > custs;
  Pointer<Customer> cust;
  Pointer<Medium> medium;
  Pointer<MediumRDHBase> mediumrdh;
  Pointer<User> user;
  Pointer<Bank> bank;
  string modulus;
  string exponent;
  string hash;
  int keyversion;
  int keynumber;
  string result;
  bool isuser;
  string id;
  string tmp;
  bool usecrypt;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }
  custs=api->getCustomers(_args.country,
                          _args.bankCode,
                          _args.customerId);
  if (custs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("Error: No matching customers found.\n"));
    return ModuleErrorInvalidArgs;
  }
  if (custs.size()>1) {
    fprintf(stderr,
            "%s",
            I18N("Error: Ambiguous customer specification.\n"));
    return ModuleErrorInvalidArgs;
  }
  cust=custs.front();
  user=cust.ref().user();
  medium=user.ref().medium();
  bank=user.ref().bank();
  if (medium.ref().securityMode()!=HBCI_SECURITY_RDH) {
    fprintf(stderr,
            "%s",
            I18N("Error: Not in RDH mode.\n"));
    return ModuleErrorInvalidArgs;
  }
  mediumrdh=medium.cast<MediumRDHBase>();

  // mount medium
  err=mediumrdh.ref().mountMedium();
  if (!err.isOk()) {
    fprintf(stderr,
            I18N("Error mounting medium (%s)\n"),
            err.errorString().c_str());
    return ModuleErrorExecution;
  }

  // select context
  err=mediumrdh.ref().selectContext(bank.ref().country(),
                                    bank.ref().bankCode(),
                                    user.ref().userId());
  if (!err.isOk()) {
    mediumrdh.ref().unmountMedium();
    fprintf(stderr,
            I18N("Error selecting user context (%s)\n"),
            err.errorString().c_str());
    return ModuleErrorExecution;
  }

  try {
    if (!mediumrdh.ref().userPubSignKey().isValid()) {
      fprintf(stderr,
              "%s",
              I18N("Error: No sign key, please create keys.\n"));
      return ModuleErrorSequence;
    }
    if (!mediumrdh.ref().userPubCryptKey().isValid()) {
      fprintf(stderr,
              "%s",
              I18N("Error: No crypt key, please create keys.\n"));
      return ModuleErrorSequence;
    }

    if (strcasecmp(_args.keyType.c_str(), "user")==0) {
      hash=mediumrdh.ref().getUserIniLetterHash();
      exponent=mediumrdh.ref().getUserIniLetterExponent();
      modulus=mediumrdh.ref().getUserIniLetterModulus();
      keynumber=mediumrdh.ref().getUserKeyNumber();
      keyversion=mediumrdh.ref().getUserKeyVersion();
      isuser=true;
      id=user.ref().userId();
    } // if user
    else if (strcasecmp(_args.keyType.c_str(), "institute")==0) {
      // does the institute use signature keys ?
      usecrypt=!mediumrdh.ref().hasInstSignKey();
      hash=mediumrdh.ref().getInstIniLetterHash(usecrypt);
      exponent=mediumrdh.ref().getInstIniLetterExponent(usecrypt);
      modulus=mediumrdh.ref().getInstIniLetterModulus(usecrypt);
      keynumber=-1;
      keyversion=-1;
      isuser=false;
      id=bank.ref().bankCode();
    }
    else {
      fprintf(stderr,
              "%s",
              I18N("Error: Only \"user\" or \"institute\" supported.\n"));
      return ModuleErrorInvalidArgs;
    }

    // now we have all we need, unmount medium
    medium.ref().unmountMedium();

    // create text/ps output
    if (strcasecmp(_args.outputType.c_str(), "ps")==0)
      result=makePSLetter(hash,
                            modulus,
                            exponent,
                            keyversion,
                            keynumber,
                            id,
                            "AqMoney " AQMONEY_VERSION_STRING,
                            isuser,
                            usecrypt);
    else
      result=makeTxtLetter(hash,
                           modulus,
                           exponent,
                           keyversion,
                           keynumber,
                           id,
                           "AqMoney " AQMONEY_VERSION_STRING,
                           isuser,
                           usecrypt);

    // if no output file name is given or the output file name is
    // a single dash, output the ini letter contents to stdout

    if ((_args.outputFile.empty()) ||
        (0 == strcmp ("-", _args.outputFile.c_str()))){
      fprintf(stdout,"%s\n",result.c_str());
    }
    // else, output it to the file with the given name
    else {
      FILE *f;

      f=fopen(_args.outputFile.c_str(), "w+");
      if (!f) {
        fprintf(stderr,
                I18N("Could not create file \"%s\": %s\n"),
                _args.outputFile.c_str(),
                strerror(errno));
        return ModuleErrorInvalidArgs;
      }

      if (fwrite(result.c_str(), result.length(), 1, f)!=result.length()) {
        fprintf(stderr,
                I18N("Could not write file \"%s\": %s\n"),
                _args.outputFile.c_str(),
                strerror(errno));
        fclose(f);
        return ModuleErrorExecution;
      }
      if (fclose(f)) {
        fprintf(stderr,
                I18N("Could not write file \"%s\": %s\n"),
                _args.outputFile.c_str(),
                strerror(errno));
        return ModuleErrorExecution;
      }
    }
  } // try

  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  return ModuleErrorSuccess;
}




