/***************************************************************************
  $RCSfile: mkpinfile.cpp,v $
  -------------------
  cvs         : $Id: mkpinfile.cpp,v 1.10 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "adminjobs.h"
#include "mkpinfile.h"
#include "accountjobs.h"

#include <gwenhywfar/debug.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>



ModuleMkPinFile::ModuleMkPinFile()
:AqmoneyModule() {
}



ModuleMkPinFile::~ModuleMkPinFile(){
}



void ModuleMkPinFile::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Creates an empty PIN file.\n"));
}


void ModuleMkPinFile::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 mkpinfile [-o %s]\n"
                  "%s   [-b %s] [-c %s] [-u %s]\n",
                  I18N("Usage:"), I18N("FILE"),
                  I18N("      "),
                  I18N("BANKCODE"), I18N("COUNTRYCODE"), I18N("USERID"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleMkPinFile::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.userId="*";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-u") {
      i++;
      if (i>=argc)
        return -1;
      args.userId=argv[i];
    }
    else if (tmp=="-o") {
      i++;
      if (i>=argc)
        return -1;
      args.outFile=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      fprintf(stderr, I18N("Argument \"%s\" is no option for this command\n"),
              tmp.c_str());
      return -1;
    }
    i++;
  } // while
  // that's it
  return 0;
}




ModuleError ModuleMkPinFile::run(AqMoneyAPI *api,
                                 int argc, char **argv,
                                 const string &openhbciFile,
                                 const string &aqmoneyFile){
  int rv;
  list<Pointer<User> > users;
  list<Pointer<User> >::iterator uit;
  FILE *f;
  Error err;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }
  users=api->getUsers(_args.country,
                      _args.bankCode,
                      _args.userId);
  if (users.empty()) {
    fprintf(stderr,
            "%s",
            I18N("No matching users found.\n"));
    return ModuleErrorInvalidArgs;
  }

  if (_args.outFile.empty() ||
      _args.outFile=="-")
    f=stdout;
  else {
    f=fopen(_args.outFile.c_str(), "w+");
    if (!f) {
      fprintf(stderr,
              I18N("Error: fopen(%s): %s\n"),
              _args.outFile.c_str(), strerror(errno));
      return ModuleErrorWritingfile;
    }
  }
  for (uit=users.begin(); uit!=users.end(); uit++) {
    Pointer<Bank> bank;
    Pointer<Medium> medium;
    string bankname;

    bank=(*uit).ref().bank();
    bankname=bank.ref().name();
    if (bankname.empty())
      bankname=bank.ref().bankCode();
    medium=(*uit).ref().medium();
    fprintf(f, "# User %s at %s\n",
            (*uit).ref().userId().c_str(),
            bankname.c_str());
    fprintf(f, "%s=\"<PIN>\"\n", medium.ref().mediumName().c_str());
  } // for

  if (!_args.outFile.empty() &&
      _args.outFile!="-") {
    if (fclose(f)) {
      fprintf(stderr,
              I18N("Error: fclose(%s): %s\n"),
              _args.outFile.c_str(), strerror(errno));
      return ModuleErrorWritingfile;
    }
  }
  return ModuleErrorSuccess;
}









