/***************************************************************************
  $RCSfile: mtransfer.cpp,v $
  -------------------
  cvs         : $Id: mtransfer.cpp,v 1.16 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "mtransfer.h"
#include "accountjobs.h"
#include "aqmapi.h"
#include "aqmconverter.h"

#include <gwenhywfar/debug.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>



ModuleMultiTransfer::AccountTransfers::AccountTransfers(Pointer<Account>
                                                        account,
                                                        Pointer<Customer>
                                                        customer,
                                                        unsigned int c,
                                                        const string &bc,
                                                        const string &aid,
                                                        GWEN_DB_NODE *xa)
:_account(account)
,_customer(customer)
,_country(c)
,_bankCode(bc)
,_accountId(aid){
  _transfers=GWEN_DB_Group_new("transfers");
  if (xa)
    addTransfer(xa);
}



ModuleMultiTransfer::AccountTransfers::~AccountTransfers(){
  GWEN_DB_Group_free(_transfers);
}



void ModuleMultiTransfer::AccountTransfers::addTransfer(GWEN_DB_NODE *xa){
  GWEN_DB_AddGroup(_transfers,
                   GWEN_DB_Group_dup(xa));
}



bool ModuleMultiTransfer::AccountTransfers::matches(unsigned int c,
                                                    const string &bc,
                                                    const string &aid) const{
  return (c==_country && bc==_bankCode && aid==_accountId);
}







ModuleMultiTransfer::ModuleMultiTransfer()
:AqmoneyModule() {
}



ModuleMultiTransfer::~ModuleMultiTransfer(){
}



void ModuleMultiTransfer::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Transfers money between accounts, multi mode\n"));
}


void ModuleMultiTransfer::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 mtransfer\n"
                  "%s   [--skip-bad] [-gf %s] [-bf %s] [-nf %s]\n"
                  "%s   [-a %s] [-as %s] [-b %s]\n"
                  "%s   [-c %s] [-oc %s]\n"
                  "%s   -oa %s -ob %s -on %s\n"
                  "%s   -p %s -v %s [-k %s]\n"
                  "\n"
                  "%s aqmoney2 mtransfer\n"
                  "%s   [--skip-bad] [-gf %s] [-bf %s] [-nf %s]\n"
                  "%s   [-t db|old|dtaus|swift] [-tp %s] %s\n",
                  I18N("Usage:"),
                  I18N("      "), I18N("FILE"), I18N("FILE"), I18N("FILE"),
                  I18N("      "),
                    I18N("ACCOUNTID"), I18N("ACCOUNTSUBID"), I18N("BANKCODE"),
                  I18N("      "), I18N("COUNTRYCODE"), I18N("COUNTRYCODE"),
                  I18N("      "),
                    I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("NAME"),
                  I18N("      "), I18N("PURPOSE"), I18N("VALUE"),
                    I18N("TEXTKEY"),
                  I18N("      "),
                  I18N("      "), I18N("FILE"), I18N("FILE"), I18N("FILE"),
                  I18N("      "), I18N("PARAM"), I18N("FILE ..."));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleMultiTransfer::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.otherCountry=280;
  args.textKey=51;
  args.bankCode="*";
  args.accountId="*";
  args.accountSubId="*";
  args.skipBad=false;
  args.tfiletype="db";

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-a") {
      i++;
      if (i>=argc)
        return -1;
      args.accountId=argv[i];
    }
    else if (tmp=="-as") {
      i++;
      if (i>=argc)
        return -1;
      args.accountSubId=argv[i];
    }
    else if (tmp=="-oc") {
      i++;
      if (i>=argc)
	return -1;
      args.otherCountry=atoi(argv[i]);
    }
    else if (tmp=="-ob") {
      i++;
      if (i>=argc)
        return -1;
      args.otherBankCode=argv[i];
    }
    else if (tmp=="-oa") {
      i++;
      if (i>=argc)
        return -1;
      args.otherAccount=argv[i];
    }
    else if (tmp=="-on") {
      i++;
      if (i>=argc)
        return -1;
      args.otherName.push_back(argv[i]);
    }
    else if (tmp=="-p") {
      i++;
      if (i>=argc)
        return -1;
      args.purpose.push_back(argv[i]);
    }
    else if (tmp=="-t") {
      i++;
      if (i>=argc)
        return -1;
      args.tfiletype=argv[i];
    }
    else if (tmp=="-tp") {
      i++;
      if (i>=argc)
        return -1;
      args.tfileparam=argv[i];
    }
    else if (tmp=="-v") {
      i++;
      if (i>=argc)
        return -1;
      try {
        args.val=Value(argv[i]);
      }
      catch (Error xerr) {
        fprintf(stderr, I18N("Bad format of value\n"));
        return -1;
      }
    }
    else if (tmp=="-k") {
      i++;
      if (i>=argc)
        return -1;
      args.textKey=atoi(argv[i]);
    }
    else if (tmp=="--skip-bad") {
      args.skipBad=true;
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else if (tmp=="-gf") {
      i++;
      if (i>=argc)
        return -1;
      args.goodFile=argv[i];
    }
    else if (tmp=="-bf") {
      i++;
      if (i>=argc)
        return -1;
      args.badFile=argv[i];
    }
    else if (tmp=="-nf") {
      i++;
      if (i>=argc)
        return -1;
      args.notDoneFile=argv[i];
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to command \"%s\"\n"),
                tmp.c_str(), name().c_str());
        return -1;
      }
      else {
        args.tfiles.push_back(argv[i]);
      }
    }
    i++;
  } // while
  // that's it

  // check required arguments
  if (args.tfiles.empty() &&
       (args.otherAccount.empty() || args.otherBankCode.empty() ||
        args.otherName.empty() || args.purpose.empty() || !args.val.isValid()))
    return -1;

  return 0;
}


bool ModuleMultiTransfer::_checkAccount(AqMoneyAPI *api,
                                   unsigned int country,
                                   const string &bankCode,
                                   const string &accountid) {
  Pointer<BankManager> mgr;
  BANKMANAGER_CHECKRESULT rv;

  mgr=api->bankManager();
  rv=mgr.ref().checkAccount(country, bankCode, accountid);
  switch(rv) {
  case BankManagerCheckResultFailed:
    fprintf(stderr, I18N("Account \"%d/%s/%s\" seems to be invalid\n"),
            country, bankCode.c_str(), accountid.c_str());
    return false;
  case BankManagerCheckResultBankUnknown:
    fprintf(stderr, I18N("Unknown bank \"%d/%s\"\n"), country, bankCode.c_str());
    return false;
  default:
    return true;
  }
}


bool ModuleMultiTransfer::_checkAccount(AqMoneyAPI *api,
                                   GWEN_DB_NODE *n) {
  assert(n);
  return _checkAccount(api,
                       GWEN_DB_GetIntValue(n,
                                           "otheraccount/country",
                                           0, 280),
                       GWEN_DB_GetCharValue(n,
                                            "otheraccount/bankcode",
                                            0, ""),
                       GWEN_DB_GetCharValue(n,
                                            "otheraccount/accountid",
                                            0, ""));
}



GWEN_DB_NODE *
ModuleMultiTransfer::_makeTransaction(Pointer<Customer> c,
                                      Pointer<Account> acc,
                                      int otherCountry,
                                      const string &otherBankCode,
                                      const string &otherAccount,
                                      const list<string> &otherName,
                                      const list<string> &purpose,
                                      const Value &val,
                                      int textKey) {
  GWEN_DB_NODE *xa;
  list<string>::const_iterator sit;

  xa=GWEN_DB_Group_new("transaction");
  GWEN_DB_SetIntValue(xa,
                      GWEN_DB_FLAGS_DEFAULT,
                      "ouraccount/country",
                      acc.ref().country());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "ouraccount/bankCode",
                       acc.ref().bankCode().c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "ouraccount/accountId",
                       acc.ref().accountId().c_str());

  GWEN_DB_SetIntValue(xa,
                      GWEN_DB_FLAGS_DEFAULT,
                      "otheraccount/country",
                      otherCountry);
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "otheraccount/bankCode",
                       otherBankCode.c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "otheraccount/accountId",
                       otherAccount.c_str());

  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "value/value",
                       val.toString(true).c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "value/currency",
                       val.getCurrency().c_str());
  GWEN_DB_SetIntValue(xa,
                      GWEN_DB_FLAGS_DEFAULT,
                      "textkey",
                      textKey);

  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "name",
                       acc.ref().userName().c_str());

  // store othername
  if (otherName.size()>2 || otherName.empty()) {
    GWEN_DB_Group_free(xa);
    throw Error("JobSingleTransfer::JobSingleTransfer",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_INVALID,
                ERROR_ADVISE_DONTKNOW,
                "bad number of otherNames");
  }

  for (sit=otherName.begin(); sit!=otherName.end(); sit++) {
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "othername",
                         (*sit).c_str());
  }

  // store purpose
  for (sit=purpose.begin(); sit!=purpose.end(); sit++) {
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "purpose",
                         (*sit).c_str());
  }

  return xa;
}



ModuleError ModuleMultiTransfer::run(AqMoneyAPI *api,
                                     int argc, char **argv,
                                     const string &openhbciFile,
                                     const string &aqmoneyFile){
  int rv;
  Error err;
  list<Pointer<Account> > accs;
  Pointer<Outbox> queue;
  list<Pointer<JobMultiTransfer> > jobs;
  list<Pointer<JobMultiTransfer> >::iterator jit;
  Pointer<JobMultiTransfer> job;
  bool errorFlag;
  GWEN_DB_NODE *skippedXA;
  GWEN_DB_NODE *badXA;
  GWEN_DB_NODE *goodXA;
  list<Pointer<AccountTransfers> > axa;
  list<Pointer<AccountTransfers> >::iterator axait;

  skippedXA=0;
  badXA=0;
  goodXA=0;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  try {
    DBG_INFO(0, "Creating new outbox");
    queue=new Outbox();

    if (_args.tfiles.empty()) {
      Pointer<Account> acc;
      Pointer<Customer> cust;
      bool skip;
      GWEN_DB_NODE *xa;

      DBG_NOTICE(0, "Creating transfer(s) from command line");
      skip=false;
      xa=0;
      accs=api->getAccounts(_args.country,
                            _args.bankCode,
                            _args.accountId,
                            _args.accountSubId);
      if (accs.empty()) {
        fprintf(stderr,
                "%s",
                I18N("No matching accounts found.\n"));
        return ModuleErrorInvalidArgs;
      }
      if (accs.size()>1) {
        fprintf(stderr,
                "%s",
                I18N("Ambiguous account specification.\n"));
        return ModuleErrorInvalidArgs;
      }
      acc=accs.front();
      // check account
      if (!_checkAccount(api, _args.otherCountry,
                         _args.otherBankCode,
                         _args.otherAccount)) {
        if (_args.skipBad) {
          skip=true;
        }
      }

      cust=api->findCustomer(acc.ref().country(),
                             acc.ref().serverBankCode(),
                             acc.ref().customer());
      if (!cust.isValid()) {
        DBG_ERROR(0, "Customer \"%s\" not found, aborting",
                  acc.ref().customer().c_str());
        return ModuleErrorInconsistentSystem;
      }
      xa=_makeTransaction(cust, acc,
                          _args.otherCountry,
                          _args.otherBankCode,
                          _args.otherAccount,
                          _args.otherName,
                          _args.purpose,
                          _args.val,
                          _args.textKey);
      if (!skip) {
        try {
          job=new JobMultiTransfer(cust, acc);
          job.ref().addTransaction(xa);
          DBG_INFO(0, "Adding job for account %d/%s/%s to queue",
                   acc.ref().country(),
                   acc.ref().bankCode().c_str(),
                   acc.ref().accountId().c_str());
          jobs.push_back(job);
          queue.ref().addJob(job.cast<OutboxJob>());
        } // try
        catch (Error xerr) {
          skip=true;
        }
      } // if !skip

      if (skip) {
        if (!skippedXA)
          skippedXA=GWEN_DB_Group_new("skipped");
        GWEN_DB_AddGroup(skippedXA, GWEN_DB_Group_dup(xa));
      }
      GWEN_DB_Group_free(xa);
      xa=0;
    } // if no tfile
    else {
      GWEN_DB_NODE *transactions;
      list<string>::iterator sit;
      Pointer<Account> acc;
      Pointer<Customer> cust;
      GWEN_DB_NODE *n;

      transactions=GWEN_DB_Group_new("transactions");

      // now read all files
      for (sit=_args.tfiles.begin(); sit!=_args.tfiles.end(); sit++) {
        int infd;
        GWEN_BUFFEREDIO *bin;
        Pointer<AQMConverter> inconv;

        DBG_NOTICE(0, "Reading from file \"%s\"", (*sit).c_str());
        infd=open((*sit).c_str(), O_RDONLY);
        if (infd==-1) {
          GWEN_DB_Group_free(transactions);
          fprintf(stderr, I18N("open(%s): %s\n"), (*sit).c_str(), strerror(errno));
          return ModuleErrorReadingfile;
        }
        bin=GWEN_BufferedIO_File_new(infd);
        GWEN_BufferedIO_SetReadBuffer(bin, 0, 1024);

        // create converters
        inconv=api->converterFactory(_args.tfiletype, _args.tfileparam);
        if (!inconv.isValid()) {
          fprintf(stderr, I18N("Could not create converter \"%s\"\n"),
                  _args.tfiletype.c_str());
          GWEN_DB_Group_free(transactions);
          GWEN_BufferedIO_Abandon(bin);
          GWEN_BufferedIO_free(bin);
          return ModuleErrorInvalidArgs;
        }

        err=inconv.ref().readFromStream(bin);
        if (!err.isOk()) {
          fprintf(stderr, I18N("Error reading from file \"%s\": %s\n"),
                  (*sit).c_str(), err.errorString().c_str());
          GWEN_DB_Group_free(transactions);
          GWEN_BufferedIO_Abandon(bin);
          GWEN_BufferedIO_free(bin);
          return ModuleErrorReadingfile;
        }

        GWEN_DB_AddGroupChildren(transactions, inconv.ref().transactions());

        GWEN_BufferedIO_Close(bin);
        GWEN_BufferedIO_free(bin);
      } // for

      // ok, now we have all transactions, sort them by account
      n=GWEN_DB_GetFirstGroup(transactions);
      while (n) {
        int country;
        string bankCode;
        string accountId;
        string accountSubId;
        string name;
        bool skip;

        skip=false;
        if (strcasecmp(GWEN_DB_GroupName(n), "transfer")==0 ||
            strcasecmp(GWEN_DB_GroupName(n), "transaction")==0) {
          country=GWEN_DB_GetIntValue(n, "ouraccount/country", 0, 280);
          bankCode=GWEN_DB_GetCharValue(n, "ouraccount/bankcode", 0, "");
          accountId=GWEN_DB_GetCharValue(n, "ouraccount/accountid", 0, "");
          accountSubId=GWEN_DB_GetCharValue(n, "ouraccount/accountsubid",
                                            0, "");
          name=GWEN_DB_GetCharValue(n, "name", 0, "");

          accs=api->getAccounts(country, bankCode, accountId);
          if (accs.empty()) {
            fprintf(stderr,
                    "%s",
                    I18N("No matching accounts found, skipping\n"));
            skip=true;
          }
          if (accs.size()>1) {
            fprintf(stderr,
                    "%s",
                    I18N("Ambiguous account specification, skipping\n"));
            skip=true;
          }

          if (!skip) {
            acc=accs.front();
            // check account
            fprintf(stderr, I18N("Now checking\n")); // xxx
            if (!_checkAccount(api,
                               GWEN_DB_GetIntValue(n,
                                                   "otheraccount/country", 0,
                                                   280),
                               GWEN_DB_GetCharValue(n,
                                                    "otheraccount/bankcode", 0,
                                                    ""),
                               GWEN_DB_GetCharValue(n,
                                                    "otheraccount/accountid",0,
                                                    ""))){
              if (_args.skipBad) {
                skip=true;
              }
            }
          }

          if (!skip) {
            bool foundIt;

            cust=api->findCustomer(acc.ref().country(),
                                   acc.ref().serverBankCode(),
                                   acc.ref().customer());
            if (!cust.isValid()) {
              DBG_ERROR(0, "Customer \"%s\" not found, aborting",
                        acc.ref().customer().c_str());
              return ModuleErrorInconsistentSystem;
            }

            if (name.empty()) {
              // set the owner name if it is not
              GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                                   "name",
                                   acc.ref().userName().c_str());
            }

            // find AccountTransfers
            foundIt=false;
            for (axait=axa.begin(); axait!=axa.end(); axait++) {
              if ((*axait).ref().matches(country,
                                         bankCode,
                                         accountId)) {
                DBG_INFO(0, "Adding transfer to existing AccountTransfers");
                (*axait).ref().addTransfer(n);
                foundIt=true;
                break;
              }
            } // for
            if (!foundIt) {
              DBG_INFO(0, "Adding AccountTransfers");
              axa.push_back(new AccountTransfers(acc,
                                                 cust,
                                                 country,
                                                 bankCode,
                                                 accountId,
                                                 n));
            }
          } // if !skip
          else {
            // skipped transaction
            if (!skippedXA)
              skippedXA=GWEN_DB_Group_new("skipped");
            GWEN_DB_AddGroup(skippedXA, GWEN_DB_Group_dup(n));
          }
        } // if transaction
        else {
          DBG_NOTICE(0, "Unknown group \"%s\"", GWEN_DB_GroupName(n));
        }
        n=GWEN_DB_GetNextGroup(n);
      } // while

      // ok, now we have all transactions, create jobs
      for (axait=axa.begin(); axait!=axa.end(); axait++) {
        unsigned int maxXa;
        unsigned int nxa;

        n=GWEN_DB_GetFirstGroup((*axait).ref().transfers());
        job=new JobMultiTransfer((*axait).ref().customer(),
                                 (*axait).ref().account());
        maxXa=job.ref().getIntProperty("params/maxtransfers", 0);
        nxa=0;
        while(n) {
          if (maxXa && nxa>=maxXa) {
            if (job.ref().transactionCount()) {
              DBG_NOTICE(0, "Adding job for account %d/%s/%s",
                         (*axait).ref().country(),
                         (*axait).ref().bankCode().c_str(),
                         (*axait).ref().accountId().c_str());
              jobs.push_back(job);
              queue.ref().addJob(job.cast<OutboxJob>());
            }
            else {
              DBG_INFO(0, "No transaction for this job, discarding");
            }
            nxa=0;
            job=new JobMultiTransfer((*axait).ref().customer(),
                                     (*axait).ref().account());
          }
          err=job.ref().addTransaction(n);
          if (!err.isOk()) {
            if (!skippedXA)
              skippedXA=GWEN_DB_Group_new("skipped");
            GWEN_DB_AddGroup(skippedXA, GWEN_DB_Group_dup(n));
          }
          else
            nxa++;
          n=GWEN_DB_GetNextGroup(n);
        } // while
        // add last job
        if (job.ref().transactionCount()) {
          DBG_NOTICE(0, "Adding job for account %d/%s/%s",
                     (*axait).ref().country(),
                     (*axait).ref().bankCode().c_str(),
                     (*axait).ref().accountId().c_str());
          jobs.push_back(job);
          queue.ref().addJob(job.cast<OutboxJob>());
        }
        else {
          DBG_INFO(0, "No transaction for this job, discarding");
        }
      } // for

      // free all AccountTransfers
      axa.clear();
    } // if tfiles given

    if (queue.ref().sizeByStatus(HBCI_JOB_STATUS_TODO)) {
      DBG_INFO(0, "Executing queue");
      errorFlag=false;
      err=api->executeQueue(queue);
      if (!err.isOk()) {
        fprintf(stderr, I18N("Error: %s\n"), err.errorString().c_str());
        errorFlag=true;
      }
      DBG_INFO(0, "Queue executed.");

      DBG_INFO(0, "Committing results");
      err=api->processOutboxResponse(queue.ref().response());
      if (!err.isOk()) {
        DBG_ERROR(0, "Error: %s", err.errorString().c_str());
        errorFlag=true;
      }
      else {
        DBG_INFO(0, "Ok.");
      };
    } // if queue not empty

    if (jobs.empty()) {
      fprintf(stdout, I18N("No job created.\n"));
    }
    else {
      fprintf(stdout, I18N("Detailed job results:\n"));
  
      // check undated jobs
      for (jit=jobs.begin(); jit!=jobs.end(); jit++) {
        bool isBad;
  
        isBad=((*jit).ref().result()==HBCI_JOB_RESULT_FAILED ||
               (*jit).ref().status()!=HBCI_JOB_STATUS_DONE);
        fprintf(stdout, "%s",
                (*jit).ref().resultString(!isBad).c_str());
  
        err=(*jit).ref().commit();
        if (!err.isOk()) {
          DBG_ERROR(0, "Error: %s", err.errorString().c_str());
          errorFlag=true;
        }
  
        if (!isBad) {
          if (!goodXA)
            goodXA=GWEN_DB_Group_new("good");
          GWEN_DB_AddGroupChildren(goodXA,
                               GWEN_DB_Group_dup((*jit).ref().transactions()));
        }
        else {
          DBG_NOTICE(0, "Error in job");
          errorFlag=true;
          if (!badXA)
            badXA=GWEN_DB_Group_new("bad");
          GWEN_DB_AddGroupChildren(badXA,
                               GWEN_DB_Group_dup((*jit).ref().transactions()));
        }
      } // for
    }

    // write various files
    if (!_args.badFile.empty()) {
      if (badXA) {
        if (GWEN_DB_WriteFile(badXA,
                              _args.badFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing badfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.badFile.c_str());
      }
    }
    if (!_args.goodFile.empty()) {
      if (goodXA) {
        if (GWEN_DB_WriteFile(goodXA,
                              _args.goodFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing goodfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.goodFile.c_str());
      }
    }
    if (!_args.notDoneFile.empty()) {
      if (skippedXA) {
        if (GWEN_DB_WriteFile(skippedXA,
                              _args.notDoneFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing skippedfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.notDoneFile.c_str());
      }
    }

    DBG_INFO(0, "Saving environment");
    err=api->saveEnvironment(openhbciFile, aqmoneyFile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return ModuleErrorSave;
    }
    DBG_INFO(0, "Saving environment done.");
  } // try
  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  if (errorFlag)
    return ModuleErrorExecution;
  return ModuleErrorSuccess;
}









