/***************************************************************************
  $RCSfile: mtransfer.h,v $
  -------------------
  cvs         : $Id: mtransfer.h,v 1.5 2004/01/25 18:52:18 aquamaniac Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/

#ifndef AQMONEY_MTRANSFER_H
#define AQMONEY_MTRANSFER_H

#include "aqmmodule.h"
#include <list>
#include <string>

using namespace std;


/**
 *
 */
class ModuleMultiTransfer: public AqmoneyModule {
private:

  class AccountTransfers {
  private:
    Pointer<Account> _account;
    Pointer<Customer> _customer;
    unsigned int _country;
    string _bankCode;
    string _accountId;
    GWEN_DB_NODE *_transfers;

  public:
    AccountTransfers(Pointer<Account> account,
                     Pointer<Customer> customer,
                     unsigned int country,
                     const string &bankCode,
                     const string &accountId,
                     GWEN_DB_NODE *xa);
    ~AccountTransfers();
    void addTransfer(GWEN_DB_NODE *xa);
    bool matches(unsigned int country,
                 const string &bankCode,
                 const string &accountId) const;
    unsigned int country() const { return _country;};
    const string &bankCode() const {return _bankCode;};
    const string &accountId() const { return _accountId;};
    GWEN_DB_NODE *transfers() { return _transfers;};
    Pointer<Account> account() { return _account;};
    Pointer<Customer> customer() { return _customer;};

  };

  struct s_args {
    int country;              // -c
    string bankCode;          // -b
    string accountId;         // -a
    string accountSubId;      // -as
    int otherCountry;         // -oc
    string otherBankCode;     // -ob
    string otherAccount;      // -oa
    Value val;                // -v
    int textKey;              // -k
    list<string> otherName;   // -on
    list<string> purpose;     // -p
    bool skipBad;             // -skip-bad
    string badFile;           // -bf
    string goodFile;          // -gf
    string notDoneFile;       // -nf
    string tfiletype;         // -t
    string tfileparam;        // -tp
    list<string> tfiles;
  };
  s_args _args;

  int _checkArgs(s_args &args, int argc, char **argv);
  void _usage();
  bool _checkAccount(AqMoneyAPI *api, GWEN_DB_NODE *n);
  bool _checkAccount(AqMoneyAPI *api,
                     unsigned int country,
                     const string &bankCode,
                     const string &accountid);
  GWEN_DB_NODE *_makeTransaction(Pointer<Customer> c,
                                 Pointer<Account> acc,
                                 int otherCountry,
                                 const string &otherBankCode,
                                 const string &otherAccount,
                                 const list<string> &otherName,
                                 const list<string> &purpose,
                                 const Value &val,
                                 int textKey);

public:
  ModuleMultiTransfer();
  virtual ~ModuleMultiTransfer();
  virtual string name() const { return "mtransfer";};
  virtual void dumpDescription(FILE *f);
  virtual ModuleError run(AqMoneyAPI *api,
                          int argc, char **argv,
                          const string &openhbciFile,
                          const string &aqmoneyFile);
  virtual void usage() { return _usage(); };

};



#endif


