/***************************************************************************
  $RCSfile: startsto.cpp,v $
  -------------------
  cvs         : $Id: startsto.cpp,v 1.11 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif



#include "startsto.h"
#include "accountjobs.h"
#include "aqmapi.h"
#include "aqmconverter.h"

#include <gwenhywfar/debug.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>



ModuleStartStandingOrder::ModuleStartStandingOrder()
:AqmoneyModule() {
}



ModuleStartStandingOrder::~ModuleStartStandingOrder(){
}



void ModuleStartStandingOrder::dumpDescription(FILE *f){
  fprintf(f, "%s",
	  I18N("Issues a standing order\n"));
}


void ModuleStartStandingOrder::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 startsto [--skip-bad]\n"
                  "%s   [-a %s] [-b %s] [-c %s]\n"
                  "%s   [-oc %s] -oa %s -ob %s\n"
                  "%s   -on %s -p %s -v %s [-k %s]\n"
                  "%s   -fd %s [-ld %s]\n"
                  "%s   [-w] [-pl %s] [-dy %s] [-d %s]\n" 
                  "\n"
                  "%s aqmoney2 startsto [--skip-bad]\n"
                  "%s   [-t db|old|dtaus|swift] [-tp %s] %s\n",
                  I18N("Usage:"),
                  I18N("      "),
                  I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("COUNTRYCODE"),
                  I18N("      "),
                  I18N("COUNTRYCODE"), I18N("ACCOUNTID"), I18N("BANKCODE"),
                  I18N("      "),
                  I18N("NAME"), I18N("PURPOSE"), I18N("VALUE"), I18N("TEXTKEY"),
                  I18N("      "), I18N("YYYYMMDD"), I18N("YYYYMMDD"),
                  I18N("      "), I18N("PERIODLENGTH"), I18N("DAY"), I18N("YYYYMMDD"),
                  I18N("      "),
                  I18N("      "), I18N("PARAM"), I18N("FILE ..."));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleStartStandingOrder::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.otherCountry=280;
  args.textKey=5;
  args.bankCode="*";
  args.accountId="*";
  args.skipBad=false;
  args.tfiletype="db";
  args.weekly=false;
  args.turnus=1;
  args.day=1;

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-a") {
      i++;
      if (i>=argc)
        return -1;
      args.accountId=argv[i];
    }
    else if (tmp=="-oc") {
      i++;
      if (i>=argc)
	return -1;
      args.otherCountry=atoi(argv[i]);
    }
    else if (tmp=="-ob") {
      i++;
      if (i>=argc)
        return -1;
      args.otherBankCode=argv[i];
    }
    else if (tmp=="-oa") {
      i++;
      if (i>=argc)
        return -1;
      args.otherAccount=argv[i];
    }
    else if (tmp=="-on") {
      i++;
      if (i>=argc)
        return -1;
      args.otherName.push_back(argv[i]);
    }
    else if (tmp=="-p") {
      i++;
      if (i>=argc)
        return -1;
      args.purpose.push_back(argv[i]);
    }
    else if (tmp=="-v") {
      i++;
      if (i>=argc)
        return -1;
      try {
        args.val=Value(argv[i]);
      }
      catch (Error xerr) {
        fprintf(stderr, I18N("Bad format of value\n"));
        return -1;
      }
    }
    else if (tmp=="-d") {
      i++;
      if (i>=argc)
        return -1;
      try {
        args.date=Date(argv[i]);
      }
      catch (Error xerr) {
        fprintf(stderr, I18N("Bad format of date\n"));
        return -1;
      }
    }
    else if (tmp=="-fd") {
      i++;
      if (i>=argc)
        return -1;
      try {
	args.firstDate=Date(argv[i]);
      }
      catch (Error xerr) {
        fprintf(stderr, I18N("Bad format of date\n"));
        return -1;
      }
    }
    else if (tmp=="-ld") {
      i++;
      if (i>=argc)
        return -1;
      try {
	args.lastDate=Date(argv[i]);
      }
      catch (Error xerr) {
        fprintf(stderr, I18N("Bad format of date\n"));
        return -1;
      }
    }
    else if (tmp=="-k") {
      i++;
      if (i>=argc)
        return -1;
      args.textKey=atoi(argv[i]);
    }
    else if (tmp=="-w") {
      args.weekly=true;
    }
    else if (tmp=="-pl") {
      i++;
      if (i>=argc)
	return -1;
      args.turnus=atoi(argv[i]);
    }
    else if (tmp=="-dy") {
      i++;
      if (i>=argc)
	return -1;
      args.day=atoi(argv[i]);
    }
    else if (tmp=="--skip-bad") {
      args.skipBad=true;
    }
    else if (tmp=="-t") {
      i++;
      if (i>=argc)
	return -1;
      args.tfiletype=argv[i];
    }
    else if (tmp=="-tp") {
      i++;
      if (i>=argc)
	return -1;
      args.tfileparam=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else if (tmp=="-gf") {
      i++;
      if (i>=argc)
        return -1;
      args.goodFile=argv[i];
    }
    else if (tmp=="-bf") {
      i++;
      if (i>=argc)
        return -1;
      args.badFile=argv[i];
    }
    else if (tmp=="-nf") {
      i++;
      if (i>=argc)
        return -1;
      args.notDoneFile=argv[i];
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to command \"%s\"\n"),
                tmp.c_str(), name().c_str());
        return -1;
      }
      else {
        args.tfiles.push_back(argv[i]);
      }
    }
    i++;
  } // while
  // that's it

  // check required arguments
  if (args.tfiles.empty() &&
       (args.otherAccount.empty() || args.otherBankCode.empty() ||
        args.otherName.empty() || args.purpose.empty() || !args.val.isValid() ||
        !args.firstDate.isValid()))
    return -1;

  return 0;
}


bool ModuleStartStandingOrder::_checkAccount(AqMoneyAPI *api,
                                   unsigned int country,
                                   const string &bankCode,
                                   const string &accountid) {
  Pointer<BankManager> mgr;
  BANKMANAGER_CHECKRESULT rv;

  mgr=api->bankManager();
  rv=mgr.ref().checkAccount(country, bankCode, accountid);
  switch(rv) {
  case BankManagerCheckResultFailed:
    fprintf(stderr, I18N("Account \"%d/%s/%s\" seems to be invalid\n"),
            country, bankCode.c_str(), accountid.c_str());
    return false;
  case BankManagerCheckResultBankUnknown:
    fprintf(stderr, I18N("Unknown bank \"%d/%s\"\n"), country, bankCode.c_str());
    return false;
  default:
    return true;
  }
}


bool ModuleStartStandingOrder::_checkAccount(AqMoneyAPI *api,
                                   GWEN_DB_NODE *n) {
  assert(n);
  return _checkAccount(api,
                       GWEN_DB_GetIntValue(n,
                                           "otheraccount/country",
                                           0, 280),
                       GWEN_DB_GetCharValue(n,
                                            "otheraccount/bankcode",
                                            0, ""),
                       GWEN_DB_GetCharValue(n,
                                            "otheraccount/accountid",
                                            0, ""));
}



GWEN_DB_NODE *ModuleStartStandingOrder::_makeTransaction(Pointer<Customer> c,
							 Pointer<Account> acc){
  GWEN_DB_NODE *xa;
  list<string>::const_iterator sit;

  xa=GWEN_DB_Group_new("standingOrder");
  GWEN_DB_SetIntValue(xa,
		      GWEN_DB_FLAGS_DEFAULT,
		      "ouraccount/country",
                      acc.ref().country());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "ouraccount/bankCode",
                       acc.ref().bankCode().c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "ouraccount/accountId",
                       acc.ref().accountId().c_str());

  GWEN_DB_SetIntValue(xa,
                      GWEN_DB_FLAGS_DEFAULT,
                      "otheraccount/country",
		      _args.otherCountry);
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "otheraccount/bankCode",
		       _args.otherBankCode.c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "otheraccount/accountId",
		       _args.otherAccount.c_str());

  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "value/value",
		       _args.val.toString(true).c_str());
  GWEN_DB_SetCharValue(xa,
                       GWEN_DB_FLAGS_DEFAULT,
                       "value/currency",
		       _args.val.getCurrency().c_str());
  GWEN_DB_SetIntValue(xa,
                      GWEN_DB_FLAGS_DEFAULT,
                      "textkey",
		      _args.textKey);
  GWEN_DB_SetCharValue(xa,
		       GWEN_DB_FLAGS_DEFAULT,
		       "unit",
		       (_args.weekly?"W":"M"));
  GWEN_DB_SetIntValue(xa,
		      GWEN_DB_FLAGS_DEFAULT,
		      "turnus",
		      _args.turnus);
  GWEN_DB_SetIntValue(xa,
		      GWEN_DB_FLAGS_DEFAULT,
		      "day",
		      _args.day);

  // store othername
  if (_args.otherName.size()>2 || _args.otherName.empty()) {
    GWEN_DB_Group_free(xa);
    throw Error("JobSingleTransfer::JobSingleTransfer",
                ERROR_LEVEL_NORMAL,
                HBCI_ERROR_CODE_INVALID,
                ERROR_ADVISE_DONTKNOW,
                "bad number of otherNames");
  }

  for (sit=_args.otherName.begin(); sit!=_args.otherName.end(); sit++) {
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "othername",
                         (*sit).c_str());
  }

  // store purpose
  for (sit=_args.purpose.begin(); sit!=_args.purpose.end(); sit++) {
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "purpose",
                         (*sit).c_str());
  }

  if (_args.date.isValid()) {
    GWEN_DB_SetCharValue(xa,
                         GWEN_DB_FLAGS_DEFAULT,
                         "date",
			 _args.date.toString().c_str());
  }

  if (_args.firstDate.isValid()) {
    GWEN_DB_SetCharValue(xa,
			 GWEN_DB_FLAGS_DEFAULT,
			 "firstDate",
			 _args.firstDate.toString().c_str());
  }

  if (_args.lastDate.isValid()) {
    GWEN_DB_SetCharValue(xa,
			 GWEN_DB_FLAGS_DEFAULT,
			 "lastDate",
			 _args.lastDate.toString().c_str());
  }

  return xa;
}



ModuleError ModuleStartStandingOrder::run(AqMoneyAPI *api,
                                          int argc, char **argv,
                                          const string &openhbciFile,
                                          const string &aqmoneyFile){
  int rv;
  Error err;
  list<Pointer<Account> > accs;
  Pointer<Outbox> queue;
  list<Pointer<JobStartStandingOrder> > jobs;
  list<Pointer<JobStartStandingOrder> >::iterator jit;
  Pointer<JobStartStandingOrder> job;
  bool errorFlag;
  GWEN_DB_NODE *skippedXA;
  GWEN_DB_NODE *badXA;
  GWEN_DB_NODE *goodXA;

  skippedXA=0;
  badXA=0;
  goodXA=0;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  try {
    DBG_INFO(0, "Creating new outbox");
    queue=new Outbox();

    if (_args.tfiles.empty()) {
      Pointer<Account> acc;
      Pointer<Customer> cust;
      bool skip;
      GWEN_DB_NODE *xa;

      skip=false;
      xa=0;
      accs=api->getAccounts(_args.country,
                            _args.bankCode,
                            _args.accountId);
      if (accs.empty()) {
        fprintf(stderr,
                "%s",
                I18N("No matching accounts found.\n"));
        return ModuleErrorInvalidArgs;
      }
      if (accs.size()>1) {
        fprintf(stderr,
                "%s",
                I18N("Ambiguous account specification.\n"));
        return ModuleErrorInvalidArgs;
      }
      acc=accs.front();
      // check account
      if (!_checkAccount(api, _args.otherCountry,
                         _args.otherBankCode,
                         _args.otherAccount)) {
        if (_args.skipBad) {
          skip=true;
        }
      }

      cust=api->findCustomer(acc.ref().country(),
                             acc.ref().serverBankCode(),
                             acc.ref().customer());
      if (!cust.isValid()) {
        DBG_ERROR(0, "Customer \"%s\" not found, aborting",
                  acc.ref().customer().c_str());
        return ModuleErrorInconsistentSystem;
      }
      xa=_makeTransaction(cust, acc);
      if (!skip) {
        try {
	  job=new JobStartStandingOrder(cust, acc, xa);
          DBG_INFO(0, "Adding job for account %d/%s/%s to queue",
                   acc.ref().country(),
                   acc.ref().bankCode().c_str(),
                   acc.ref().accountId().c_str());
          jobs.push_back(job);
          queue.ref().addJob(job.cast<OutboxJob>());
        } // try
	catch (Error xerr) {
	  DBG_INFO(0, "Exception: %s", xerr.errorString().c_str());
          skip=true;
        }
      } // if !skip

      if (skip) {
        if (!skippedXA)
          skippedXA=GWEN_DB_Group_new("skipped");
        GWEN_DB_AddGroup(skippedXA, GWEN_DB_Group_dup(xa));
      }
      GWEN_DB_Group_free(xa);
      xa=0;
    } // if no tfile
    else {
      GWEN_DB_NODE *standingOrder;
      list<string>::iterator sit;
      Pointer<Account> acc;
      Pointer<Customer> cust;
      GWEN_DB_NODE *n;

      standingOrder=GWEN_DB_Group_new("standingOrders");

      // now read all files
      for (sit=_args.tfiles.begin(); sit!=_args.tfiles.end(); sit++) {
	int infd;
	GWEN_BUFFEREDIO *bin;
	Pointer<AQMConverter> inconv;

	DBG_NOTICE(0, "Reading from file \"%s\"", (*sit).c_str());
	infd=open((*sit).c_str(), O_RDONLY);
	if (infd==-1) {
	  GWEN_DB_Group_free(standingOrder);
	  fprintf(stderr, I18N("open(%s): %s\n"), (*sit).c_str(), strerror(errno));
          return ModuleErrorReadingfile;
	}
	bin=GWEN_BufferedIO_File_new(infd);
	GWEN_BufferedIO_SetReadBuffer(bin, 0, 1024);

	// create converters
	inconv=api->converterFactory(_args.tfiletype, _args.tfileparam);
	if (!inconv.isValid()) {
	  fprintf(stderr, I18N("Could not create converter \"%s\"\n"),
                  _args.tfiletype.c_str());
	  GWEN_DB_Group_free(standingOrder);
	  GWEN_BufferedIO_Abandon(bin);
	  GWEN_BufferedIO_free(bin);
	  return ModuleErrorInvalidArgs;
	}

	err=inconv.ref().readFromStream(bin);
	if (!err.isOk()) {
	  fprintf(stderr, I18N("Error reading from file \"%s\": %s\n"),
                  (*sit).c_str(), err.errorString().c_str());
          GWEN_DB_Group_free(standingOrder);
          GWEN_BufferedIO_Abandon(bin);
          GWEN_BufferedIO_free(bin);
          return ModuleErrorReadingfile;
        }

	GWEN_DB_AddGroupChildren(standingOrder, inconv.ref().transactions());

	GWEN_BufferedIO_Close(bin);
	GWEN_BufferedIO_free(bin);
      } // for

      // ok, now we have all standingOrder, create jobs
      n=GWEN_DB_GetFirstGroup(standingOrder);
      while (n) {
        int country;
        string bankCode;
        string accountId;
        string name;
        bool skip;

        skip=false;
        country=GWEN_DB_GetIntValue(n, "ouraccount/country", 0, 280);
        bankCode=GWEN_DB_GetCharValue(n, "ouraccount/bankcode", 0, "");
        accountId=GWEN_DB_GetCharValue(n, "ouraccount/accountid", 0, "");
        name=GWEN_DB_GetCharValue(n, "name", 0, "");

        accs=api->getAccounts(country, bankCode, accountId);
        if (accs.empty()) {
          fprintf(stderr,
                  "%s",
                  I18N("No matching accounts found, skipping\n"));
          AccountJob::storeResult(n, 9999, "no matching accounts found");
          skip=true;
        }
        if (accs.size()>1) {
          fprintf(stderr,
                  "%s",
                  I18N("Ambiguous account specification, skipping\n"));
          AccountJob::storeResult(n, 9999, "ambiguous account specification");
          skip=true;
        }

        if (!skip) {
          acc=accs.front();
          // check account
          if (!_checkAccount(api,
                             GWEN_DB_GetIntValue(n,
                                                 "otheraccount/country", 0,
                                                 280),
                             GWEN_DB_GetCharValue(n,
                                                  "otheraccount/bankcode", 0,
                                                  ""),
                             GWEN_DB_GetCharValue(n,
                                                  "otheraccount/accountid",0,
                                                  ""))){
            if (_args.skipBad) {
              AccountJob::storeResult(n, 9999, "account/bankcode not found");
              skip=true;
            }
          }
        }

        if (!skip) {
          cust=api->findCustomer(acc.ref().country(),
                                 acc.ref().serverBankCode(),
                                 acc.ref().customer());
          if (!cust.isValid()) {
            DBG_ERROR(0, "Customer \"%s\" not found, aborting",
                      acc.ref().customer().c_str());
            return ModuleErrorInconsistentSystem;
          }

          if (name.empty()) {
            // set the owner name if it is not
            GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                                 "name",
                                 acc.ref().userName().c_str());
          }

          try {
	    job=new JobStartStandingOrder(cust, acc);
	    err=job.ref().setStandingOrder(n);
	    if (!err.isOk()) {
	      skip=true;
	    }
	    else {
	      DBG_INFO(0, "Adding job for account %d/%s/%s to queue",
		       acc.ref().country(),
		       acc.ref().bankCode().c_str(),
		       acc.ref().accountId().c_str());
	      jobs.push_back(job);
	      queue.ref().addJob(job.cast<OutboxJob>());
	    }
	  } // try
          catch (Error xerr) {
            DBG_INFO(0, "Error creating job: %s",
                     xerr.errorString().c_str());
            AccountJob::storeResult(n, 9999,
                                    "job probably not supported"
                                    " by your bank");
            skip=true;
          }
        } // if !skip

        if (skip) {
          if (!skippedXA)
            skippedXA=GWEN_DB_Group_new("skipped");
          GWEN_DB_AddGroup(skippedXA, GWEN_DB_Group_dup(n));
        }
        n=GWEN_DB_GetNextGroup(n);
      } // while
    } // if tfiles given

    if (queue.ref().sizeByStatus(HBCI_JOB_STATUS_TODO)) {
      DBG_INFO(0, "Executing queue");
      errorFlag=false;
      err=api->executeQueue(queue);
      if (!err.isOk()) {
        fprintf(stderr, "Error: %s\n", err.errorString().c_str());
        errorFlag=true;
      }
      DBG_INFO(0, "Queue executed.");

      DBG_INFO(0, "Committing results");
      err=api->processOutboxResponse(queue.ref().response());
      if (!err.isOk()) {
        DBG_ERROR(0, "Error: %s", err.errorString().c_str());
        errorFlag=true;
      }
      else {
        DBG_INFO(0, "Ok.");
      };
    } // if queue not empty
    if (jobs.empty()) {
      fprintf(stdout, I18N("No job created.\n"));
    }
    else {
      fprintf(stdout, I18N("Detailed job results:\n"));
  
      // check jobs
      for (jit=jobs.begin(); jit!=jobs.end(); jit++) {
        bool isBad;
  
        isBad=((*jit).ref().result()==HBCI_JOB_RESULT_FAILED ||
               (*jit).ref().status()!=HBCI_JOB_STATUS_DONE);
        fprintf(stdout, "%s",
                (*jit).ref().resultString(!isBad).c_str());
  
        if (!isBad) {
          err=(*jit).ref().commit();
          if (!err.isOk()) {
            DBG_ERROR(0, "Error: %s", err.errorString().c_str());
            errorFlag=true;
            if (!badXA)
              badXA=GWEN_DB_Group_new("bad");
            GWEN_DB_AddGroup(badXA,
			     GWEN_DB_Group_dup((*jit).ref().standingOrder()));
          }
          else {
            if (!goodXA)
              goodXA=GWEN_DB_Group_new("good");
            GWEN_DB_AddGroup(goodXA,
                             GWEN_DB_Group_dup((*jit).ref().standingOrder()));
          }
        }
        else {
          DBG_ERROR(0, "Error in job");
          errorFlag=true;
          if (!badXA)
            badXA=GWEN_DB_Group_new("bad");
          GWEN_DB_AddGroup(badXA,
                           GWEN_DB_Group_dup((*jit).ref().standingOrder()));
        }
      } // for
    }

    // write various files
    if (!_args.badFile.empty()) {
      if (badXA) {
        if (GWEN_DB_WriteFile(badXA,
                              _args.badFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing badfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.badFile.c_str());
      }
    }
    if (!_args.goodFile.empty()) {
      if (goodXA) {
        if (GWEN_DB_WriteFile(goodXA,
                              _args.goodFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing goodfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.goodFile.c_str());
      }
    }
    if (!_args.notDoneFile.empty()) {
      if (skippedXA) {
        if (GWEN_DB_WriteFile(skippedXA,
                              _args.notDoneFile.c_str(),
                              GWEN_DB_FLAGS_DEFAULT)) {
          DBG_WARN(0, "Error writing skippedfile");
          errorFlag=true;
        }
      }
      else {
        unlink(_args.notDoneFile.c_str());
      }
    }

    DBG_INFO(0, "Saving environment");
    err=api->saveEnvironment(openhbciFile, aqmoneyFile);
    if (!err.isOk()) {
      DBG_ERROR(0, "Error saving environment (%s)",
                err.errorString().c_str());
      return ModuleErrorSave;
    }
    DBG_INFO(0, "Saving environment done.");
  } // try
  catch (Error xerr) {
    DBG_ERROR(0, "Error: %s", xerr.errorString().c_str());
    return ModuleErrorExecution;
  }

  if (errorFlag)
    return ModuleErrorExecution;
  return ModuleErrorSuccess;
}









