/***************************************************************************
  $RCSfile: texport.cpp,v $
  -------------------
  cvs         : $Id: texport.cpp,v 1.17 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif


#include "texport.h"

#include <gwenhywfar/debug.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>


ModuleExport::ModuleExport()
:AqmoneyModule() {
}



ModuleExport::~ModuleExport(){
}



void ModuleExport::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Exports transactions and other usefull data to a\n"
               "   file using a specified format\n"));
}


void ModuleExport::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 texport\n"
                  "%s   [-a %s] [-b %s] [-c %s]\n"
                  "%s   [-f %s] [-t %s] [--pos|--neg]\n"
                  "%s   [-oc %s] [-oa %s] [-ob %s]\n"
                  "%s   [-et transaction|transfer|dated|noted|sto]\n"
                  "%s   -o %s [-ot db|dtaus|csv] [-tp %s]\n",
                  I18N("Usage:"),
                  I18N("      "),
                  I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("COUNTRYCODE"),
                  I18N("      "), I18N("YYYYMMDD"), I18N("YYYYMMDD"),
                  I18N("      "),
                  I18N("ACCOUNTID"), I18N("BANKCODE"), I18N("COUNTRYCODE"),
                  I18N("      "),
                  I18N("      "), I18N("FILE"), I18N("PARAM"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleExport::_checkArgs(s_args &args, int argc, char **argv){
  int i;
  string tmp;

  i=0;
  args.country=0;
  args.bankCode="*";
  args.accountId="*";
  args.userId="*";
  args.otherCountry=0;
  args.otherBankCode="*";
  args.otherAccount="*";
  args.outtype="db";
  args.flags=AQM_TRANSACTION_MATCH_POSITIVE | AQM_TRANSACTION_MATCH_NEGATIVE;
  args.exportType=ExportTypeTransactions;

  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-c") {
      i++;
      if (i>=argc)
	return -1;
      args.country=atoi(argv[i]);
    }
    else if (tmp=="-b") {
      i++;
      if (i>=argc)
        return -1;
      args.bankCode=argv[i];
    }
    else if (tmp=="-a") {
      i++;
      if (i>=argc)
        return -1;
      args.accountId=argv[i];
    }
    else if (tmp=="-oa") {
      i++;
      if (i>=argc)
        return -1;
      args.otherAccount=argv[i];
    }
    else if (tmp=="-ob") {
      i++;
      if (i>=argc)
        return -1;
      args.otherBankCode=argv[i];
    }
    else if (tmp=="-oc") {
      i++;
      if (i>=argc)
        return -1;
      args.otherCountry=atoi(argv[i]);
    }
    else if (tmp=="-f") {
      i++;
      if (i>=argc)
        return -1;
      args.fromDate=Date(argv[i]);
      if (!args.fromDate.isValid()) {
        fprintf(stderr,
                I18N("Bad format of fromDate (must be YYYYMMDD)\n"));
        return -1;
      }
    }
    else if (tmp=="-t") {
      i++;
      if (i>=argc)
        return -1;
      args.toDate=Date(argv[i]);
      if (!args.toDate.isValid()) {
        fprintf(stderr,
                I18N("Bad format of toDate (must be YYYYMMDD)\n"));
        return -1;
      }
    }
    else if (tmp=="-et") {
      i++;
      if (i>=argc)
        return -1;
      if (strcasecmp(argv[i], "transaction")==0)
        args.exportType=ExportTypeTransactions;
      else if (strcasecmp(argv[i], "transfer")==0)
        args.exportType=ExportTypeTransfers;
      else if (strcasecmp(argv[i], "dated")==0)
        args.exportType=ExportTypeDatedTransfers;
      else if (strcasecmp(argv[i], "noted")==0)
        args.exportType=ExportTypeNotedTransfers;
      else if (strcasecmp(argv[i], "sto")==0)
        args.exportType=ExportTypeStandingOrders;
      else {
        fprintf(stderr,
                I18N("Unknown export type \"%s\"\n"),
                argv[i]);
        return -1;
      }
    }
    else if (tmp=="--neg") {
      args.flags&=~AQM_TRANSACTION_MATCH_POSITIVE;
    }
    else if (tmp=="--pos") {
      args.flags&=~AQM_TRANSACTION_MATCH_NEGATIVE;
    }
    else if (tmp=="-o") {
      i++;
      if (i>=argc)
        return -1;
      args.outfile=argv[i];
    }
    else if (tmp=="-ot") {
      i++;
      if (i>=argc)
        return -1;
      args.outtype=argv[i];
    }
    else if (tmp=="-op") {
      i++;
      if (i>=argc)
        return -1;
      args.outparam=argv[i];
    }
    else if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      args.params.push_back(argv[i]);
    }
    i++;
  } // while
  // that's it

  // check required arguments
  if (args.outfile.empty()) {
    fprintf(stderr, I18N("No output file given.\n"));
    return -1;
  }

  return 0;
}




ModuleError ModuleExport::_exportTransactions(AqMoneyAPI *api,
                                              ModuleExport::EXPORT_TYPE et) {
  list<Pointer<Account> > accs;
  list<Pointer<Account> >::iterator ait;
  GWEN_DB_NODE *allxas;
  GWEN_BUFFEREDIO *bout;
  int outfd;
  GWEN_ERRORCODE gerr;
  Pointer<AQMConverter> outconv;
  Error err;

  accs=api->getAccounts(_args.country,
                        _args.bankCode,
                        _args.accountId);
  if (accs.empty()) {
    fprintf(stderr,
            "%s",
            I18N("No matching accounts found.\n"));
    return ModuleErrorInvalidArgs;
  }

  allxas=GWEN_DB_Group_new("transactions");

  /* sample all transactions */
  for (ait=accs.begin(); ait!=accs.end(); ait++) {
    GWEN_DB_NODE *xas;
    GWEN_DB_NODE *n;
    Pointer<TransactionStorage> st;

    DBG_NOTICE(0, "Checking account %d/%s/%s",
               (*ait).ref().country(),
               (*ait).ref().bankCode().c_str(),
               (*ait).ref().accountId().c_str());
    switch(et) {
    case ExportTypeTransfers:
      st=(*ait).ref().transferStorage();
      xas=st.ref().getTransactions(_args.otherCountry,
                                   _args.otherBankCode,
                                   _args.otherAccount,
                                   _args.fromDate,
                                   _args.toDate,
                                   _args.flags);
      break;
    case ExportTypeStandingOrders:
      xas=(*ait).ref().getStandingOrders(_args.otherCountry,
                                         _args.otherBankCode,
                                         _args.otherAccount);
      break;
    case ExportTypeDatedTransfers:
      xas=(*ait).ref().getDatedTransfers(_args.otherCountry,
                                         _args.otherBankCode,
                                         _args.otherAccount);
      break;
    case ExportTypeNotedTransfers:
      xas=(*ait).ref().getNotedTransfers(_args.otherCountry,
                                         _args.otherBankCode,
                                         _args.otherAccount);
      break;
    default:
      st=(*ait).ref().storage();
      xas=st.ref().getTransactions(_args.otherCountry,
                                   _args.otherBankCode,
                                   _args.otherAccount,
                                   _args.fromDate,
                                   _args.toDate,
                                   _args.flags);
      break;
    } // switch

    if (xas) {
      // insert our account
      n=GWEN_DB_GetFirstGroup(xas);
      while(n) {
        GWEN_DB_DeleteGroup(n, "ourAccount");
        GWEN_DB_SetIntValue(n, GWEN_DB_FLAGS_DEFAULT,
                            "ourAccount/country",
                            (*ait).ref().country());
        GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                             "ourAccount/bankCode",
                             (*ait).ref().bankCode().c_str());
        GWEN_DB_SetCharValue(n, GWEN_DB_FLAGS_DEFAULT,
                             "ourAccount/accountId",
                             (*ait).ref().accountId().c_str());
        n=GWEN_DB_GetNextGroup(n);
      } /* while */
      GWEN_DB_AddGroupChildren(allxas, xas);
      GWEN_DB_Group_free(xas);
    }
  } // for accounts

  // create output file
  outfd=open(_args.outfile.c_str(),
             O_RDWR | O_CREAT | O_EXCL,
             S_IRUSR | S_IWUSR);
  if (outfd==-1) {
    fprintf(stderr, I18N("open(%s): %s\n"), _args.outfile.c_str(), strerror(errno));
    GWEN_DB_Group_free(allxas);
    return ModuleErrorWritingfile;
  }
  bout=GWEN_BufferedIO_File_new(outfd);
  GWEN_BufferedIO_SetWriteBuffer(bout, 0, 1024);

  // create output converter
  outconv=api->converterFactory(_args.outtype, _args.outparam);
  if (!outconv.isValid()) {
    fprintf(stderr, I18N("Could not create converter \"%s\"\n"),
            _args.outtype.c_str());
    GWEN_DB_Group_free(allxas);
    GWEN_BufferedIO_Abandon(bout);
    GWEN_BufferedIO_free(bout);
    return ModuleErrorInvalidArgs;
  }

  outconv.ref().setTransactions(allxas, true);

  err=outconv.ref().writeToStream(bout);
  if (!err.isOk()) {
    fprintf(stderr, I18N("Error writing to file \"%s\": %s\n"),
            _args.outfile.c_str(), err.errorString().c_str());
    GWEN_BufferedIO_Abandon(bout);
    GWEN_BufferedIO_free(bout);
    return ModuleErrorWritingfile;
  }
  gerr=GWEN_BufferedIO_Close(bout);
  if (!GWEN_Error_IsOk(gerr)) {
    fprintf(stderr, I18N("Error closing outfile.\n"));
    GWEN_BufferedIO_free(bout);
    return ModuleErrorWritingfile;
  }
  GWEN_BufferedIO_free(bout);

  return ModuleErrorSuccess;
}




ModuleError ModuleExport::run(AqMoneyAPI *api,
                              int argc, char **argv,
                              const string &openhbciFile,
                              const string &aqmoneyFile){
  int rv;
  Error err;

  rv=_checkArgs(_args,argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1) {
    _usage();
    return ModuleErrorArguments;
  }

  err=api->loadEnvironment(openhbciFile, aqmoneyFile);
  if (!err.isOk()) {
    DBG_ERROR(0, "Error loading environment (%s)",
              err.errorString().c_str());
    return ModuleErrorLoad;
  }

  return _exportTransactions(api, _args.exportType);
}









