/***************************************************************************
  $RCSfile: version.cpp,v $
  -------------------
  cvs         : $Id: version.cpp,v 1.12 2004/02/29 21:30:36 rbla Exp $
  begin       : Wed Oct 15 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de

 ***************************************************************************
 *    Please see the file COPYING in top source dir for licence details    *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

/* Internationalization */
#ifdef ENABLE_NLS
# include <libintl.h>
# include <locale.h>
# define I18N(m) gettext(m)
#else
# define I18N(m) m
#endif


#include "version.h"

#include <gwenhywfar/version.h>
#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/debug.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>


ModuleVersion::ModuleVersion()
:AqmoneyModule() {
}



ModuleVersion::~ModuleVersion(){
}



void ModuleVersion::dumpDescription(FILE *f){
  fprintf(f, "%s",
          I18N("Shows information about the version of used libs\n"));
}


void ModuleVersion::_usage(){
  fprintf(stderr, "\n");
  fprintf(stderr, "%s aqmoney2 version\n",
                  I18N("Usage:"));
  fprintf(stderr, "\n");
  dumpDescription(stderr);
  fprintf(stderr, "\n%s\n\n", I18N("See aqmoney2(1) for detailed help."));
}



int ModuleVersion::_checkArgs(int argc, char **argv){
  int i;
  string tmp;

  i=0;
  while (i<argc){
    tmp=argv[i];
    if (tmp.empty()) {
      fprintf(stderr, I18N("Error: Empty command specific argument\n"));
      return -1;
    }
    if (tmp=="-h" || tmp=="--help") {
      _usage();
      return -2;
    }
    else {
      if (tmp[0]=='-') {
        fprintf(stderr, I18N("Option \"%s\" is unknown to this command\n"),
                tmp.c_str());
        return -1;
      }
      fprintf(stderr, I18N("Argument \"%s\" is not an option to this command\n"),
              tmp.c_str());
      return -1;
    }
    i++;
  } // while
  // that's it
  return 0;
}



ModuleError ModuleVersion::run(AqMoneyAPI *api,
                               int argc, char **argv,
                               const string &openhbciFile,
                               const string &aqmoneyFile){
  int rv;
  int major, minor, patchlevel, build;

  rv=_checkArgs(argc,argv);
  if (rv==-2)
    return ModuleErrorSuccess;
  else if (rv==-1)
    return ModuleErrorArguments;

  fprintf(stdout, I18N("Versions:\n"));

  fprintf(stdout, " AqMoney   : %s\n",
          AQMONEY_VERSION_FULL_STRING);
  Hbci::libraryVersion(major, minor, patchlevel, build);
  fprintf(stdout,I18N(
          " OpenHBCI\n"
          "   Headers : %s\n"
          "   Library : %d.%d.%d.%d\n"),
          OPENHBCI_VERSION_FULL_STRING,
          major, minor, patchlevel, build);
  GWEN_Version(&major, &minor, &patchlevel, &build);
  fprintf(stdout,I18N(
          " Gwenhywfar\n"
          "   Headers : %s\n"
          "   Library : %d.%d.%d.%d\n"),
          GWENHYWFAR_VERSION_FULL_STRING,
          major, minor, patchlevel, build);
  return ModuleErrorSuccess;
}









